%{ 
 ****************************************************************************
 * CLASSIFICATION: Unclassified Unlimited Release
 *
 * SAND2020-0526 O
 *
 ****************************************************************************
%}

classdef gff < handle
% GFF A full featured GFF 1.x/2.x reader/writer and conversion suite
%
% GFF is a class based file parsing suite for GFF files. It can read in
% both 1.x (refered to here as 'legacy') and GFF 2.x ('modern') GFF files.
%
% Both legacy fields AND fields are available for both legacy AND modern
% files if conversion is possible.
%
% Any enum fields are optionally (and by default) converted from integers
% to their human friendly string names.
%
% If desired, it is possible to use this parsing suite entirely with a
% functional interface. The MATLAB class based interface, while powerful,
% is not forced on the end user.
%
% I have made every attempt for this class to do the right thing by
% default but this suite can be configured as desired.
%
% Features:
%   - reading then writing a file will yield a binary identical file*
%   - access to reconstructed images and raw binary image data is provided
%   - it is possible to write out a 1.x/2.x file from either a 1.x/2.x file
%
%  *with the one caviet is that some known GFF bugs are fixed on writing
%
% Functional Interface:
%     GFF.READ_GFF
%     GFF.WRITE_GFF
%     GFF.WRITE_LEGACY_GFF
%     GFF.READ_LEGACY_GFF
%
% Author: Rick Naething
%
% Do not modify these lines:
% <%comment%>
%10-Jul-2014 02:04:03
%</%comment%>
%    
    
    % These properties are created on the fly when accessed
    % This allows the class to provide things like a complex image while
    % still storing the raw image data without worrying about changes to
    % the file being lost.
    properties (Dependent = true)
        cimg;
        header;
        legacy_header;
        ext;
        options;
    end
    
    % These properties may be useful, so they are kept public
    properties        
        filename;
        image_raw;
        raw_data;
    end
   
    % Properties that end users should not be changing:
    properties(Access = private)
        ver_min;
        ver_maj;
        endian;
        fid;
        valid;
        
        legacy;
        
        cache;
        modified;
        
        % this is to keep track of ext that we read in that are unknown,
        % either completly or this revision
        unknown_ext;
        
        % the internal versions (used by the dependent 'on the fly'
        % versions under the hood)
        legacy_header_;
        header_;
        options_;               
        ext_;
        
        image_type;
        
        first;
        second;
    end
    
    methods
        
        %------------------------------------------------------------------
        % constructor
        %     we don't *have* to open a file when we first create the class
        %------------------------------------------------------------------
        function obj = gff(filename)
            obj.set_default_options();
            obj.reset();
            if nargin > 0;
                obj.filename = filename;
                obj.open()
            end
        end
        
        function set_default_options(obj)
           obj.options_.native_types = 0; % return raw data types rather than
                                         % doubles
           obj.options_.load_image = 1;
           obj.options_.validate = 1;
           obj.options_.verbose = 0;
           obj.options_.expand_enum = 1; % return string for enum values (default)
        end
        
        function reset(obj)
            obj.legacy = 0;
            obj.valid = 0;
            obj.modified = 0;
            obj.ver_min = -1;
            obj.ver_maj = -1;
            obj.cache.cimg = [];
            obj.clear_header_cache();
        end
        
        function clear_header_cache(obj)
            obj.cache.header = [];
            obj.cache.legacy_header = [];
            obj.cache.ext = [];
        end
        
        %------------------------------------------------------------------
        % Read in a 1.x or 2.x gff file
        %------------------------------------------------------------------
        function open(obj, filename)
            obj.reset();
            obj.valid = 1;            
            
            if exist('filename', 'var')
               obj.filename = filename; 
            end
            obj.fid = fopen(obj.filename, 'r', 'ieee-le', 'US-ASCII');
            obj.gff_version();
            
            %-------------------------------------------------------------%
            % Read in the raw header / extensions
            if(obj.ver_maj<2)
                obj.read_legacy_header();

                % found some strange GFF 1.8 files that really look to be GFF 1.6 files
                % but not quite...
                 if(obj.legacy_header_.Length == 1024)
                     error('GFF 1.8 files with Header Length == 1024 not supported');
                 end
            else
                % read in the header (GSATIMG)
                block       = read_block_header(obj.fid);
                obj.header_ = obj.read_block(obj.fid, block);
                
                % read in the extensions
                while(1)
                    block = read_block_header(obj.fid);
                    block = repair_block_header(block);
                    if(strcmp(block.name,'IMAGEDATA')), break; end
                    obj.ext_.(block.name) = obj.read_block(obj.fid, block);
                end
            end

            % store the header enums as strings internally
            % to make parsing easier
            obj.header_ = obj.header_enum_to_string(obj.header_, 1);            
            
            obj.validate_header();
            
            if obj.options_.load_image
                obj.read_image();
            end

            fclose(obj.fid);
        end
        
        %------------------------------------------------------------------
        % Write out a gff
        %------------------------------------------------------------------
        function write(obj,filename)
        % GFF.WRITE
            out_fid = fopen(filename,'w','native','US-ASCII');

            raw = obj.pack_image();
            header = obj.header_;
            header.imageLengthBytes = length(raw);

            header = obj.header_enum_to_string(header, 0);
            obj.write_block(header, 'GSATIMG', out_fid);

            for f=fieldnames(obj.ext_).'
                s=char(f);
                obj.write_block(obj.ext_.(s), s, out_fid);
            end

            write_block_header(out_fid, 'IMAGEDATA', header, length(raw));
            fwrite(out_fid, raw, '*uint8');
            fclose(out_fid);
        end
        
        %------------------------------------------------------------------
        % Write out a legacy gff
        %------------------------------------------------------------------        
        function write_legacy(obj, filename)
            out_fid = fopen(filename,'w');
            
            obj.fix_legacy_image_data(); % modern -> legacy
            raw = obj.pack_image();
            obj.fix_legacy_image_data(); % legacy -> modern (reset)
            
            if(~obj.legacy)
                obj.convert();
            end
            header = obj.legacy_header_;
            header.ImageLength = length(raw);
            
            buffer = zeros(1,8,'uint8');
            buffer(1:7)   = uint8('GSATIMG');
            fwrite(out_fid, buffer, 'uint8');
            
            block_tag =  sprintf('GSATIMG_%d_%d', header.ver_maj, header.ver_min);            
            gff_extension_library(block_tag, 'write', out_fid, header);
            
            fwrite(out_fid, raw, '*uint8');
            fclose(out_fid);
        end        
        
        %------------------------------------------------------------------
        % Read in a legacy header
        %------------------------------------------------------------------
        function read_legacy_header(obj)
            obj.legacy = 1;
            fseek(obj.fid, 8, 'bof');
            block.ver_maj = 1;
            block.ver_min = 8;
            block.name = 'GSATIMG';
            obj.legacy_header_ = obj.read_block(obj.fid, block);
            obj.convert();
        end
        
        %------------------------------------------------------------------
        % gff_version
        %------------------------------------------------------------------
        % this works with both v1.x and v2.x gff files
        % it resets the fid to the beginning of the file when done
        function gff_version(obj)

            buffer = fread(obj.fid, 20, '*uint8');
            
            t = buffer(8);
            obj.ver_min = typecast(buffer(9:10),'uint16');
            
            % we have a 1.x file
            if (t==0 && obj.ver_min ~= 0)
                obj.endian_check();
                obj.ver_min = typecast(buffer(9:10),  'uint16');
                obj.ver_maj = typecast(buffer(11:12), 'uint16');
                
            % we have a 2.x file
            else
                obj.ver_maj = typecast(buffer(17:18), 'uint16');
                obj.ver_min = typecast(buffer(19:20), 'uint16');
            end

            fseek(obj.fid, 0, 'bof');
        end
        
        %-------------------------------------------------------------------------%
        % determain endian for 1.x files
        %-------------------------------------------------------------------------%
        function endian_check(obj)
            fseek(obj.fid, 54, 'bof');
            %if ( typecast(obj.mmap.Data(54:56), 'uint16'))
            if(fread(obj.fid, 1, '*uint16'))
                error('Can''t yet handle BigEndian with memmapfile');
            end
        end

        %-------------------------------------------------------------------------%
        % Read a block using synthesized code
        function out = read_block(obj, fid, block)

            % First, handle the GSATIMG Cases
            if(strcmp(block.name, 'GSATIMG') && block.ver_maj==2)
                buffer = fread(fid, block_size('GSATIMG_2_5'), '*uint8');
                out = read_GSATIMG_2_5(buffer);
                out.ver_min = block.ver_min;
                return
            end

            % all other blocks
             block_tag = sprintf('%s_%d_%d',strtrim(block.name),block.ver_maj,block.ver_min);
             read_fun = ['read_' block_tag];
             if(~isfield(block,'bytes') || block.bytes == -1)
                 block.bytes = block_size(block_tag); 
             end
             buffer = fread(fid, block.bytes, '*uint8');

             % now, handle the normal blocks
             if exist(read_fun)  %#ok<EXIST> There is no 2 input version for non built-ins!!!
                 read_fun = str2func(read_fun);
                 out = read_fun(buffer);
                 return;
             end

             % --------------------- % 
             % At this point it is some type of unknown block
             obj.unknown_ext.(block.name).bytes = block.bytes;

             % now, handle blocks with min_ver > than we know about
             v_min = double(block.ver_min)-1;
             while v_min >= 0
                 block_tag2 = sprintf('%s_%d_%d',strtrim(block.name),block.ver_maj,v_min);
                 read_fun = sprintf('read_%s',block_tag2);
                 if ~exist(read_fun) %#ok<EXIST> There is no 2 input version for non built-ins!!!
                     v_min = v_min - 1;
                     continue;
                 end
                 fprintf(2, 'Using %s in order to read %s\n', block_tag2, block_tag);
                 read_fun = str2func(read_fun);
                 out = read_fun(buffer);
                 read = block_size(block_tag2);
                 skipped = block.bytes - read;
                 fprintf(2, '\t%d bytes skipped\n', skipped);
                 out.raw = buffer(end-skipped+1:end);
                 out.ver_min = block.ver_min;

                 % set up compatible functions for marshaling / unmarshaling
                 obj.unknown_ext.(block.name).marshal   = str2func(['marshal_'   block_tag2]);
                 obj.unknown_ext.(block.name).unmarshal = str2func(['unmarshal_' block_tag2]);
                 obj.unknown_ext.(block.name).write     = str2func(['write_'     block_tag2]);
                 return;
             end

             % finally, handle completly unknown blocks
             fprintf(2,'Unknown block %s: size %d\n', block_tag, block.bytes);
             out = block;
             out.raw = buffer;
             obj.unknown_ext.(block.name).marshal   = [];
             obj.unknown_ext.(block.name).unmarshal = [];
             obj.unknown_ext.(block.name).write     = [];
        end

        %-------------------------------------------------------------------------%
        % Write a block using synthesized code
        function write_block(obj, block, title, gff_fid)
            block.name = title;
            block_tag =  sprintf('%s_%d_%d',strtrim(block.name), block.ver_maj, block.ver_min);

            % --------------------- % 
            % first handle blocks we don't know about
            if isfield(obj.unknown_ext, block.name)
                write_block_header(gff_fid, title, block, obj.unknown_ext.(block.name).bytes);
                if ~isempty(obj.unknown_ext.(block.name).write)
                    obj.unknown_ext.(block.name).write(gff_fid, block);
                end
                fwrite(gff_fid, block.raw, 'uint8');
                return;
            end

            % --------------------- % 
            % now the rest of the blocks
            bytes = block_size(block_tag);
            if(isempty(bytes))
                bytes=calculate_size_from_data(block);
            end
            
            write_block_header(gff_fid, title, block, bytes);
            
            % write out GSATIMG 2.x blocks
            if(strcmp(title, 'GSATIMG') && block.ver_maj==2)
                write_GSATIMG_2_5(gff_fid, block);
                return;
            end
            
            % other blocks we know about
            write_fun = ['write_' block_tag];
            if(exist(write_fun))
                write_fun = str2func(write_fun);
                write_fun(gff_fid, block);
                return;
            end  
        
        end        
        
        %------------------------------------------------------------------
        % validate header
        function validate_header(obj)
            
            if(strcmp(obj.header_.imageCompressionScheme,'none'))
                % Check imageLengthBytes
                bytes = sum([obj.header_.pixelFormat.compFormat.bitSize])/8;
                total_bytes = bytes * obj.header_.rangePixels * obj.header_.azPixels;
                if(obj.header_.imageLengthBytes ~= total_bytes)
                    fprintf(2, 'WARNING: imageLengthBytes incorrect -> recalculating\n');
                    fprintf(2, '\t%d bytes (header.imageLengthBytes)\n', obj.header_.imageLengthBytes);
                    fprintf(2, '\t%d bytes (rangePixels * azPixels * length(compFormat))\n', total_bytes);
                    obj.header_.imageLengthBytes = total_bytes;
                    if ~isempty(obj.legacy_header_)
                        obj.legacy_header_.ImageLength = total_bytes;
                    end
                end
            end
            
        end
        
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        % Read / Write Image Data
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%

        %---------------------------------------------------------------------%
        function read_image(obj)
            
            % Read in raw data
            obj.raw_data = fread(obj.fid,  obj.header_.imageLengthBytes, '*uint8', 0);
            
            comp_type = obj.header_.imageCompressionScheme;
            
            % Parse raw data
            switch comp_type
                case 'none'
                    obj.format_raw_data_read(obj.raw_data);
                    
                case 'zlib'
                    raw = obj.zlib('uncompress', obj.raw_data);
                    obj.format_raw_data_read(raw);
                    
                case {'jpeg','jpeg2000'}
                    obj.image_raw = read_image_format_data(obj.raw_data, comp_type);
            end
            
            % patches for legacy images
            if(obj.legacy)
                obj.fix_legacy_image_data();
            end

        end
        
        %------------------------------------------------------------------
        % Pack image data into raw bytes ready to be written to disk
        function raw = pack_image(obj)
                   
            if ~obj.modified
               raw = obj.raw_data; 
               return;
            end

            comp_type = obj.header_.imageCompressionScheme;
           
            % Format raw data
            switch comp_type
                case 'none'
                    raw = obj.format_raw_data_write();
                    
                case 'zlib'
                    raw = obj.format_raw_data_write();
                    raw = obj.zlib('compress', raw);
                    
                case {'jpeg','jpeg2000'}
                    raw = pack_image_format_data(obj.image_raw.mag, comp_type);

            end
    
        end        
        
        %---------------------------------------------------------------------%
        % this is a bit ugly... but some patches are needed for certain
        % types of legacy images.
        % this function needs to be invertible, that is, running it once
        % goes from legacy -> modern, and again will go back from
        % modern -> legacy (see usage in write_legacy() to see why)
        function fix_legacy_image_data(obj)
            if(obj.legacy)
                switch obj.legacy_header_.ImageType
                    % legacy always has 'PM', but we only support 'MP' in
                    % modern...
                    case 1 % phase, mag -> mag, phase
                        temp = obj.image_raw.phase;
                        obj.image_raw.phase = obj.image_raw.mag;
                        obj.image_raw.mag   = temp;
                    case 2
                end
            end
        end
        
        %---------------------------------------------------------------------%
        % Convert from a GFF enum to matlab internal data type
        function type = raw_data_type(obj)
           switch obj.header_.pixDataType
               case {'MAG_UCHAR','MAG_PHASE_UCHAR'}
                   type = 'uint8';
               case {'MAG_PHASE_USHORT','COMPLEX_USHORT'}
                   type = 'uint16';
               case {'COMPLEX_UINT','COMPLEX_ULONG'}
                   type = 'uint32';
               case 'MAG_CHAR';
                   type = 'int8';
               case {'MAG_PHASE_SHORT','COMPLEX_SHORT'}
                   type = 'int16';
               case {'COMPLEX_INT','COMPLEX_LONG'}
                   type = 'int32';
               case 'COMPLEX_SINGLE';
                   type = 'single';
               case 'COMPLEX_DOUBLE';
                   type = 'double';
               case 'UNDEFINED_PIXEL_TYPE';
                   warning(1, 'Undefined pixel type');
                   type = 'uint8';
           end

        end      
        
        %---------------------------------------------------------------------%
        function format_raw_data_read(obj, raw)
                        
            raw = typecast(raw, obj.raw_data_type());
            len = obj.header_.rangePixels * obj.header_.azPixels;

            sz = [obj.header_.rangePixels, obj.header_.azPixels];
            if strcmp('AZ_CONSECUTIVE', obj.header_.pixOrder)
                sz = fliplr(sz);
            end
            
            obj.complex_domain_dispatch(@interleaved, @sequential, @singular);

            if strcmp('AZ_CONSECUTIVE', obj.header_.pixOrder)
                obj.image_raw = obj.transpose_struct(obj.image_raw);
            end
            
            % helper functions:
            function interleaved(one, two)
                obj.image_raw.(one) = reshape(raw(1:2:end),sz);
                obj.image_raw.(two) = reshape(raw(2:2:end),sz);
            end
            
            function sequential(one, two)
                obj.image_raw.(one) = reshape(raw(    1:len), sz);
                obj.image_raw.(two) = reshape(raw(len+1:end), sz);
            end
            
            function singular(one)
               obj.image_raw.(one) = reshape(raw, sz);
            end
            
        end

        %---------------------------------------------------------------------%
        function raw = format_raw_data_write(obj)
            
            image = obj.image_raw;
            if strcmp('AZ_CONSECUTIVE', obj.header_.pixOrder)
                image = obj.transpose_struct(image);
            end
            
            len = obj.header_.rangePixels * obj.header_.azPixels;
            len = len * sum([obj.header_.pixelFormat.compFormat.bitSize]~=0);
            raw = zeros([len 1], obj.raw_data_type);
            
            obj.complex_domain_dispatch(@interleaved, @sequential, @singular);
            
            raw = typecast(raw, 'uint8');
            
            % helper functions:            
            function interleaved(one, two)
                raw(1:2:end) = image.(one)(:);
                raw(2:2:end) = image.(two)(:);
            end
            
            function sequential(one, two)
                raw(:) = [image.(one)(:); image.(two)(:)];
            end
            
            function singular(one)
                raw(:) = image.(one)(:);
            end
        end        

        %-------------------------------------------------------------------------%
        % This is called by both read and pack functions to dispatch the 
        % appropriate helper function for the appropriate image type
        function complex_domain_dispatch(obj, interleaved, sequential, singular)
            switch obj.header_.pixelFormat.cmplxDomain
                case 'IQ'
                    interleaved('I','Q');
                case 'QI'
                    interleaved('Q','I');
                case 'I1Q2'
                    sequential('I','Q');
                case 'Q1I2'
                    sequential('Q','I');
                case 'MP'
                    interleaved('mag','phase');
                case 'M1P2'
                    sequential('mag','phase');
                case 'P1M2'
                    sequential('phase','mag');
                case 'M'
                    singular('mag');
                case 'P'
                    singular('phase');
                otherwise
                    disp('Domain type invalid');
            end
        end   
        
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        % Utility Routines
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%        

        %-------------------------------------------------------------------------%
        function rcs = rcs(obj)
            rcs = 20*log10(obj.ext_.IFINFO.imgCalParam * obj.mag());
        end        
        
        %-------------------------------------------------------------------------%
        function mag = mag(obj)
            mag = [];
            if ~obj.valid, return; end
            
            switch obj.header_.pixDataType
                case {'MAG_UCHAR','MAG_PHASE_UCHAR'}
                    mag = double(obj.image_raw.mag).^2;
                case {'MAG_PHASE_USHORT','COMPLEX_USHORT'}
                    mag = double(obj.image_raw.mag);
                case {'MAG_PHASE_SHORT'}
                    mag = obj.image_raw.mag;
                    inds=find(mag < 0);
                    mag(inds) = mag(inds)+2^16;
                    mag = double(mag);
                otherwise
                    error('pixDataType %s case not yet handled', obj.header_.pixDataType);
            end
        end

        %-------------------------------------------------------------------------%
        function phase = phase(obj)
            phase = [];
            if ~obj.valid, return; end
                       
            switch obj.header_.pixDataType
                case {'MAG_UCHAR','MAG_PHASE_UCHAR'}
                    phase = exp(1i*2*pi/(2^8)*(double(obj.image_raw.phase)-2^7));
                case {'MAG_PHASE_USHORT','COMPLEX_USHORT'}
                    phase = exp(1i*2*pi/(2^16)*(double(obj.image_raw.phase)-2^15));
                case {'MAG_PHASE_SHORT'}
                    phase = exp(1i*2*pi/(2^16)*double(obj.image_raw.phase));
                otherwise
                    error('pixDataType %s case not yet handled', obj.header_.pixDataType);
            end
        end
        
        %-------------------------------------------------------------------------%
        function display_qp(obj, LUT)
            if ~obj.valid, return; end
            
            if ~exist('LUT','var')
                if(strcmp(obj.header_.imageCompressionScheme,'zlib'))
                    LUT = 7;
                else
                    LUT = 3;
                end
            end

            rng = double(obj.header_.rangePixels);
            az  = double(obj.header_.azPixels);
            figure;
            x = linspace(-az/2,az/2,az)   *obj.ext_.GEOINFO.azPixSpacing;            
            y = linspace(-rng/2,rng/2,rng)*obj.ext_.GEOINFO.rangePixSpacing;
            image(x,y,sqrt(obj.mag())*LUT);
            colormap gray;
            axis image;
            xlabel('meters');
            ylabel('meters');
            
        end

        %-------------------------------------------------------------------------%
        function display_rcs(obj)
            if ~obj.valid, return; end
            
            rng = double(obj.header_.rangePixels);
            az  = double(obj.header_.azPixels);
            figure;
            x = linspace(-az/2,az/2,az)    * obj.ext_.GEOINFO.azPixSpacing;            
            y = linspace(-rng/2,rng/2,rng) * obj.ext_.GEOINFO.rangePixSpacing;
            imagesc(x,y,obj.rcs());
            title('Image RCS (dB)');
            colormap gray;
            axis image;
            xlabel('meters');
            ylabel('meters');
            colorbar;
        end         
        
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        % Get / Set Routines
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        function cimg = get.cimg(obj)
            cimg = [];
            if ~obj.valid, return; end

            if isempty(obj.cache.cimg)
                switch obj.header_.pixelFormat.cmplxDomain
                    case {'MP','M1P2','P1M2'}
                        cimg = obj.mag() .* obj.phase();
                    case 'M'
                        cimg = obj.mag();
                    case 'P'
                        cimg = obj.phase();
                    case 'IQ'
                        cimg = complex(obj.image_raw.I, obj.image_raw.Q);
                    otherwise
                       error('cmplxDomain case %s not yet handled', cmplxDomain); 
                end
                obj.cache.cimg = cimg;
            end
            cimg = obj.cache.cimg;
            
        end
        
        %-------------------------------------------------------------------------%
        % Call if we have reassigned cimg
        function update_header_info(obj)
            
            rng = size(obj.cimg,1);
            az  = size(obj.cimg,2);
            
            obj.header.rangePixels = rng;
            obj.header.azPixels    = az;
            
            if (isfield(obj.ext_, 'COMPRESSINFO') && isfield(obj.ext_.COMPRESSINFO, 'uncompressedSize'))
                bytes_per_pix = sum([obj.header_.pixelFormat.compFormat.bitSize])/8;
                obj.ext_.COMPRESSINFO.uncompressedSize = az * rng * bytes_per_pix;
            end
            
            if (isfield(obj.ext_, 'GEOINFO') && isfield(obj.ext_.GEOINFO,'pixLocImCtrCol'))
                obj.ext_.GEOINFO.pixLocImCtrRow = floor(rng/2);
                obj.ext_.GEOINFO.pixLocImCtrCol = floor(az/2);
            end
            
            if (isfield(obj.ext_, 'IFINFO') && isfield(obj.ext_.IFINFO,'sampLocDCRow'))
                obj.ext_.IFINFO.sampLocDCRow = floor(rng/2);
                obj.ext_.IFINFO.sampLocDCCol = floor(az/2);
            end
            
        end
        
        %-------------------------------------------------------------------------%
        function set.cimg(obj, cimg)
            obj.valid  = 1;
            obj.modified = 1;
                        
            %obj.header.imageLen
            obj.cache.cimg = cimg;
            
            obj.update_header_info();
            
            switch obj.header_.pixelFormat.cmplxDomain
                case {'MP','M1P2','P1M2'}
                    obj.image_raw.mag = get_mag_from_cimg();
                    obj.image_raw.phase = get_phase_from_cimg();
                case 'M'
                    obj.image_raw.mag = get_mag_from_cimg();
                case 'P'
                    obj.image_raw.phase = get_phase_from_cimg();
                case 'IQ'
                    obj.image_raw.I = real(cimg);
                    obj.image_raw.Q = imag(cimg);
                otherwise
                    error('cmplxDomain case %s not yet handled', obj.header_.pixelFormat.cmplxDomain);
            end

            function mag = get_mag_from_cimg()
                switch obj.header_.pixDataType
                    case {'MAG_UCHAR','MAG_PHASE_UCHAR'}
                        mag = sqrt(abs(cimg));
                    case {'MAG_PHASE_USHORT','COMPLEX_USHORT'}
                        mag = abs(cimg);
                    otherwise
                        error('pixDataType %s case not yet handled', obj.header_.pixDataType);
                end
                mag = cast(mag, obj.raw_data_type);
            end
            
            function phase = get_phase_from_cimg()
                switch obj.header_.pixDataType
                    case {'MAG_UCHAR','MAG_PHASE_UCHAR'}
                        phase = (angle(cimg)*2^8/(2*pi))+2^7;
                    case {'MAG_PHASE_USHORT','COMPLEX_USHORT'}
                        phase = (angle(cimg)*2^16/(2*pi))+2^15;
                    otherwise
                        error('pixDataType %s case not yet handled', obj.header_.pixDataType);
                end
                phase = cast(phase, obj.raw_data_type);
            end
            
        end
        
        %-------------------------------------------------------------------------%
        function header = get.legacy_header(obj)
            header = [];
            if ~obj.valid, return; end           
            
            if isempty(obj.cache.legacy_header)
                if obj.ver_maj > 1
                    obj.convert();
                end
                obj.cache.legacy_header = obj.legacy_header_raw_to_human(obj.legacy_header_);
     
                if(~obj.options_.native_types)
                    obj.cache.legacy_header = obj.to_double(obj.cache.legacy_header);
                end
            end
            header = obj.cache.legacy_header;
        end
        
        %-------------------------------------------------------------------------%        
        function set.legacy_header(obj, header) %#ok<INUSD>
            error('Setting legacy header not yet supported');
        end
        
        %-------------------------------------------------------------------------%        
        function header = get.header(obj)
            header = [];
            if ~obj.valid, return; end           
            
            if isempty(obj.cache.header)
                 obj.cache.header = unmarshal_GSATIMG_2_5(obj.header_);

                 if(~obj.options_.native_types)
                    obj.cache.header = obj.to_double(obj.cache.header);
                 end
                 
                 if(~obj.options_.expand_enum)
                     obj.cache.header = obj.header_enum_to_string(obj.cache.header, 0);
                 end
            end
            header = obj.cache.header;
        end
        
        %-------------------------------------------------------------------------%
        function set.header(obj, header)
            obj.valid  = 1;
            obj.cache.header = header;
            
            if(~obj.options_.expand_enum)
               header = obj.header_enum_to_string(header, 1); 
            end
            
            obj.header_ = marshal_GSATIMG_2_5(obj.header);
            
            if ((-1 == obj.ver_min) || (-1 == obj.ver_maj))
               obj.ver_min = header.ver_min;
               obj.ver_min = header.ver_maj;               
            end
        end
        
        %-------------------------------------------------------------------------%
        function ext = get.ext(obj)
            ext = [];
            if ~obj.valid, return; end          
            
            if isempty(obj.cache.ext)
                obj.cache.ext = obj.ext_;

                % Marshal the extensions
                for f=fieldnames(obj.cache.ext).'
                    fld = f{1};
                    if isfield(obj.unknown_ext, fld)
                        m_fun = obj.unknown_ext.(fld).unmarshal;
                        if isempty(m_fun)
                            continue;
                        end
                    else
                        block_tag = sprintf('%s_%d_%d', fld, obj.cache.ext.(fld).ver_maj,...
                                                             obj.cache.ext.(fld).ver_min);
                        m_fun = str2func(['unmarshal_' block_tag]);
                    end
                    obj.cache.ext.(fld)=m_fun(obj.cache.ext.(fld));
                end 
                
                % Convert from native types to doubles (if set)
                if(~obj.options_.native_types)
                    obj.cache.ext = obj.to_double(obj.cache.ext);
                end

                % Convert integers to strings
                if(obj.options_.expand_enum)
                    % no-op for now
                end
            end
            ext = obj.cache.ext;
        end
        
        %-------------------------------------------------------------------------%
        function set.ext(obj, ext)
            obj.valid  = 1;
            for f=fieldnames(ext).'
                fld = f{1};
                block_tag = sprintf('%s_%d_%d', fld, ext.(fld).ver_maj, ext.(fld).ver_min);
                m_fun = str2func(['marshal_' block_tag]);
                ext.(fld)=m_fun(ext.(fld));
            end
            obj.cache.ext = ext;
            obj.ext_ = ext;
        end

        %-------------------------------------------------------------------------%
        function options = get.options(obj)
            options = obj.options_;
        end

        %-------------------------------------------------------------------------%
        function set.options(obj, options)
            if ((obj.options_.native_types ~= options.native_types) || ...
                (obj.options_.expand_enum ~= options.expand_enum))
                  obj.clear_header_cache();
            end
            obj.options_ = options;
        end
  
        %-------------------------------------------------------------------------%
        function warning(obj, str, varargin)
           if(obj.options_.verbose)
               fprintf(2, str, varargin{:});
           end
        end
                
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        % Conversion Routines
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%             

        %------------------------------------------------------------------
        % Legacy <-> Modern Header and Extensions
        %------------------------------------------------------------------
        function convert(obj)
           if obj.ver_maj > 1               
               legacy_header = init_block('GSATIMG_1_8');
               legacy_header.Version_Major = 1;
               legacy_header.Version_Minor = 8;
               legacy_header.Length = 2048;
               legacy_header.ver_maj = 1;
               legacy_header.ver_min = 8;

               legacy_header.BytesPerPixel    = sum([obj.header_.pixelFormat.compFormat(:).bitSize])/8;
               
               switch obj.header_.pixelFormat.cmplxDomain
                   case 'M'
                       legacy_header.ImageType = 0;
                   case {'MP','M1P2','P1M2'}
                       legacy_header.ImageType = 1;
                   case {'IQ','QI','I1Q2','Q1I2'}
                       legacy_header.ImageType = 2;
                   case 'P'
                       warning('Converting phase only images to 1.8 not fully supported');
                   otherwise
                       warning('Unknown cmplxDomain');
               end
               
               if (length(obj.header_.pixelFormat.compFormat) == 2)
                   legacy_header.BitsPerMagnitude = obj.header_.pixelFormat.compFormat(1).bitSize;
                   legacy_header.BitsPerPhase     = obj.header_.pixelFormat.compFormat(2).bitSize;
               else
                  if strcmp(obj.header_.pixelFormat.cmplxDomain,'P')
                      legacy_header.BitsPerPhase     = obj.header_.pixelFormat.compFormat(1).bitSize;
                      legacy_header.BitsPerMagnitude = 0;
                  else
                      legacy_header.BitsPerPhase     = 0;
                      legacy_header.BitsPerMagnitude = obj.header_.pixelFormat.compFormat(1).bitSize;
                  end
               end
               
               legacy_header.DateTime(1) = obj.ext_.APINFO.yearMidAp;
               legacy_header.DateTime(2) = obj.ext_.APINFO.monthMidAp;
               legacy_header.DateTime(3) = obj.ext_.APINFO.dayMidAp;
               legacy_header.DateTime(4) = obj.ext_.APINFO.hourMidAp;
               legacy_header.DateTime(5) = obj.ext_.APINFO.minuteMidAp;
               legacy_header.DateTime(6) = obj.ext_.APINFO.secondMidAp;
               
               header = obj.header_enum_to_string(obj.header_, 0);            
               legacy_header = obj.mapped_fields_legacy_to_modern(header, obj.ext_, legacy_header, 0);               
               obj.legacy_header_ = gff.legacy_header_human_to_raw(legacy_header);
           else
               % fill in a default set (should probably do something more
               % sophisticated here)
               header         = init_block('GSATIMG_2_5');
               ext.RADARINFO  = init_block('RADARINFO_2_0');
               ext.GEOINFO    = init_block('GEOINFO_1_1');
               ext.APINFO     = init_block('APINFO_5_0');
               ext.IFINFO     = init_block('IFINFO_3_0');
               ext.MOMEASINFO = init_block('MOMEASINFO_2_0');
               
               legacy_header = gff.legacy_header_raw_to_human(obj.legacy_header_);

               switch legacy_header.ImageType
                   case 0 % Magnitude only
                       header.pixelFormat.cmplxDomain = 'M';
                       header.pixelFormat.numComponents = 1;
                       if (legacy_header.ImageCmpScheme == 1)
                           header.pixOrder = ~header.pixOrder;
                       end
                       header.pixDataType = 'MAG_UCHAR';
                   case 1 % Magnitude & Phase
                       header.pixelFormat.numComponents = 2;
                       switch legacy_header.BytesPerPixel
                           case 2
                               header.pixelFormat.cmplxDomain = 'M1P2';
                               header.pixDataType = 'MAG_PHASE_UCHAR';
                           case 4
                               header.pixelFormat.cmplxDomain = 'MP';                               
                               header.pixDataType = 'MAG_PHASE_SHORT';
                           case 8
                               header.pixDataType = 'MAG_PHASE_SINGLE';
                               error('MAG_PHASE_SINGLE images not supported in GFF 2.x');
                       end
                   case 2 % I & Q
                       header.pixelFormat.cmplxDomain = 'IQ';
                       header.pixelFormat.numComponents = 2;
                       switch legacy_header.BytesPerPixel
                           case 4
                               header.pixDataType = 'COMPLEX_SHORT';
                           case 8
                               header.pixDataType = 'COMPLEX_SINGLE';
                       end
               end
               
               header.pixDataType = ...
                   gff.enum_lookup('pixDataType',header.pixDataType, 0);
               header.pixelFormat.cmplxDomain = ...
                   gff.enum_lookup('cmplxDomain',header.pixelFormat.cmplxDomain, 0);
               
               header.pixelFormat.compFormat(1).bitSize = legacy_header.BitsPerMagnitude;
               if legacy_header.ImageType % if not Mag only
                   header.pixelFormat.compFormat(2).bitSize = legacy_header.BitsPerPhase;
               end
               
               ext.GEOINFO.pixLocImCtrRow      = (legacy_header.RgCnt-1)/2;
               ext.GEOINFO.pixLocImCtrCol      = (legacy_header.AzCnt-1)/2;

               ext.APINFO.yearMidAp   = legacy_header.DateTime(1);
               ext.APINFO.monthMidAp  = legacy_header.DateTime(2);
               ext.APINFO.dayMidAp    = legacy_header.DateTime(3);
               ext.APINFO.hourMidAp   = legacy_header.DateTime(4);
               ext.APINFO.minuteMidAp = legacy_header.DateTime(5);
               ext.APINFO.secondMidAp = legacy_header.DateTime(6);               
               
               [obj.header_, obj.ext_] = obj.mapped_fields_legacy_to_modern(header, ext, legacy_header, 1);
           end
        end

        function varargout = mapped_fields_legacy_to_modern(obj, header, ext, legacy_header, forward)
            % loop through header fields
            skip=0;
            rosetta = gff.rosetta(1);
            
            for f = fieldnames(legacy_header).'
                s = char(f);
                switch s
                    % no-op
                    case {'Version_Minor','Version_Major', 'Length',...
                          'Res2', 'Res3', 'Res5', 'Res6', 'Res7'}

                    % special cases                      
                    case {'BytesPerPixel', 'ImageType', 'BitsPerPhase', 'BitsPerMagnitude'}
                        
                    % this is just detritis from my reader...
                    case {'ver_maj','ver_min'}
                        
                    case fieldnames(rosetta.header)
                        if(forward), 
                            header.(rosetta.header.(s)) = legacy_header.(s);
                        else
                           legacy_header.(s) = header.(rosetta.header.(s));
                        end
                        
                    case fieldnames(rosetta.ext)
                        fields = regexp(rosetta.ext.(s),'\.','split');
                        if(forward)
                            ext = setfield(ext, fields{:}, legacy_header.(s));
                        else
                            if isfield(ext,fields{1})
                                legacy_header.(s) = getfield(ext, fields{:});
                            end
                        end
                        
                    case 'Endian' % (0 is little endian in gff pre 2.0)
                        if(forward)
                            header.endian = ~legacy_header.Endian;
                        else
                            legacy_header.Endian = ~header.endian;
                        end
                        
                    case 'RowMajor' % pixOrder 1 == AZ_CONSECUTIVE
                        if(forward)
                            header.pixOrder = ~legacy_header.RowMajor;
                        else
                            legacy_header.RowMajor = ~header.pixOrder;
                        end

                    otherwise
                        skip = skip+1;
                        obj.warning('Skipping %20s => no known conversion to modern HDR/EXT field\n', s);
                end
            end
   
            obj.warning('=================================\n');
            obj.warning('Skipped %d fields in legacy header\n', skip);
            
            if forward
               varargout{1}=header; 
               varargout{2}=ext;
            else
                varargout{1}=legacy_header;
            end

        end        
    end
    
    methods(Static)
        
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        % Functional Interfaces for gff.m:
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        
        function [header, ext, image,rawmag] = read_gff(filename)
        %GFF.READ_GFF reads a GFF 1.8/2.x file in a GFF 2.x format
        %    [HEADER, EXT]        = GFF.READ_GFF(FILENAME)
        %    [HEADER, EXT, IMAGE] = GFF.READ_GFF(FILENAME)
        %
        % FILENAME is the full GFF path that is to be loaded
        %
        % HEADER is a MATLAB struct containing GFF 2.x header fields
        %
        % EXT is a MATLAB struct of structs where each child struct
        % represents a GFF 2.x extension contained in the file (for GFF 2.x
        % files) or a base set of synthesized GFF 2.x extensions (for GFF
        % 1.x files).
        %
        % IMAGE is an optional output that has a complex matrix (real for a
        % magnitude only source image) with the image
        %
        %   See also GFF.WRITE_GFF
        %            GFF.WRITE_LEGACY_GFF
        %            GFF.READ_LEGACY_GFF
            gff_image = gff();
            if(nargout<3)
               gff_image.options.load_image=0;
            end
            gff_image.open(filename);
            header = gff_image.header;
            ext    = gff_image.ext;
            if (gff_image.options.load_image)
                image  = gff_image.cimg;
                if (isfield(gff_image.image_raw,'mag'))
                    rawmag = gff_image.image_raw.mag;
                end
            end
        end

        function write_gff(filename, header, ext, image)        
        %GFF.WRITE_GFF will write a GFF image version 2.x
        %    WRITE_GFF(FILENAME, HEADER, EXT, IMAGE)
        %
        % FILENAME, HEADER, and EXT are formatted as read by the function
        % GFF.READ_GFF
        %
        % FILENAME is the full GFF path that is to be written
        %
        % HEADER is a MATLAB struct containing GFF 2.x header fields
        %
        % EXT is a MATLAB struct of structs where each child struct
        % represents a GFF 2.x extension contained in the file (for GFF 2.x
        % files) or a base set of synthesized GFF 2.x extensions (for GFF
        % 1.x files).
        %
        % IMAGE is a complex matrix (real for a magnitude only source
        % image) with the image to be written
        %
        %   See also GFF.READ_GFF
        %            GFF.WRITE_LEGACY_GFF
        %            GFF.READ_LEGACY_GFF        
            gff_image = gff();
            gff_image.header = header;        
            gff_image.ext    = ext;
            gff_image.cimg   = image;
            gff_image.write(filename);
        end

        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        % Development tools for the gff reader/writer suite
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        function dev_regenerate_library()
        % GFF.DEV_REGENERATE_LIBRARY Regenerate gff header and extension parsing code
        % 
        % GFF.DEV_REGENERATE_LIBRARY replaces the functions that read and
        % write gff header (1.x/2.x) and gff extensions (2.x). This
        % involves gff.m modifying and rewriting its own code.
            
            % Issue a warning to the user as this is potentially dangerous.
            fprintf(2, 'WARNING: gff.m is about to modify itself.\n');
            fprintf(2, 'This will create a new gff.m in the current directory,\n');
            fprintf(2, 'moving the existing gff.m to gff.m_backup if needed.\n');
            reply = input('Do you want to continue? Y/N [N]:','s');
            if ~strcmp(reply,'Y')
                return;
            end
            
            % Find the correct gff.m
            file = which('gff');
            fin = fopen(file, 'r');
            gff_class = fread(fin, inf, '*char').';
            fclose(fin);
            
            % If this is in the current directory move gff.m to
            % gff.m_backup
            if(strcmp(fileparts(file),pwd))
                success = copyfile('gff.m','gff.m_backup');
                if ~success
                    error('Creating backup of gff.m failed.');
                end
            end
            
            % Here we create the new extension library code
            %file = which('gff_extension_library');
            %fid = fopen(file, 'r');
            %gff_ext = fread(fid, inf, '*char').';
            %fclose(fid);
            gff_ext = synthesize_extension_library();
            
            % Add the date
            % strcat: don't want to match ourself on these lines!
            cb = strcat('<%com','ment%>');
            ce = strcat('</%com','ment%>');
            comment = strcat('%', datestr(now));
            gff_class = regexprep(gff_class, strcat(cb,'.*',ce), sprintf('%s\n%s\n%%%s', cb, comment, ce));
            
            % Now add the newly synthesized library code
            gb = strcat('<%gff','lib%>');
            ge = strcat('</%gff','lib%>');
            gff_class = regexprep(gff_class, strcat(gb,'.*',ge), sprintf('%s\n%s\n%%%s', gb, gff_ext, ge));            
            
            %
            file = 'gff.m';
            fout = fopen(file, 'w');
            fwrite(fout, gff_class);
            fclose(fout);

%             function str = warning_string()
%                 str = [ ...
%                     '%-----------------------------------------------------------------------------\n'...
%                     '%-----------------------------------------------------------------------------\n'...
%                     '%\n'...
%                     '% WARNING: THIS IS MACHINE GENERATED CODE, PLEASE DO NOT EDIT DIRECTLY!\n' ...
%                     '%\n'...
%                     '%-----------------------------------------------------------------------------\n'...
%                     '%-----------------------------------------------------------------------------\n'...
%                     ];
%                 str = strcat(str);
%             end
        end
        
        function dev_create_standalone_library()
            bl_fid = fopen('gff_extension_library.m','w'); %#ok<MCMFL>            
            lib=synthesize_extension_library();
            fwrite(bl_fid, lib);
            fclose(bl_fid);
        end
                
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        % Utilities Routines
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%        

        %-------------------------------------------------------------------------%
        % enum -> string (forward) or string -> enum (backward, that is, ~forward)
        function output = enum_lookup(enum_name, index, forward)
            
            % forward -> number to string
            % backward -> string to number
            
            enum_table.endian                 = {'ieee-be','ieee-le','ieee-be.l64','ieee-le.l64'};
            enum_table.pixOrder               = {'RANGE_CONSECUTIVE', 'AZ_CONSECUTIVE'};
            enum_table.imageCompressionScheme = {'none', 'jpeg', 'zlib', 'jpeg2000'};
            enum_table.pixDataType            = {'MAG_UCHAR';
                                                 'MAG_PHASE_USHORT';
                                                 'COMPLEX_USHORT';
                                                 'COMPLEX_UINT';
                                                 'COMPLEX_ULONG';
                                                 'MAG_CHAR';
                                                 'MAG_PHASE_SHORT';
                                                 'COMPLEX_SHORT';
                                                 'COMPLEX_INT';
                                                 'COMPLEX_LONG';
                                                 'COMPLEX_SINGLE';
                                                 'COMPLEX_DOUBLE';
                                                 'MAG_PHASE_UCHAR';
                                                 'UNDEFINED_PIXEL_TYPE'};
            enum_table.cmplxDomain            = {'IQ', 'QI', 'MP', 'I1Q2', ...
                                                 'Q1I2', 'M1P2', 'P1M2', ...
                                                 'M', 'P'};

            if ~isfield(enum_table, enum_name)
                error('Unknown enum %s', enum_name);
            end
            
            if forward
                % Indexing is from 0 (C style), we need to convert to indexing from 1
                % (Matlab style)
                index = index+1;
                output = enum_table.(enum_name){index};
            else
                output = find(strcmp(enum_table.(enum_name),index))-1;
            end
            
        end

        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%
        % Conversion Routines
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%

        %-------------------------------------------------------------------------%
        % Enum -> String (Header)
        %-------------------------------------------------------------------------%
        function header = header_enum_to_string(header, forward)
            for f={'pixDataType','pixOrder','endian','imageCompressionScheme'}
                fld = char(f);
                header.(fld) = gff.enum_lookup(fld, header.(fld), forward); 
            end
            header.pixelFormat.cmplxDomain = ...
                gff.enum_lookup('cmplxDomain', header.pixelFormat.cmplxDomain, forward);
        end
            
        %-------------------------------------------------------------------------%
        % Enum -> String (Ext)
        %-------------------------------------------------------------------------%
        function ext = ext_enum_to_string(ext, forward) %#ok<INUSD>
            % no-op for now
        end
            
        %-------------------------------------------------------------------------%
        % Enum -> String (Legacy Header)
        %-------------------------------------------------------------------------%        
        function header = legacy_header_enum_to_string(header, forward) %#ok<INUSD>
            % no-op for now
        end    
        
        %-------------------------------------------------------------------------%
        % 1.x Header Raw bytes -> Human Readable Form
        %-------------------------------------------------------------------------%
        % Unlike 2.x files, the 1.x file format scales certain fields. I
        % believe this is because until 1.6 the GSAT image format could not
        % contain floating point numbers
        function Header = legacy_header_raw_to_human(Header)
            Header.RgPixelSz            = double(Header.RgPixelSz)         * 2^(-16);
            Header.AzPixelSz            = double(Header.AzPixelSz)         * 2^(-16);
            Header.AzOverlap            = double(Header.AzOverlap)         * 2^(-16);
            Header.SRPLat               = double(Header.SRPLat)            * 2^(-23);
            Header.SRPLong              = double(Header.SRPLong)           * 2^(-23);
            Header.SRPAlt               = double(Header.SRPAlt)            * 2^(-16);
            Header.RFOA                 = double(Header.RFOA)              * 2^(-23);
            Header.XtoSRP               = double(Header.XtoSRP)            * 2^(-16);
            Header.GrzAngle             = double(Header.GrzAngle)          * 2^(-23);
            Header.Squint               = double(Header.Squint)            * 2^(-23);
            Header.GTA                  = double(Header.GTA)               * 2^(-23);
            Header.RgBeamCtr            = double(Header.RgBeamCtr)         * 2^(-8);
            Header.RgChirpRate          = double(Header.RgChirpRate)       * 2^(+12);
            Header.XtoStart             = double(Header.XtoStart)          * 2^(-16);
            Header.V_X                  = double(Header.V_X)               * 2^(-16);
            Header.APCLat               = double(Header.APCLat)            * 2^(-23);
            Header.APCLong              = double(Header.APCLong)           * 2^(-23);
            Header.APCAlt               = double(Header.APCAlt)            * 2^(-16);
            Header.Calparm              = double(Header.Calparm)           * 2^(-24);
            Header.AzResolution         = double(Header.AzResolution)      * 2^(-16);
            Header.RgResolution         = double(Header.RgResolution)      * 2^(-16);
            Header.DesSigmaN            = double(Header.DesSigmaN)         * 2^(-23);
            Header.DesGrazAngle         = double(Header.DesGrazAngle)      * 2^(-23);
            Header.DesSquint            = double(Header.DesSquint)         * 2^(-23);
            Header.DesRng               = double(Header.DesRng)            * 2^(-8);
            Header.SceneTrackAngle      = double(Header.SceneTrackAngle)   * 2^(-23);
            Header.VelocY               = double(Header.VelocY)            * 2^(-16);
            Header.VelocZ               = double(Header.VelocZ)            * 2^(-16);
            Header.SigmaN               = double(Header.SigmaN)            * 2^(-23);
        end
        
        %-------------------------------------------------------------------------%
        % 1.x Header Human Readable Form -> Raw bytes
        %-------------------------------------------------------------------------%        
        % See above
        function Header = legacy_header_human_to_raw(Header)
            Header.RgPixelSz            = Header.RgPixelSz            * 2^(+16);
            Header.AzPixelSz            = Header.AzPixelSz            * 2^(+16);
            Header.AzOverlap            = Header.AzOverlap            * 2^(+16);
            Header.SRPLat               = Header.SRPLat               * 2^(+23);
            Header.SRPLong              = Header.SRPLong              * 2^(+23);
            Header.SRPAlt               = Header.SRPAlt               * 2^(+16);
            Header.RFOA                 = Header.RFOA                 * 2^(+23);
            Header.XtoSRP               = Header.XtoSRP               * 2^(+16);
            Header.GrzAngle             = Header.GrzAngle             * 2^(+23);
            Header.Squint               = Header.Squint               * 2^(+23);
            Header.GTA                  = Header.GTA                  * 2^(+23);
            Header.RgBeamCtr            = Header.RgBeamCtr            * 2^(+8);
            Header.RgChirpRate          = Header.RgChirpRate          * 2^(-12);
            Header.XtoStart             = Header.XtoStart             * 2^(+16);
            Header.V_X                  = Header.V_X                  * 2^(+16);
            Header.APCLat               = Header.APCLat               * 2^(+23);
            Header.APCLong              = Header.APCLong              * 2^(+23);
            Header.APCAlt               = Header.APCAlt               * 2^(+16);
            Header.Calparm              = Header.Calparm              * 2^(+24);
            Header.AzResolution         = Header.AzResolution         * 2^(+16);
            Header.RgResolution         = Header.RgResolution         * 2^(+16);
            Header.DesSigmaN            = Header.DesSigmaN            * 2^(+23);
            Header.DesGrazAngle         = Header.DesGrazAngle         * 2^(+23);
            Header.DesSquint            = Header.DesSquint            * 2^(+23);
            Header.DesRng               = Header.DesRng               * 2^(+8);
            Header.SceneTrackAngle      = Header.SceneTrackAngle      * 2^(+23);
            Header.VelocY               = Header.VelocY               * 2^(+16);
            Header.VelocZ               = Header.VelocZ               * 2^(+16);
            Header.SigmaN               = Header.SigmaN               * 2^(+23);
        end

    end % end Static methods
    
    methods(Static, Access = private)

        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%        
        % Helper fuction for reading / writing raw data
        %-------------------------------------------------------------------------%
        %-------------------------------------------------------------------------%        
        
        %-------------------------------------------------------------------------%        
        % This will transpose all matrices in a struct. This is helpful for
        % reading in row major OR col major files in a clean way.
        function struct = transpose_struct(struct)
            for f=fieldnames(struct).'
                s = char(f);
                struct.(s) = struct.(s).';
            end
        end     

        %-------------------------------------------------------------------------%                
        % This is a lookup table that is used to convert from GFF 1.x files
        % to GFF 2.x (and back!)
        function rosetta = rosetta(forward)
            
            % GSATIMG
            table.header = {'Creator_Length',  'imageCreatorLen';
                            'Creator',         'imageCreator';
                            'RgCnt',           'rangePixels';
                            'AzCnt',           'azPixels';
                            'ImageLength',     'imageLengthBytes';
                            'ImageCmpScheme',  'imageCompressionScheme';
                            'PixValLinearity', 'pixValLin';};
                        
            % EXTENSIONS
            
            % GEOINFO 1.1
            table.ext = {'ImagePlane',      'GEOINFO.imagePlane';
                         'RgPixelSz',       'GEOINFO.rangePixSpacing';
                         'DesGrazAngle',    'GEOINFO.desiredGrazAng';
                         'AzPixelSz',       'GEOINFO.azPixSpacing';
                         'SRPLat',          'GEOINFO.patchCtrLat';
                         'SRPLong',         'GEOINFO.patchCtrLong';
                         'SRPAlt',          'GEOINFO.patchCtrAlt';
                         'SceneTrackAngle', 'GEOINFO.imgRotAngle';
                         
            % APINFO 1.1
                         'PhaseName_Length', 'APINFO.phNameLen';
                         'PhaseName',        'APINFO.phName';
                         'FlightTime',       'APINFO.flightTime ';
                         'GPSWeek',          'APINFO.flightWeek ';
                         'RgChirpRate',      'APINFO.chirpRate';
                         'GrzAngle',         'APINFO.grazingAngle';
                         'Squint',           'APINFO.squintAngle';
                         'GTA',              'APINFO.gta';
                         'Dgta',             'APINFO.desGTA';
                         'RgBeamCtr',        'APINFO.rngToBeamCtr';
                         'XtoStart',         'APINFO.xDistToStart';
                         'MoCompMode',       'APINFO.momeasMode';
                         'APCLat',           'APINFO.apcLat';
                         'APCLong',          'APINFO.apcLon';
                         'APCAlt',           'APINFO.apcAlt';
                         'DesSqint',         'APINFO.desSquint';
                         'DesRng',           'APINFO.desRng';
                         'VelocY',           'APINFO.xVel';
                         'VelocZ',           'APINFO.yVel';
                         'Ba',               'APINFO.azBeamWidth';
                         'Be',               'APINFO.elBeamWidth';
                         
             % IFINFO 
                         'ProcProduct',      'IFINFO.procProduct';
                         %'                   'IFINFO.imgFileNameLen';
                         %'                   'IFINFO.imgFileName';
                         'AzResolution',     'IFINFO.azResolution';
                         'RgResolution',     'IFINFO.rngResolution';                       
                         'Calparm',          'IFINFO.imgCalParam';
                         'SigmaN',           'IFINFO.sigmaN';
                         'IFAlgo',           'IFINFO.ifAlgo';
                         'ImageFlags',       'IFINFO.imgFlag';
                         'AzCoeff',          'IFINFO.azCoeff';
                         'ElevationCoeff',   'IFINFO.elCoeff';
                         'AzGeomCorr',       'IFINFO.azGeoCorrect';
                         'RngGeomCorr',      'IFINFO.rngGeoCorrect';
                         'AzWinFacBW',       'IFINFO.wndBwFactAz';
                         'RngWinFacBW',      'IFINFO.wndBwFactRng';
                         'AzWinID',          'IFINFO.wndFncIdAz';
                         'RngWinID',         'IFINFO.wndFncIdRng';
                         'Comment_Length',   'IFINFO.cmtLen';
                         'Comment',          'IFINFO.cmtText';
                         };
                     
            if forward % legacy -> modern
                for idx = 1:size(table.header,1)
                    rosetta.header.(table.header{idx,1}) = table.header{idx,2};
                end
                for idx = 1:size(table.ext,1)
                    rosetta.ext.(table.ext{idx,1}) = table.ext{idx,2};
                end
            else
                for idx = 1:size(table.header,1)
                   rosetta.header.(table.header{idx,2}) = table.header{idx,1};
                end                
            end
                            
        end        
        
        %---------------------------------------------------------------------%
        %---------------------------------------------------------------------%
        % General Utility Functions
        %---------------------------------------------------------------------%
        %---------------------------------------------------------------------%        

        %-------------------------------------------------------------------------%        
        % This function handles unzipping and zipping buffers 
        function output = zlib(operation, input)
            
            input = input(:);
            
            ostream=java.io.ByteArrayOutputStream;
            
            switch operation
                case 'compress'
                    buffer = java.io.ByteArrayOutputStream();
                    zlib = java.util.zip.DeflaterOutputStream(buffer);
                    zlib.write(input, 0, numel(input));
                    zlib.close();
                    output = typecast(buffer.toByteArray(), 'uint8')';
                    
                case 'uncompress'
                    buffer = java.io.ByteArrayOutputStream();
                    zlib = java.util.zip.InflaterOutputStream(buffer);
                    zlib.write(input, 0, numel(input));
                    zlib.close();
                    output = typecast(buffer.toByteArray(), 'uint8')';
                otherwise
                    error('Unknown operation %s', operation);
            end
            
            ostream.close;
        end
                
        %-------------------------------------------------------------------------%
        % size of datatype in bytes, given datatype as string
        function bytes = sizeof(type)
           bytes = length(typecast(cast(0,type),'uint8'));
        end        

        %-------------------------------------------------------------------------%
        % Convert every numeric value in a structure to double
        function struct = to_double(struct)
            for f=fieldnames(struct).'
                fld = (char(f));
                type = class(struct.(fld));
                switch type
                    case {'double','char'}
                        % no-op
                    case {'uint8','uint16', 'int16', 'uint32', 'int32','logical','single'}
                        struct.(fld) = double(struct.(fld));                        
                    case 'struct'
                        for ii=1:length(struct.(fld))
                            struct.(fld)(ii) = gff.to_double(struct.(fld)(ii));
                        end
                    otherwise
                        warning(2, 'No support for type %s, \n', type); 
                end
            end
        end        
        
    end
end

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Utilities to Help Read and Write Images
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%

%---------------------------------------------------------------------%
% Convert jpeg/jpeg2000 data to raw image
function raw = read_image_format_data(raw_data, file_type)

    ext = '.jp2';
    if strcmp(file_type, 'jpeg'), ext = '.jpg'; end
    fname = fullfile(pwd, ['delete_me' ext]);

    %Write to the file
    temp_fid = fopen(fname, 'w');
    fwrite(temp_fid, raw_data);
    fclose(temp_fid);

    %Read the file as an image
    raw.mag = imread(fname);
    delete(fname);
end

%---------------------------------------------------------------------%
% Convert jpeg/jpeg2000 data to raw bytes ready to be written to
% disk
function raw = pack_image_format_data(input, file_type)

    comp_val = 5;
    try %#ok<TRYNC>
       comp_val = ext.COMPRESSINFO.compressionVal;
    end 

    fname = fullfile(pwd, 'tempImg');
    switch file_type
        case 'jpeg'
            fname = 'tempImg.jpg';
            opts = {'Quality', comp_val};
        case 'jp2'
            fname = 'tempImg.jp2';
            opts = {'jp2','ReductionLevels',5,'CompressionRatio', comp_val};
    end

    % write image to disk
    imwrite(input, fname, opts{:});

    %Read the file as a data
    temp_fid = fopen(fullfile(pwd,fname), 'r');
    raw = fread(temp_fid, inf, '*uint8');
    fclose(temp_fid);
    try delete(fname); end %#ok<TRYNC>
end

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Init / Read / Write Blocks and Block Headers
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%
function block = init_block(block_tag)
    init_fun = str2func(['init_' block_tag]);
    block = init_fun();
end

%------------------------------------------------------------------
% Fix a known problem with early headers
function block = repair_block_header(block)
    if ~exist(sprintf('read_%s_%d_%d', block.name, block.ver_maj, block.ver_min))
        v = @(ma, mi, b) ((block.ver_maj == ma && ...
            block.ver_min == 0 && ...
            block.bytes == b));
        if (v(2,0,64) || v(1,0,52))
            block.name = 'RADARINFO';
        end
    end
end

%-------------------------------------------------------------------------%
function block = read_block_header(fid)
    buffer = fread(fid, 32, '*uint8');
    block.name    = char(buffer(1:16)).';
    block.ver_maj = typecast(buffer(17:18), 'uint16');
    block.ver_min = typecast(buffer(19:20), 'uint16');
    block.bytes   = typecast(buffer(25:28), 'int32');
    block.name(block.name==0)=32;
    block.name = strtrim(block.name);
end

%-------------------------------------------------------------------------%
function write_block_header(fid, title, block, bytes)
    fwrite(fid, [title, zeros(1,16-length(title))], 'char');
    fwrite(fid, double(block.ver_maj),              'uint16');
    fwrite(fid, double(block.ver_min),              'uint16');
    fwrite(fid, repmat(255,1,4),                    'uint8');
    fwrite(fid, bytes,                              'uint32');
    fwrite(fid, repmat(255,1,4),                    'uint8');
end

%-------------------------------------------------------------------------%
function bytes=calculate_size_from_data(block)

    switch block.name
        case 'SMDINFO'
            bytes = 20+17*block.eVarLenType.numElements;
    end
    return;

    % this doesn't work if the block has been modified... MATLAB
    % changes everything to a double.... 
    bytes = 0;
    for f=fieldnames(block).'
        fld = f{1};
        switch fld
            case {'ver_maj','ver_min','name'}
            otherwise
                if isstruct(block.(fld))
                    bytes = bytes + calculate_size_from_data(block.(fld));
                else
                    bytes = bytes + length(typecast(block.(fld),'uint8'));
                end
        end
    end
end

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Utilities to Help Marshal / Unmarshal Blocks
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%

function m_str = marshal_string(str, len)
    m_str = zeros([len, 1], 'uint8');
    slen = length(str);
    m_str(1:min(len,slen))=uint8(str);
end

%-------------------------------------------------------------------------%        
function libstr = synthesize_extension_library()

    built_in = {'uint','uint8','uint16','uint32','uint64',...
                'uchar','ushort','ulong',...
                'int','int8','int16','int32','int64',...
                'schar','short','long',...
                'single','double','float','float32','float64',...
                'char'};

    libstr = '';
    
    function puts(str)
        % not strcat because it ignores final whitespace
        %libstr = strcat(libstr, sprintf(str));
        libstr = [libstr sprintf('%s', str)];
    end
    
    puts('%% machine generated: do not edit\n');
    
    %% Load defs
    defs = dir(fullfile(def_dir(), '*.def'));
    names = regexp({defs.name},'(\w+_\d_\d).def','tokens');
    names = names(~cellfun(@isempty, names));
    for idx=1:length(names); names{idx} = names{idx}{1}{1}; end
    
    %% Remove the entry for 'GSATIMG' as it is 'special'
    names(find(strncmp(names, 'GSATIMG_2', 9),1,'first')) = [];
    
    %% make new entry for GSATIMG blocks:
    gff_versions = cell(1,6);
    for min_ver = 0:5
       gff_versions{min_ver+1} = sprintf('GSATIMG_%d_%d',2,min_ver); 
    end
  
    %---------------------------------------------------------------------%
    % Block Size
    puts('function size = block_size(blockname)\n\n');
    
    puts('    gff_versions = {');
    for idx=1:length(gff_versions)
        puts(sprintf('''%s''', gff_versions{idx}));
        if idx ~= length(gff_versions)
           puts(';\n                    '); 
        end
    end
    puts('};\n\n');    
    
    puts('    switch blockname\n');
    puts('        case gff_versions\n');
    def = load_def(fullfile(def_dir(),'GSATIMG_2_5.def'));
    puts(sprintf('            size = %d;\n\n', count_def_bytes(def, 'GSATIMG_2_5')));
    for idx = 1:length(names)
        puts(sprintf('        case ''%s''\n', names{idx}));
        def = load_def(fullfile(def_dir(),[names{idx} '.def']));
        bytes = count_def_bytes(def, names{idx});
        if isempty(bytes)
            puts('            size = [];\n\n');
        else
            puts(sprintf('            size = %d;\n\n', bytes));
        end
    end
    puts('    end\n');    
    puts('end\n\n');
    
    %% Create Function for Individual blocks
    for idx = 1:length(defs)
        fprintf('Synthesizing code for: %s\n', defs(idx).name);
        out = regexp(defs(idx).name,'(\w+)_(\d)_(\d).def','tokens');
        if(~isempty(out))
           block.name = out{1}{1};
           block.ver_maj = str2double(out{1}{2});
           block.ver_min = str2double(out{1}{3});

           block_tag = sprintf('%s_%d_%d',block.name,block.ver_maj,block.ver_min);
           filename = fullfile([block_tag '.def']);
           def = load_def(fullfile(def_dir(),filename));
           
           %--------------------------------------------------------------%
           % Block Readers
%           puts(sprintf('function %s = %s(fid) %%#ok<DEFNU>\n', block.name, ['read_' block_tag]));
           puts(sprintf('function %s = %s(buffer) %%#ok<DEFNU>\n', block.name, ['read_' block_tag]));
           curr = 1;
%           puts(sprintf('    buffer = fread(fid, %d, ''*uint8'');\n', count_def_bytes(def, block_tag)));
           gen_reader_using_def(@puts, def, ['    ' block.name], block_tag);
           puts('end\n\n');

           %--------------------------------------------------------------%
           % Block "Unmarshal" (handle cleanups such as terminating strings)
           puts(sprintf('function %s = %s(%s) %%#ok<DEFNU>\n', block.name, ['unmarshal_' block_tag],block.name));
           curr = 1;
           gen_unmarshal_using_def(@puts, def, ['    ' block.name], block_tag);
           puts('end\n\n');
           
           %--------------------------------------------------------------%
           % Block "Marshal" (convert strings back into uint8, etc)
           puts(sprintf('function %s = %s(%s) %%#ok<DEFNU>\n', block.name, ['marshal_' block_tag],block.name));
           curr = 1;
           gen_marshal_using_def(@puts, def, ['    ' block.name], block_tag);
           puts('end\n\n');               
           
           %--------------------------------------------------------------%
           % Block Writer
           puts(sprintf('function %s(fid, %s) %%#ok<DEFNU>\n',['write_' block_tag],  block.name));
           curr = 1;
           gen_writer_using_def(@puts, def, block.name, block_tag)
           puts('end\n\n');
           
           %--------------------------------------------------------------%
           % Block Init
           puts(sprintf('function %s = %s() %%#ok<DEFNU>\n', block.name, ['init_' block_tag]));
           curr = 1;
           gen_init_using_def(@puts, def, ['    ' block.name], block_tag)
           puts('end\n\n');
        end
    end
    
    %-------------------------------------------------------------------------%
    function gen_reader_using_def(puts, def, prefix, block_tag)

        var_length = false;
        
        for f = 1:size(def,2)

           lhand = [prefix '.' def(f).name ];

           switch def(f).type
% I removed this so we can read and write without byte
% changes. Getting rid of 'garbage' characters is now
% handled when we export the structs.               
%                case 'char'
%                    sz = def(f).count;
%                    %rhand = sprintf('char(buffer(%d:%d)).'';', curr, curr+sz-1);
%                    rhand = sprintf('int8(buffer(%d:%d)).'';', curr, curr+sz-1);
%                    curr = curr+sz;
%                    puts(sprintf('%-45s = %-30s\n', lhand, rhand));
%                    puts(sprintf('%s(find(%s==0,1,''first''):end)=0;\n', lhand,strtrim(lhand)));
               case built_in          
                   sz = sizeof(def(f).type) * def(f).count;
                   rhand = sprintf('typecast(buffer(%d:%d),''%s'');', curr, curr+sz-1, conv_type(def(f).type));
                   curr = curr+sz;
                   if(var_length)
                        lhand = [lhand '(ii)'];  %#ok<AGROW>
                   end
                   puts(sprintf('%-45s = %-30s\n', lhand, rhand));
               case 'GffVarLenData_t'
                   recursive_call(@gen_reader_using_def, puts, def(f), prefix, block_tag);
                   for ff=(f+1):size(def,2)
                        temp = [prefix '.' def(ff).name ];
                        puts(sprintf('%-45s = [];\n', temp));
                   end
                   puts(sprintf('    for ii=1:%s.numElements\n', strtrim(lhand)));
                   prefix_old = prefix;
                   %prefix = ['    ' prefix '(ii)']; %#ok<AGROW>
                   prefix = ['    ' prefix]; %#ok<AGROW>
                   var_length = true;
                   byte_skip = sprintf('%s.elementSize', strtrim(lhand));
               otherwise
                   recursive_call(@gen_reader_using_def, puts, def(f), prefix, block_tag);
           end
        end
                
        if(var_length)
             puts(sprintf('        buffer=buffer(%s+1:end);\n',byte_skip));
             puts('    end\n'); 
             prefix = prefix_old;
        end
        
        puts(sprintf('%-45s = %-30s\n', [prefix '.ver_maj'], [num2str(block.ver_maj) ';']));
        puts(sprintf('%-45s = %-30s\n', [prefix '.ver_min'], [num2str(block.ver_min) ';']));
                
    end

    %-------------------------------------------------------------------------%
    function gen_writer_using_def(puts, def, prefix, block_tag)

        var_length = false;        
        
        for f = 1:size(def,2)

           lhand = [prefix '.' def(f).name ];

           switch def(f).type
% See above (gen_reader) comments
% 		       case 'char'
% 			       pf_string = sprintf('[%s zeros(1, %d - length(%s))]', lhand, def(f).count, lhand);
% 			       puts(sprintf('    fwrite(fid, %s, ''char'');\n', pf_string));
               case built_in
                   if(var_length)
                       puts('    ');
                       lhand = [lhand '(ii)']; %#ok<AGROW>
                   end
                   puts(sprintf('    fwrite(fid, %-45s ''%s'');\n', [lhand ','], conv_type(def(f).type)));
               case 'GffVarLenData_t'
                   recursive_call(@gen_writer_using_def, puts, def(f), prefix, block_tag);
                   puts(sprintf('    for ii=1:%s.numElements\n', strtrim(lhand)));
                   %prefix = [prefix '(ii)']; %#ok<AGROW>
                   var_length = true;
               otherwise
                   recursive_call(@gen_writer_using_def, puts, def(f), prefix, block_tag);
           end
        end
        
        if(var_length)
             puts('    end\n'); 
        end        
        
    end

    %-------------------------------------------------------------------------%
    function gen_init_using_def(puts, def, prefix, block_tag)

        for f = 1:size(def,2)

           lhand = [prefix '.' def(f).name ];

           switch def(f).type
               case 'char'
                   rhand = sprintf('char(zeros(1,%d));',def(f).count);
                   puts(sprintf('%-45s = %-30s\n', lhand, rhand));
               case built_in          
                   rhand = sprintf('zeros(1,%d,''%s'');', def(f).count, conv_type(def(f).type));
                   puts(sprintf('%-45s = %-30s\n', lhand, rhand));
               otherwise
                   recursive_call(@gen_init_using_def, puts, def(f), prefix, block_tag);
           end
        end
        
        puts(sprintf('%-45s = %-30s\n', [prefix '.ver_maj'], [num2str(block.ver_maj) ';']));
        puts(sprintf('%-45s = %-30s\n', [prefix '.ver_min'], [num2str(block.ver_min) ';']));
    end

    %-------------------------------------------------------------------------%
    function gen_marshal_using_def(puts, def, prefix, block_tag)

        for f = 1:size(def,2)

           lhand = [prefix '.' def(f).name ];

           switch def(f).type         
                case 'char'
                    sz = def(f).count;
                    puts(sprintf('%s = marshal_string(%s, %d);\n', lhand, strtrim(lhand), sz));
               case built_in          
                   % no-op
               otherwise
                   recursive_call(@gen_marshal_using_def, puts, def(f), prefix, block_tag);
           end
        end
    end

    %-------------------------------------------------------------------------%
    function gen_unmarshal_using_def(puts, def, prefix, block_tag)

        for f = 1:size(def,2)

           lhand = [prefix '.' def(f).name ];

           switch def(f).type         
                case 'char'
                    puts(sprintf('%s = char(%s.'');\n', lhand, strtrim(lhand)));
                    puts(sprintf('%s(find(%s==0,1,''first''):end)=0;\n', lhand,strtrim(lhand)));
               case built_in          
                   % no-op
               otherwise
                   recursive_call(@gen_unmarshal_using_def, puts, def(f), prefix, block_tag);
           end
        end
    end

    %-------------------------------------------------------------------------%
    function recursive_call(fun_handle, puts, def, prefix, block_tag)
        rdef = load_def(fullfile(def_dir(),[block_tag '_' def.type '.def']));
        if(def.count == 1)
            fun_handle(puts, rdef, [prefix '.' def.name], block_tag)
        else
            for idx2 = 1:def.count
                fun_handle(puts, rdef, [prefix '.' def.name '(' num2str(idx2) ')'], block_tag)
            end
        end
    end

    %-------------------------------------------------------------------------%
    function bytes = count_def_bytes(def, block_tag)

        bytes = 0;
        for f = 1:size(def,2)
            switch def(f).type
                case built_in
                    b = sizeof(def(f).type);
                case 'GffVarLenData_t' % Variable Length, Give Up!
                    b = [];
                otherwise % recursive
                    filename = fullfile(def_dir(), [block_tag '_' def(f).type '.def']);
                    b = count_def_bytes(load_def(filename), block_tag);
            end
            if(isempty(b))
                bytes = [];
                return;
            end
            bytes = bytes + b*def(f).count;
        end
    end

end

%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
% Utility Functions for Synthesizing Extension Library
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%

%-------------------------------------------------------------------------%
% size of datatype in bytes, given datatype as string
function s = sizeof(type)
    type = conv_type(type);
    t = typecast([], type);
    t(1)=0; %#ok;
    t=whos('t');
    s=t.bytes;
end

%---------------------------------------------------------------------%
function type=conv_type(type)
    type = regexprep(type, 'char', 'uint8');
    type = regexprep(type, 'ushort', 'uint16');
    type = regexprep(type, 'float32', 'single');
    type = regexprep(type, 'float64', 'double');
end

%-------------------------------------------------------------------------%
function d = def_dir()
    d = fullfile(fileparts(which('gff')),'defs');
end

%-------------------------------------------------------------------------%
function [ def_struct ] = load_def(def_file)
    def_struct = struct('name', [], 'type', [], 'count', []);

    % With recent versions of MATLAB importdata is *really* slow
    %     def = importdata(def_file, ',');
    %     for i=1:size(def.data,1)
    %         def_struct(i).name = cell2mat(def.textdata(i,1));
    %         def_struct(i).type = cell2mat(def.textdata(i,2));
    %         def_struct(i).count = def.data(i);
    %     end

    % import data with textscan into a cell array
    fid = fopen(def_file,'r');
    if fid < 0
        fprintf(2,'Can''t open %s\n', def_file);
    end
    input = textscan(fid,'%s','delimiter',{',','\n'});
    fclose(fid);
    input = input{1};

    % strip off empty cells at the end ('') and then reshape
    idx = length(input);
    while(strcmp(input{idx},''))
        idx = idx-1;
    end
    input = reshape(input(1:idx), 3, idx/3);

    % move from a cell array to an array of structs
    for i=1:idx/3
        def_struct(i).name = input{1,i};
        def_struct(i).type = input{2,i};
        def_struct(i).count = str2double(input{3,i});
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%<%gfflib%>
%% machine generated: do not edit
function size = block_size(blockname)

    gff_versions = {'GSATIMG_2_0';
                    'GSATIMG_2_1';
                    'GSATIMG_2_2';
                    'GSATIMG_2_3';
                    'GSATIMG_2_4';
                    'GSATIMG_2_5'};

    switch blockname
        case gff_versions
            size = 82;

        case 'APINFO_1_0'
            size = 314;

        case 'APINFO_2_0'
            size = 318;

        case 'APINFO_3_0'
            size = 334;

        case 'APINFO_4_0'
            size = 418;

        case 'APINFO_5_0'
            size = 426;

        case 'APINFO_5_1'
            size = 430;

        case 'APINFO_5_2'
            size = 434;

        case 'CCDINFO_1_0'
            size = 544;

        case 'CCDINFO_1_1'
            size = 552;

        case 'CHIPINFO_1_0'
            size = 44;

        case 'COMPRESSINFO_1_0'
            size = 12;

        case 'COMPRESSINFO_1_1'
            size = 20;

        case 'GEOINFO_1_0'
            size = 52;

        case 'GEOINFO_1_1'
            size = 52;

        case 'GMTIINFO_1_0'
            size = 8;

        case 'GSATIMG_1_6'
            size = 952;

        case 'GSATIMG_1_8'
            size = 2040;

        case 'IFINFO_1_0'
            size = 514;

        case 'IFINFO_2_0'
            size = 582;

        case 'IFINFO_2_1'
            size = 582;

        case 'IFINFO_2_2'
            size = 582;

        case 'IFINFO_3_0'
            size = 586;

        case 'MOMEASINFO_1_0'
            size = 16;

        case 'MOMEASINFO_2_0'
            size = 92;

        case 'MULTILOOKINFO_1_0'
            size = 208;

        case 'RADARINFO_1_0'
            size = 52;

        case 'RADARINFO_2_0'
            size = 64;

        case 'SMDINFO_2_0'
            size = [];

        case 'SMDINFO_3_0'
            size = [];

    end
end

function APINFO = read_APINFO_1_0(buffer) %#ok<DEFNU>
    APINFO.missionText                        = typecast(buffer(1:8),'uint8');
    APINFO.swVerNum                           = typecast(buffer(9:16),'uint8');
    APINFO.radarSerNum                        = typecast(buffer(17:20),'uint32');
    APINFO.phSource                           = typecast(buffer(21:24),'uint32');
    APINFO.phNameLen                          = typecast(buffer(25:26),'uint16');
    APINFO.phName                             = typecast(buffer(27:154),'uint8');
    APINFO.ctrFreq                            = typecast(buffer(155:158),'single');
    APINFO.wavelength                         = typecast(buffer(159:162),'single');
    APINFO.rxPolarization                     = typecast(buffer(163:166),'uint32');
    APINFO.txPolarization                     = typecast(buffer(167:170),'uint32');
    APINFO.azBeamWidth                        = typecast(buffer(171:174),'single');
    APINFO.elBeamWidth                        = typecast(buffer(175:178),'single');
    APINFO.grazingAngle                       = typecast(buffer(179:182),'single');
    APINFO.squintAngle                        = typecast(buffer(183:186),'single');
    APINFO.gta                                = typecast(buffer(187:190),'single');
    APINFO.rngToBeamCtr                       = typecast(buffer(191:194),'single');
    APINFO.desSquint                          = typecast(buffer(195:198),'single');
    APINFO.desRng                             = typecast(buffer(199:202),'single');
    APINFO.desGTA                             = typecast(buffer(203:206),'single');
    APINFO.antPhaseCtrBear                    = typecast(buffer(207:210),'single');
    APINFO.yearMidAp                          = typecast(buffer(211:212),'uint16');
    APINFO.monthMidAp                         = typecast(buffer(213:214),'uint16');
    APINFO.dayMidAp                           = typecast(buffer(215:216),'uint16');
    APINFO.hourMidAp                          = typecast(buffer(217:218),'uint16');
    APINFO.minuteMidAp                        = typecast(buffer(219:220),'uint16');
    APINFO.secondMidAp                        = typecast(buffer(221:222),'uint16');
    APINFO.flightTime                         = typecast(buffer(223:226),'uint32');
    APINFO.flightWeek                         = typecast(buffer(227:230),'uint32');
    APINFO.chirpRate                          = typecast(buffer(231:234),'single');
    APINFO.xDistToStart                       = typecast(buffer(235:238),'single');
    APINFO.momeasMode                         = typecast(buffer(239:242),'uint32');
    APINFO.radarMode                          = typecast(buffer(243:246),'uint32');
    APINFO.rfoa                               = typecast(buffer(247:250),'single');
    APINFO.xVel                               = typecast(buffer(251:258),'double');
    APINFO.yVel                               = typecast(buffer(259:266),'double');
    APINFO.zVel                               = typecast(buffer(267:274),'double');
    APINFO.apcLat                             = typecast(buffer(275:282),'double');
    APINFO.apcLon                             = typecast(buffer(283:290),'double');
    APINFO.apcAlt                             = typecast(buffer(291:298),'double');
    APINFO.keepOutViol                        = typecast(buffer(299:302),'single');
    APINFO.gimStopTwist                       = typecast(buffer(303:306),'single');
    APINFO.gimStopTilt                        = typecast(buffer(307:310),'single');
    APINFO.gimbalStopAz                       = typecast(buffer(311:314),'single');
    APINFO.ver_maj                            = 1;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = unmarshal_APINFO_1_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = char(APINFO.missionText.');
    APINFO.missionText(find(APINFO.missionText==0,1,'first'):end)=0;
    APINFO.swVerNum = char(APINFO.swVerNum.');
    APINFO.swVerNum(find(APINFO.swVerNum==0,1,'first'):end)=0;
    APINFO.phName = char(APINFO.phName.');
    APINFO.phName(find(APINFO.phName==0,1,'first'):end)=0;
end

function APINFO = marshal_APINFO_1_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = marshal_string(APINFO.missionText, 8);
    APINFO.swVerNum = marshal_string(APINFO.swVerNum, 8);
    APINFO.phName = marshal_string(APINFO.phName, 128);
end

function write_APINFO_1_0(fid, APINFO) %#ok<DEFNU>
    fwrite(fid, APINFO.missionText,                           'uint8');
    fwrite(fid, APINFO.swVerNum,                              'uint8');
    fwrite(fid, APINFO.radarSerNum,                           'uint32');
    fwrite(fid, APINFO.phSource,                              'uint32');
    fwrite(fid, APINFO.phNameLen,                             'uint16');
    fwrite(fid, APINFO.phName,                                'uint8');
    fwrite(fid, APINFO.ctrFreq,                               'single');
    fwrite(fid, APINFO.wavelength,                            'single');
    fwrite(fid, APINFO.rxPolarization,                        'uint32');
    fwrite(fid, APINFO.txPolarization,                        'uint32');
    fwrite(fid, APINFO.azBeamWidth,                           'single');
    fwrite(fid, APINFO.elBeamWidth,                           'single');
    fwrite(fid, APINFO.grazingAngle,                          'single');
    fwrite(fid, APINFO.squintAngle,                           'single');
    fwrite(fid, APINFO.gta,                                   'single');
    fwrite(fid, APINFO.rngToBeamCtr,                          'single');
    fwrite(fid, APINFO.desSquint,                             'single');
    fwrite(fid, APINFO.desRng,                                'single');
    fwrite(fid, APINFO.desGTA,                                'single');
    fwrite(fid, APINFO.antPhaseCtrBear,                       'single');
    fwrite(fid, APINFO.yearMidAp,                             'uint16');
    fwrite(fid, APINFO.monthMidAp,                            'uint16');
    fwrite(fid, APINFO.dayMidAp,                              'uint16');
    fwrite(fid, APINFO.hourMidAp,                             'uint16');
    fwrite(fid, APINFO.minuteMidAp,                           'uint16');
    fwrite(fid, APINFO.secondMidAp,                           'uint16');
    fwrite(fid, APINFO.flightTime,                            'uint32');
    fwrite(fid, APINFO.flightWeek,                            'uint32');
    fwrite(fid, APINFO.chirpRate,                             'single');
    fwrite(fid, APINFO.xDistToStart,                          'single');
    fwrite(fid, APINFO.momeasMode,                            'uint32');
    fwrite(fid, APINFO.radarMode,                             'uint32');
    fwrite(fid, APINFO.rfoa,                                  'single');
    fwrite(fid, APINFO.xVel,                                  'double');
    fwrite(fid, APINFO.yVel,                                  'double');
    fwrite(fid, APINFO.zVel,                                  'double');
    fwrite(fid, APINFO.apcLat,                                'double');
    fwrite(fid, APINFO.apcLon,                                'double');
    fwrite(fid, APINFO.apcAlt,                                'double');
    fwrite(fid, APINFO.keepOutViol,                           'single');
    fwrite(fid, APINFO.gimStopTwist,                          'single');
    fwrite(fid, APINFO.gimStopTilt,                           'single');
    fwrite(fid, APINFO.gimbalStopAz,                          'single');
end

function APINFO = init_APINFO_1_0() %#ok<DEFNU>
    APINFO.missionText                        = char(zeros(1,8));             
    APINFO.swVerNum                           = char(zeros(1,8));             
    APINFO.radarSerNum                        = zeros(1,1,'uint32');          
    APINFO.phSource                           = zeros(1,1,'uint32');          
    APINFO.phNameLen                          = zeros(1,1,'uint16');          
    APINFO.phName                             = char(zeros(1,128));           
    APINFO.ctrFreq                            = zeros(1,1,'single');          
    APINFO.wavelength                         = zeros(1,1,'single');          
    APINFO.rxPolarization                     = zeros(1,1,'uint32');          
    APINFO.txPolarization                     = zeros(1,1,'uint32');          
    APINFO.azBeamWidth                        = zeros(1,1,'single');          
    APINFO.elBeamWidth                        = zeros(1,1,'single');          
    APINFO.grazingAngle                       = zeros(1,1,'single');          
    APINFO.squintAngle                        = zeros(1,1,'single');          
    APINFO.gta                                = zeros(1,1,'single');          
    APINFO.rngToBeamCtr                       = zeros(1,1,'single');          
    APINFO.desSquint                          = zeros(1,1,'single');          
    APINFO.desRng                             = zeros(1,1,'single');          
    APINFO.desGTA                             = zeros(1,1,'single');          
    APINFO.antPhaseCtrBear                    = zeros(1,1,'single');          
    APINFO.yearMidAp                          = zeros(1,1,'uint16');          
    APINFO.monthMidAp                         = zeros(1,1,'uint16');          
    APINFO.dayMidAp                           = zeros(1,1,'uint16');          
    APINFO.hourMidAp                          = zeros(1,1,'uint16');          
    APINFO.minuteMidAp                        = zeros(1,1,'uint16');          
    APINFO.secondMidAp                        = zeros(1,1,'uint16');          
    APINFO.flightTime                         = zeros(1,1,'uint32');          
    APINFO.flightWeek                         = zeros(1,1,'uint32');          
    APINFO.chirpRate                          = zeros(1,1,'single');          
    APINFO.xDistToStart                       = zeros(1,1,'single');          
    APINFO.momeasMode                         = zeros(1,1,'uint32');          
    APINFO.radarMode                          = zeros(1,1,'uint32');          
    APINFO.rfoa                               = zeros(1,1,'single');          
    APINFO.xVel                               = zeros(1,1,'double');          
    APINFO.yVel                               = zeros(1,1,'double');          
    APINFO.zVel                               = zeros(1,1,'double');          
    APINFO.apcLat                             = zeros(1,1,'double');          
    APINFO.apcLon                             = zeros(1,1,'double');          
    APINFO.apcAlt                             = zeros(1,1,'double');          
    APINFO.keepOutViol                        = zeros(1,1,'single');          
    APINFO.gimStopTwist                       = zeros(1,1,'single');          
    APINFO.gimStopTilt                        = zeros(1,1,'single');          
    APINFO.gimbalStopAz                       = zeros(1,1,'single');          
    APINFO.ver_maj                            = 1;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = read_APINFO_2_0(buffer) %#ok<DEFNU>
    APINFO.missionText                        = typecast(buffer(1:8),'uint8');
    APINFO.swVerNum                           = typecast(buffer(9:16),'uint8');
    APINFO.radarSerNum                        = typecast(buffer(17:20),'uint32');
    APINFO.phSource                           = typecast(buffer(21:24),'uint32');
    APINFO.phNameLen                          = typecast(buffer(25:26),'uint16');
    APINFO.phName                             = typecast(buffer(27:154),'uint8');
    APINFO.ctrFreq                            = typecast(buffer(155:158),'single');
    APINFO.wavelength                         = typecast(buffer(159:162),'single');
    APINFO.rxPolarization                     = typecast(buffer(163:166),'uint32');
    APINFO.txPolarization                     = typecast(buffer(167:170),'uint32');
    APINFO.azBeamWidth                        = typecast(buffer(171:174),'single');
    APINFO.elBeamWidth                        = typecast(buffer(175:178),'single');
    APINFO.grazingAngle                       = typecast(buffer(179:182),'single');
    APINFO.squintAngle                        = typecast(buffer(183:186),'single');
    APINFO.gta                                = typecast(buffer(187:190),'single');
    APINFO.rngToBeamCtr                       = typecast(buffer(191:194),'single');
    APINFO.desSquint                          = typecast(buffer(195:198),'single');
    APINFO.desRng                             = typecast(buffer(199:202),'single');
    APINFO.desGTA                             = typecast(buffer(203:206),'single');
    APINFO.antPhaseCtrBear                    = typecast(buffer(207:210),'single');
    APINFO.yearMidAp                          = typecast(buffer(211:212),'uint16');
    APINFO.monthMidAp                         = typecast(buffer(213:214),'uint16');
    APINFO.dayMidAp                           = typecast(buffer(215:216),'uint16');
    APINFO.hourMidAp                          = typecast(buffer(217:218),'uint16');
    APINFO.minuteMidAp                        = typecast(buffer(219:220),'uint16');
    APINFO.secondMidAp                        = typecast(buffer(221:222),'uint16');
    APINFO.flightTime                         = typecast(buffer(223:226),'uint32');
    APINFO.flightWeek                         = typecast(buffer(227:230),'uint32');
    APINFO.chirpRate                          = typecast(buffer(231:234),'single');
    APINFO.xDistToStart                       = typecast(buffer(235:238),'single');
    APINFO.momeasMode                         = typecast(buffer(239:242),'uint32');
    APINFO.radarMode                          = typecast(buffer(243:246),'uint32');
    APINFO.rfoa                               = typecast(buffer(247:250),'single');
    APINFO.xVel                               = typecast(buffer(251:258),'double');
    APINFO.yVel                               = typecast(buffer(259:266),'double');
    APINFO.zVel                               = typecast(buffer(267:274),'double');
    APINFO.apcLat                             = typecast(buffer(275:282),'double');
    APINFO.apcLon                             = typecast(buffer(283:290),'double');
    APINFO.apcAlt                             = typecast(buffer(291:298),'double');
    APINFO.keepOutViol                        = typecast(buffer(299:302),'single');
    APINFO.gimStopTwist                       = typecast(buffer(303:306),'single');
    APINFO.gimStopTilt                        = typecast(buffer(307:310),'single');
    APINFO.gimbalStopAz                       = typecast(buffer(311:314),'single');
    APINFO.apfdFactor                         = typecast(buffer(315:318),'int32');
    APINFO.ver_maj                            = 2;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = unmarshal_APINFO_2_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = char(APINFO.missionText.');
    APINFO.missionText(find(APINFO.missionText==0,1,'first'):end)=0;
    APINFO.swVerNum = char(APINFO.swVerNum.');
    APINFO.swVerNum(find(APINFO.swVerNum==0,1,'first'):end)=0;
    APINFO.phName = char(APINFO.phName.');
    APINFO.phName(find(APINFO.phName==0,1,'first'):end)=0;
end

function APINFO = marshal_APINFO_2_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = marshal_string(APINFO.missionText, 8);
    APINFO.swVerNum = marshal_string(APINFO.swVerNum, 8);
    APINFO.phName = marshal_string(APINFO.phName, 128);
end

function write_APINFO_2_0(fid, APINFO) %#ok<DEFNU>
    fwrite(fid, APINFO.missionText,                           'uint8');
    fwrite(fid, APINFO.swVerNum,                              'uint8');
    fwrite(fid, APINFO.radarSerNum,                           'uint32');
    fwrite(fid, APINFO.phSource,                              'uint32');
    fwrite(fid, APINFO.phNameLen,                             'uint16');
    fwrite(fid, APINFO.phName,                                'uint8');
    fwrite(fid, APINFO.ctrFreq,                               'single');
    fwrite(fid, APINFO.wavelength,                            'single');
    fwrite(fid, APINFO.rxPolarization,                        'uint32');
    fwrite(fid, APINFO.txPolarization,                        'uint32');
    fwrite(fid, APINFO.azBeamWidth,                           'single');
    fwrite(fid, APINFO.elBeamWidth,                           'single');
    fwrite(fid, APINFO.grazingAngle,                          'single');
    fwrite(fid, APINFO.squintAngle,                           'single');
    fwrite(fid, APINFO.gta,                                   'single');
    fwrite(fid, APINFO.rngToBeamCtr,                          'single');
    fwrite(fid, APINFO.desSquint,                             'single');
    fwrite(fid, APINFO.desRng,                                'single');
    fwrite(fid, APINFO.desGTA,                                'single');
    fwrite(fid, APINFO.antPhaseCtrBear,                       'single');
    fwrite(fid, APINFO.yearMidAp,                             'uint16');
    fwrite(fid, APINFO.monthMidAp,                            'uint16');
    fwrite(fid, APINFO.dayMidAp,                              'uint16');
    fwrite(fid, APINFO.hourMidAp,                             'uint16');
    fwrite(fid, APINFO.minuteMidAp,                           'uint16');
    fwrite(fid, APINFO.secondMidAp,                           'uint16');
    fwrite(fid, APINFO.flightTime,                            'uint32');
    fwrite(fid, APINFO.flightWeek,                            'uint32');
    fwrite(fid, APINFO.chirpRate,                             'single');
    fwrite(fid, APINFO.xDistToStart,                          'single');
    fwrite(fid, APINFO.momeasMode,                            'uint32');
    fwrite(fid, APINFO.radarMode,                             'uint32');
    fwrite(fid, APINFO.rfoa,                                  'single');
    fwrite(fid, APINFO.xVel,                                  'double');
    fwrite(fid, APINFO.yVel,                                  'double');
    fwrite(fid, APINFO.zVel,                                  'double');
    fwrite(fid, APINFO.apcLat,                                'double');
    fwrite(fid, APINFO.apcLon,                                'double');
    fwrite(fid, APINFO.apcAlt,                                'double');
    fwrite(fid, APINFO.keepOutViol,                           'single');
    fwrite(fid, APINFO.gimStopTwist,                          'single');
    fwrite(fid, APINFO.gimStopTilt,                           'single');
    fwrite(fid, APINFO.gimbalStopAz,                          'single');
    fwrite(fid, APINFO.apfdFactor,                            'int32');
end

function APINFO = init_APINFO_2_0() %#ok<DEFNU>
    APINFO.missionText                        = char(zeros(1,8));             
    APINFO.swVerNum                           = char(zeros(1,8));             
    APINFO.radarSerNum                        = zeros(1,1,'uint32');          
    APINFO.phSource                           = zeros(1,1,'uint32');          
    APINFO.phNameLen                          = zeros(1,1,'uint16');          
    APINFO.phName                             = char(zeros(1,128));           
    APINFO.ctrFreq                            = zeros(1,1,'single');          
    APINFO.wavelength                         = zeros(1,1,'single');          
    APINFO.rxPolarization                     = zeros(1,1,'uint32');          
    APINFO.txPolarization                     = zeros(1,1,'uint32');          
    APINFO.azBeamWidth                        = zeros(1,1,'single');          
    APINFO.elBeamWidth                        = zeros(1,1,'single');          
    APINFO.grazingAngle                       = zeros(1,1,'single');          
    APINFO.squintAngle                        = zeros(1,1,'single');          
    APINFO.gta                                = zeros(1,1,'single');          
    APINFO.rngToBeamCtr                       = zeros(1,1,'single');          
    APINFO.desSquint                          = zeros(1,1,'single');          
    APINFO.desRng                             = zeros(1,1,'single');          
    APINFO.desGTA                             = zeros(1,1,'single');          
    APINFO.antPhaseCtrBear                    = zeros(1,1,'single');          
    APINFO.yearMidAp                          = zeros(1,1,'uint16');          
    APINFO.monthMidAp                         = zeros(1,1,'uint16');          
    APINFO.dayMidAp                           = zeros(1,1,'uint16');          
    APINFO.hourMidAp                          = zeros(1,1,'uint16');          
    APINFO.minuteMidAp                        = zeros(1,1,'uint16');          
    APINFO.secondMidAp                        = zeros(1,1,'uint16');          
    APINFO.flightTime                         = zeros(1,1,'uint32');          
    APINFO.flightWeek                         = zeros(1,1,'uint32');          
    APINFO.chirpRate                          = zeros(1,1,'single');          
    APINFO.xDistToStart                       = zeros(1,1,'single');          
    APINFO.momeasMode                         = zeros(1,1,'uint32');          
    APINFO.radarMode                          = zeros(1,1,'uint32');          
    APINFO.rfoa                               = zeros(1,1,'single');          
    APINFO.xVel                               = zeros(1,1,'double');          
    APINFO.yVel                               = zeros(1,1,'double');          
    APINFO.zVel                               = zeros(1,1,'double');          
    APINFO.apcLat                             = zeros(1,1,'double');          
    APINFO.apcLon                             = zeros(1,1,'double');          
    APINFO.apcAlt                             = zeros(1,1,'double');          
    APINFO.keepOutViol                        = zeros(1,1,'single');          
    APINFO.gimStopTwist                       = zeros(1,1,'single');          
    APINFO.gimStopTilt                        = zeros(1,1,'single');          
    APINFO.gimbalStopAz                       = zeros(1,1,'single');          
    APINFO.apfdFactor                         = zeros(1,1,'int32');           
    APINFO.ver_maj                            = 2;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = read_APINFO_3_0(buffer) %#ok<DEFNU>
    APINFO.missionText                        = typecast(buffer(1:8),'uint8');
    APINFO.swVerNum                           = typecast(buffer(9:16),'uint8');
    APINFO.radarSerNum                        = typecast(buffer(17:20),'uint32');
    APINFO.phSource                           = typecast(buffer(21:24),'uint32');
    APINFO.phNameLen                          = typecast(buffer(25:26),'uint16');
    APINFO.phName                             = typecast(buffer(27:154),'uint8');
    APINFO.ctrFreq                            = typecast(buffer(155:158),'single');
    APINFO.wavelength                         = typecast(buffer(159:162),'single');
    APINFO.rxPolarization                     = typecast(buffer(163:166),'uint32');
    APINFO.txPolarization                     = typecast(buffer(167:170),'uint32');
    APINFO.azBeamWidth                        = typecast(buffer(171:174),'single');
    APINFO.elBeamWidth                        = typecast(buffer(175:178),'single');
    APINFO.grazingAngle                       = typecast(buffer(179:182),'single');
    APINFO.squintAngle                        = typecast(buffer(183:186),'single');
    APINFO.gta                                = typecast(buffer(187:190),'single');
    APINFO.rngToBeamCtr                       = typecast(buffer(191:194),'single');
    APINFO.desSquint                          = typecast(buffer(195:198),'single');
    APINFO.desRng                             = typecast(buffer(199:202),'single');
    APINFO.desGTA                             = typecast(buffer(203:206),'single');
    APINFO.antPhaseCtrBear                    = typecast(buffer(207:210),'single');
    APINFO.yearMidAp                          = typecast(buffer(211:212),'uint16');
    APINFO.monthMidAp                         = typecast(buffer(213:214),'uint16');
    APINFO.dayMidAp                           = typecast(buffer(215:216),'uint16');
    APINFO.hourMidAp                          = typecast(buffer(217:218),'uint16');
    APINFO.minuteMidAp                        = typecast(buffer(219:220),'uint16');
    APINFO.secondMidAp                        = typecast(buffer(221:222),'uint16');
    APINFO.flightTime                         = typecast(buffer(223:226),'uint32');
    APINFO.flightWeek                         = typecast(buffer(227:230),'uint32');
    APINFO.chirpRate                          = typecast(buffer(231:234),'single');
    APINFO.xDistToStart                       = typecast(buffer(235:238),'single');
    APINFO.momeasMode                         = typecast(buffer(239:242),'uint32');
    APINFO.radarMode                          = typecast(buffer(243:246),'uint32');
    APINFO.rfoa                               = typecast(buffer(247:250),'single');
    APINFO.xVel                               = typecast(buffer(251:258),'double');
    APINFO.yVel                               = typecast(buffer(259:266),'double');
    APINFO.zVel                               = typecast(buffer(267:274),'double');
    APINFO.apcLat                             = typecast(buffer(275:282),'double');
    APINFO.apcLon                             = typecast(buffer(283:290),'double');
    APINFO.apcAlt                             = typecast(buffer(291:298),'double');
    APINFO.keepOutViol                        = typecast(buffer(299:302),'single');
    APINFO.gimStopTwist                       = typecast(buffer(303:306),'single');
    APINFO.gimStopTilt                        = typecast(buffer(307:310),'single');
    APINFO.gimbalStopAz                       = typecast(buffer(311:314),'single');
    APINFO.apfdFactor                         = typecast(buffer(315:318),'int32');
    APINFO.fastTimeSamples                    = typecast(buffer(319:322),'uint32');
    APINFO.adSampleFreq                       = typecast(buffer(323:326),'single');
    APINFO.apertureTime                       = typecast(buffer(327:330),'single');
    APINFO.numPhaseHistories                  = typecast(buffer(331:334),'uint32');
    APINFO.ver_maj                            = 3;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = unmarshal_APINFO_3_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = char(APINFO.missionText.');
    APINFO.missionText(find(APINFO.missionText==0,1,'first'):end)=0;
    APINFO.swVerNum = char(APINFO.swVerNum.');
    APINFO.swVerNum(find(APINFO.swVerNum==0,1,'first'):end)=0;
    APINFO.phName = char(APINFO.phName.');
    APINFO.phName(find(APINFO.phName==0,1,'first'):end)=0;
end

function APINFO = marshal_APINFO_3_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = marshal_string(APINFO.missionText, 8);
    APINFO.swVerNum = marshal_string(APINFO.swVerNum, 8);
    APINFO.phName = marshal_string(APINFO.phName, 128);
end

function write_APINFO_3_0(fid, APINFO) %#ok<DEFNU>
    fwrite(fid, APINFO.missionText,                           'uint8');
    fwrite(fid, APINFO.swVerNum,                              'uint8');
    fwrite(fid, APINFO.radarSerNum,                           'uint32');
    fwrite(fid, APINFO.phSource,                              'uint32');
    fwrite(fid, APINFO.phNameLen,                             'uint16');
    fwrite(fid, APINFO.phName,                                'uint8');
    fwrite(fid, APINFO.ctrFreq,                               'single');
    fwrite(fid, APINFO.wavelength,                            'single');
    fwrite(fid, APINFO.rxPolarization,                        'uint32');
    fwrite(fid, APINFO.txPolarization,                        'uint32');
    fwrite(fid, APINFO.azBeamWidth,                           'single');
    fwrite(fid, APINFO.elBeamWidth,                           'single');
    fwrite(fid, APINFO.grazingAngle,                          'single');
    fwrite(fid, APINFO.squintAngle,                           'single');
    fwrite(fid, APINFO.gta,                                   'single');
    fwrite(fid, APINFO.rngToBeamCtr,                          'single');
    fwrite(fid, APINFO.desSquint,                             'single');
    fwrite(fid, APINFO.desRng,                                'single');
    fwrite(fid, APINFO.desGTA,                                'single');
    fwrite(fid, APINFO.antPhaseCtrBear,                       'single');
    fwrite(fid, APINFO.yearMidAp,                             'uint16');
    fwrite(fid, APINFO.monthMidAp,                            'uint16');
    fwrite(fid, APINFO.dayMidAp,                              'uint16');
    fwrite(fid, APINFO.hourMidAp,                             'uint16');
    fwrite(fid, APINFO.minuteMidAp,                           'uint16');
    fwrite(fid, APINFO.secondMidAp,                           'uint16');
    fwrite(fid, APINFO.flightTime,                            'uint32');
    fwrite(fid, APINFO.flightWeek,                            'uint32');
    fwrite(fid, APINFO.chirpRate,                             'single');
    fwrite(fid, APINFO.xDistToStart,                          'single');
    fwrite(fid, APINFO.momeasMode,                            'uint32');
    fwrite(fid, APINFO.radarMode,                             'uint32');
    fwrite(fid, APINFO.rfoa,                                  'single');
    fwrite(fid, APINFO.xVel,                                  'double');
    fwrite(fid, APINFO.yVel,                                  'double');
    fwrite(fid, APINFO.zVel,                                  'double');
    fwrite(fid, APINFO.apcLat,                                'double');
    fwrite(fid, APINFO.apcLon,                                'double');
    fwrite(fid, APINFO.apcAlt,                                'double');
    fwrite(fid, APINFO.keepOutViol,                           'single');
    fwrite(fid, APINFO.gimStopTwist,                          'single');
    fwrite(fid, APINFO.gimStopTilt,                           'single');
    fwrite(fid, APINFO.gimbalStopAz,                          'single');
    fwrite(fid, APINFO.apfdFactor,                            'int32');
    fwrite(fid, APINFO.fastTimeSamples,                       'uint32');
    fwrite(fid, APINFO.adSampleFreq,                          'single');
    fwrite(fid, APINFO.apertureTime,                          'single');
    fwrite(fid, APINFO.numPhaseHistories,                     'uint32');
end

function APINFO = init_APINFO_3_0() %#ok<DEFNU>
    APINFO.missionText                        = char(zeros(1,8));             
    APINFO.swVerNum                           = char(zeros(1,8));             
    APINFO.radarSerNum                        = zeros(1,1,'uint32');          
    APINFO.phSource                           = zeros(1,1,'uint32');          
    APINFO.phNameLen                          = zeros(1,1,'uint16');          
    APINFO.phName                             = char(zeros(1,128));           
    APINFO.ctrFreq                            = zeros(1,1,'single');          
    APINFO.wavelength                         = zeros(1,1,'single');          
    APINFO.rxPolarization                     = zeros(1,1,'uint32');          
    APINFO.txPolarization                     = zeros(1,1,'uint32');          
    APINFO.azBeamWidth                        = zeros(1,1,'single');          
    APINFO.elBeamWidth                        = zeros(1,1,'single');          
    APINFO.grazingAngle                       = zeros(1,1,'single');          
    APINFO.squintAngle                        = zeros(1,1,'single');          
    APINFO.gta                                = zeros(1,1,'single');          
    APINFO.rngToBeamCtr                       = zeros(1,1,'single');          
    APINFO.desSquint                          = zeros(1,1,'single');          
    APINFO.desRng                             = zeros(1,1,'single');          
    APINFO.desGTA                             = zeros(1,1,'single');          
    APINFO.antPhaseCtrBear                    = zeros(1,1,'single');          
    APINFO.yearMidAp                          = zeros(1,1,'uint16');          
    APINFO.monthMidAp                         = zeros(1,1,'uint16');          
    APINFO.dayMidAp                           = zeros(1,1,'uint16');          
    APINFO.hourMidAp                          = zeros(1,1,'uint16');          
    APINFO.minuteMidAp                        = zeros(1,1,'uint16');          
    APINFO.secondMidAp                        = zeros(1,1,'uint16');          
    APINFO.flightTime                         = zeros(1,1,'uint32');          
    APINFO.flightWeek                         = zeros(1,1,'uint32');          
    APINFO.chirpRate                          = zeros(1,1,'single');          
    APINFO.xDistToStart                       = zeros(1,1,'single');          
    APINFO.momeasMode                         = zeros(1,1,'uint32');          
    APINFO.radarMode                          = zeros(1,1,'uint32');          
    APINFO.rfoa                               = zeros(1,1,'single');          
    APINFO.xVel                               = zeros(1,1,'double');          
    APINFO.yVel                               = zeros(1,1,'double');          
    APINFO.zVel                               = zeros(1,1,'double');          
    APINFO.apcLat                             = zeros(1,1,'double');          
    APINFO.apcLon                             = zeros(1,1,'double');          
    APINFO.apcAlt                             = zeros(1,1,'double');          
    APINFO.keepOutViol                        = zeros(1,1,'single');          
    APINFO.gimStopTwist                       = zeros(1,1,'single');          
    APINFO.gimStopTilt                        = zeros(1,1,'single');          
    APINFO.gimbalStopAz                       = zeros(1,1,'single');          
    APINFO.apfdFactor                         = zeros(1,1,'int32');           
    APINFO.fastTimeSamples                    = zeros(1,1,'uint32');          
    APINFO.adSampleFreq                       = zeros(1,1,'single');          
    APINFO.apertureTime                       = zeros(1,1,'single');          
    APINFO.numPhaseHistories                  = zeros(1,1,'uint32');          
    APINFO.ver_maj                            = 3;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = read_APINFO_4_0(buffer) %#ok<DEFNU>
    APINFO.missionText                        = typecast(buffer(1:50),'uint8');
    APINFO.swVerNum                           = typecast(buffer(51:100),'uint8');
    APINFO.radarSerNum                        = typecast(buffer(101:104),'uint32');
    APINFO.phSource                           = typecast(buffer(105:108),'uint32');
    APINFO.phNameLen                          = typecast(buffer(109:110),'uint16');
    APINFO.phName                             = typecast(buffer(111:238),'uint8');
    APINFO.ctrFreq                            = typecast(buffer(239:242),'single');
    APINFO.wavelength                         = typecast(buffer(243:246),'single');
    APINFO.rxPolarization                     = typecast(buffer(247:250),'uint32');
    APINFO.txPolarization                     = typecast(buffer(251:254),'uint32');
    APINFO.azBeamWidth                        = typecast(buffer(255:258),'single');
    APINFO.elBeamWidth                        = typecast(buffer(259:262),'single');
    APINFO.grazingAngle                       = typecast(buffer(263:266),'single');
    APINFO.squintAngle                        = typecast(buffer(267:270),'single');
    APINFO.gta                                = typecast(buffer(271:274),'single');
    APINFO.rngToBeamCtr                       = typecast(buffer(275:278),'single');
    APINFO.desSquint                          = typecast(buffer(279:282),'single');
    APINFO.desRng                             = typecast(buffer(283:286),'single');
    APINFO.desGTA                             = typecast(buffer(287:290),'single');
    APINFO.antPhaseCtrBear                    = typecast(buffer(291:294),'single');
    APINFO.yearMidAp                          = typecast(buffer(295:296),'uint16');
    APINFO.monthMidAp                         = typecast(buffer(297:298),'uint16');
    APINFO.dayMidAp                           = typecast(buffer(299:300),'uint16');
    APINFO.hourMidAp                          = typecast(buffer(301:302),'uint16');
    APINFO.minuteMidAp                        = typecast(buffer(303:304),'uint16');
    APINFO.secondMidAp                        = typecast(buffer(305:306),'uint16');
    APINFO.flightTime                         = typecast(buffer(307:310),'uint32');
    APINFO.flightWeek                         = typecast(buffer(311:314),'uint32');
    APINFO.chirpRate                          = typecast(buffer(315:318),'single');
    APINFO.xDistToStart                       = typecast(buffer(319:322),'single');
    APINFO.momeasMode                         = typecast(buffer(323:326),'uint32');
    APINFO.radarMode                          = typecast(buffer(327:330),'uint32');
    APINFO.rfoa                               = typecast(buffer(331:334),'single');
    APINFO.xVel                               = typecast(buffer(335:342),'double');
    APINFO.yVel                               = typecast(buffer(343:350),'double');
    APINFO.zVel                               = typecast(buffer(351:358),'double');
    APINFO.apcLat                             = typecast(buffer(359:366),'double');
    APINFO.apcLon                             = typecast(buffer(367:374),'double');
    APINFO.apcAlt                             = typecast(buffer(375:382),'double');
    APINFO.keepOutViol                        = typecast(buffer(383:386),'single');
    APINFO.gimStopTwist                       = typecast(buffer(387:390),'single');
    APINFO.gimStopTilt                        = typecast(buffer(391:394),'single');
    APINFO.gimbalStopAz                       = typecast(buffer(395:398),'single');
    APINFO.apfdFactor                         = typecast(buffer(399:402),'int32');
    APINFO.fastTimeSamples                    = typecast(buffer(403:406),'uint32');
    APINFO.adSampleFreq                       = typecast(buffer(407:410),'single');
    APINFO.apertureTime                       = typecast(buffer(411:414),'single');
    APINFO.numPhaseHistories                  = typecast(buffer(415:418),'uint32');
    APINFO.ver_maj                            = 4;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = unmarshal_APINFO_4_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = char(APINFO.missionText.');
    APINFO.missionText(find(APINFO.missionText==0,1,'first'):end)=0;
    APINFO.swVerNum = char(APINFO.swVerNum.');
    APINFO.swVerNum(find(APINFO.swVerNum==0,1,'first'):end)=0;
    APINFO.phName = char(APINFO.phName.');
    APINFO.phName(find(APINFO.phName==0,1,'first'):end)=0;
end

function APINFO = marshal_APINFO_4_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = marshal_string(APINFO.missionText, 50);
    APINFO.swVerNum = marshal_string(APINFO.swVerNum, 50);
    APINFO.phName = marshal_string(APINFO.phName, 128);
end

function write_APINFO_4_0(fid, APINFO) %#ok<DEFNU>
    fwrite(fid, APINFO.missionText,                           'uint8');
    fwrite(fid, APINFO.swVerNum,                              'uint8');
    fwrite(fid, APINFO.radarSerNum,                           'uint32');
    fwrite(fid, APINFO.phSource,                              'uint32');
    fwrite(fid, APINFO.phNameLen,                             'uint16');
    fwrite(fid, APINFO.phName,                                'uint8');
    fwrite(fid, APINFO.ctrFreq,                               'single');
    fwrite(fid, APINFO.wavelength,                            'single');
    fwrite(fid, APINFO.rxPolarization,                        'uint32');
    fwrite(fid, APINFO.txPolarization,                        'uint32');
    fwrite(fid, APINFO.azBeamWidth,                           'single');
    fwrite(fid, APINFO.elBeamWidth,                           'single');
    fwrite(fid, APINFO.grazingAngle,                          'single');
    fwrite(fid, APINFO.squintAngle,                           'single');
    fwrite(fid, APINFO.gta,                                   'single');
    fwrite(fid, APINFO.rngToBeamCtr,                          'single');
    fwrite(fid, APINFO.desSquint,                             'single');
    fwrite(fid, APINFO.desRng,                                'single');
    fwrite(fid, APINFO.desGTA,                                'single');
    fwrite(fid, APINFO.antPhaseCtrBear,                       'single');
    fwrite(fid, APINFO.yearMidAp,                             'uint16');
    fwrite(fid, APINFO.monthMidAp,                            'uint16');
    fwrite(fid, APINFO.dayMidAp,                              'uint16');
    fwrite(fid, APINFO.hourMidAp,                             'uint16');
    fwrite(fid, APINFO.minuteMidAp,                           'uint16');
    fwrite(fid, APINFO.secondMidAp,                           'uint16');
    fwrite(fid, APINFO.flightTime,                            'uint32');
    fwrite(fid, APINFO.flightWeek,                            'uint32');
    fwrite(fid, APINFO.chirpRate,                             'single');
    fwrite(fid, APINFO.xDistToStart,                          'single');
    fwrite(fid, APINFO.momeasMode,                            'uint32');
    fwrite(fid, APINFO.radarMode,                             'uint32');
    fwrite(fid, APINFO.rfoa,                                  'single');
    fwrite(fid, APINFO.xVel,                                  'double');
    fwrite(fid, APINFO.yVel,                                  'double');
    fwrite(fid, APINFO.zVel,                                  'double');
    fwrite(fid, APINFO.apcLat,                                'double');
    fwrite(fid, APINFO.apcLon,                                'double');
    fwrite(fid, APINFO.apcAlt,                                'double');
    fwrite(fid, APINFO.keepOutViol,                           'single');
    fwrite(fid, APINFO.gimStopTwist,                          'single');
    fwrite(fid, APINFO.gimStopTilt,                           'single');
    fwrite(fid, APINFO.gimbalStopAz,                          'single');
    fwrite(fid, APINFO.apfdFactor,                            'int32');
    fwrite(fid, APINFO.fastTimeSamples,                       'uint32');
    fwrite(fid, APINFO.adSampleFreq,                          'single');
    fwrite(fid, APINFO.apertureTime,                          'single');
    fwrite(fid, APINFO.numPhaseHistories,                     'uint32');
end

function APINFO = init_APINFO_4_0() %#ok<DEFNU>
    APINFO.missionText                        = char(zeros(1,50));            
    APINFO.swVerNum                           = char(zeros(1,50));            
    APINFO.radarSerNum                        = zeros(1,1,'uint32');          
    APINFO.phSource                           = zeros(1,1,'uint32');          
    APINFO.phNameLen                          = zeros(1,1,'uint16');          
    APINFO.phName                             = char(zeros(1,128));           
    APINFO.ctrFreq                            = zeros(1,1,'single');          
    APINFO.wavelength                         = zeros(1,1,'single');          
    APINFO.rxPolarization                     = zeros(1,1,'uint32');          
    APINFO.txPolarization                     = zeros(1,1,'uint32');          
    APINFO.azBeamWidth                        = zeros(1,1,'single');          
    APINFO.elBeamWidth                        = zeros(1,1,'single');          
    APINFO.grazingAngle                       = zeros(1,1,'single');          
    APINFO.squintAngle                        = zeros(1,1,'single');          
    APINFO.gta                                = zeros(1,1,'single');          
    APINFO.rngToBeamCtr                       = zeros(1,1,'single');          
    APINFO.desSquint                          = zeros(1,1,'single');          
    APINFO.desRng                             = zeros(1,1,'single');          
    APINFO.desGTA                             = zeros(1,1,'single');          
    APINFO.antPhaseCtrBear                    = zeros(1,1,'single');          
    APINFO.yearMidAp                          = zeros(1,1,'uint16');          
    APINFO.monthMidAp                         = zeros(1,1,'uint16');          
    APINFO.dayMidAp                           = zeros(1,1,'uint16');          
    APINFO.hourMidAp                          = zeros(1,1,'uint16');          
    APINFO.minuteMidAp                        = zeros(1,1,'uint16');          
    APINFO.secondMidAp                        = zeros(1,1,'uint16');          
    APINFO.flightTime                         = zeros(1,1,'uint32');          
    APINFO.flightWeek                         = zeros(1,1,'uint32');          
    APINFO.chirpRate                          = zeros(1,1,'single');          
    APINFO.xDistToStart                       = zeros(1,1,'single');          
    APINFO.momeasMode                         = zeros(1,1,'uint32');          
    APINFO.radarMode                          = zeros(1,1,'uint32');          
    APINFO.rfoa                               = zeros(1,1,'single');          
    APINFO.xVel                               = zeros(1,1,'double');          
    APINFO.yVel                               = zeros(1,1,'double');          
    APINFO.zVel                               = zeros(1,1,'double');          
    APINFO.apcLat                             = zeros(1,1,'double');          
    APINFO.apcLon                             = zeros(1,1,'double');          
    APINFO.apcAlt                             = zeros(1,1,'double');          
    APINFO.keepOutViol                        = zeros(1,1,'single');          
    APINFO.gimStopTwist                       = zeros(1,1,'single');          
    APINFO.gimStopTilt                        = zeros(1,1,'single');          
    APINFO.gimbalStopAz                       = zeros(1,1,'single');          
    APINFO.apfdFactor                         = zeros(1,1,'int32');           
    APINFO.fastTimeSamples                    = zeros(1,1,'uint32');          
    APINFO.adSampleFreq                       = zeros(1,1,'single');          
    APINFO.apertureTime                       = zeros(1,1,'single');          
    APINFO.numPhaseHistories                  = zeros(1,1,'uint32');          
    APINFO.ver_maj                            = 4;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = read_APINFO_5_0(buffer) %#ok<DEFNU>
    APINFO.missionText                        = typecast(buffer(1:50),'uint8');
    APINFO.swVerNum                           = typecast(buffer(51:100),'uint8');
    APINFO.radarSerNum                        = typecast(buffer(101:104),'uint32');
    APINFO.phSource                           = typecast(buffer(105:108),'uint32');
    APINFO.phNameLen                          = typecast(buffer(109:110),'uint16');
    APINFO.phName                             = typecast(buffer(111:238),'uint8');
    APINFO.ctrFreq                            = typecast(buffer(239:242),'single');
    APINFO.wavelength                         = typecast(buffer(243:246),'single');
    APINFO.rxPolarization                     = typecast(buffer(247:250),'uint32');
    APINFO.txPolarization                     = typecast(buffer(251:254),'uint32');
    APINFO.azBeamWidth                        = typecast(buffer(255:258),'single');
    APINFO.elBeamWidth                        = typecast(buffer(259:262),'single');
    APINFO.grazingAngle                       = typecast(buffer(263:266),'single');
    APINFO.squintAngle                        = typecast(buffer(267:270),'single');
    APINFO.gta                                = typecast(buffer(271:274),'single');
    APINFO.rngToBeamCtr                       = typecast(buffer(275:278),'single');
    APINFO.desSquint                          = typecast(buffer(279:282),'single');
    APINFO.desRng                             = typecast(buffer(283:286),'single');
    APINFO.desGTA                             = typecast(buffer(287:290),'single');
    APINFO.antPhaseCtrBear                    = typecast(buffer(291:294),'single');
    APINFO.yearMidAp                          = typecast(buffer(295:296),'uint16');
    APINFO.monthMidAp                         = typecast(buffer(297:298),'uint16');
    APINFO.dayMidAp                           = typecast(buffer(299:300),'uint16');
    APINFO.hourMidAp                          = typecast(buffer(301:302),'uint16');
    APINFO.minuteMidAp                        = typecast(buffer(303:304),'uint16');
    APINFO.secondMidAp                        = typecast(buffer(305:306),'uint16');
    APINFO.flightTime                         = typecast(buffer(307:310),'uint32');
    APINFO.flightWeek                         = typecast(buffer(311:314),'uint32');
    APINFO.chirpRate                          = typecast(buffer(315:318),'single');
    APINFO.xDistToStart                       = typecast(buffer(319:322),'single');
    APINFO.momeasMode                         = typecast(buffer(323:326),'uint32');
    APINFO.radarMode                          = typecast(buffer(327:330),'uint32');
    APINFO.rfoa                               = typecast(buffer(331:334),'single');
    APINFO.xVel                               = typecast(buffer(335:342),'double');
    APINFO.yVel                               = typecast(buffer(343:350),'double');
    APINFO.zVel                               = typecast(buffer(351:358),'double');
    APINFO.apcLat                             = typecast(buffer(359:366),'double');
    APINFO.apcLon                             = typecast(buffer(367:374),'double');
    APINFO.apcAlt                             = typecast(buffer(375:382),'double');
    APINFO.keepOutViol                        = typecast(buffer(383:386),'single');
    APINFO.gimStopTwist                       = typecast(buffer(387:390),'single');
    APINFO.gimStopTilt                        = typecast(buffer(391:394),'single');
    APINFO.gimbalStopAz                       = typecast(buffer(395:398),'single');
    APINFO.apfdFactor                         = typecast(buffer(399:402),'int32');
    APINFO.fastTimeSamples                    = typecast(buffer(403:406),'uint32');
    APINFO.adSampleFreq                       = typecast(buffer(407:410),'single');
    APINFO.apertureTime                       = typecast(buffer(411:414),'single');
    APINFO.numPhaseHistories                  = typecast(buffer(415:418),'uint32');
    APINFO.lightSpeed                         = typecast(buffer(419:426),'double');
    APINFO.ver_maj                            = 5;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = unmarshal_APINFO_5_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = char(APINFO.missionText.');
    APINFO.missionText(find(APINFO.missionText==0,1,'first'):end)=0;
    APINFO.swVerNum = char(APINFO.swVerNum.');
    APINFO.swVerNum(find(APINFO.swVerNum==0,1,'first'):end)=0;
    APINFO.phName = char(APINFO.phName.');
    APINFO.phName(find(APINFO.phName==0,1,'first'):end)=0;
end

function APINFO = marshal_APINFO_5_0(APINFO) %#ok<DEFNU>
    APINFO.missionText = marshal_string(APINFO.missionText, 50);
    APINFO.swVerNum = marshal_string(APINFO.swVerNum, 50);
    APINFO.phName = marshal_string(APINFO.phName, 128);
end

function write_APINFO_5_0(fid, APINFO) %#ok<DEFNU>
    fwrite(fid, APINFO.missionText,                           'uint8');
    fwrite(fid, APINFO.swVerNum,                              'uint8');
    fwrite(fid, APINFO.radarSerNum,                           'uint32');
    fwrite(fid, APINFO.phSource,                              'uint32');
    fwrite(fid, APINFO.phNameLen,                             'uint16');
    fwrite(fid, APINFO.phName,                                'uint8');
    fwrite(fid, APINFO.ctrFreq,                               'single');
    fwrite(fid, APINFO.wavelength,                            'single');
    fwrite(fid, APINFO.rxPolarization,                        'uint32');
    fwrite(fid, APINFO.txPolarization,                        'uint32');
    fwrite(fid, APINFO.azBeamWidth,                           'single');
    fwrite(fid, APINFO.elBeamWidth,                           'single');
    fwrite(fid, APINFO.grazingAngle,                          'single');
    fwrite(fid, APINFO.squintAngle,                           'single');
    fwrite(fid, APINFO.gta,                                   'single');
    fwrite(fid, APINFO.rngToBeamCtr,                          'single');
    fwrite(fid, APINFO.desSquint,                             'single');
    fwrite(fid, APINFO.desRng,                                'single');
    fwrite(fid, APINFO.desGTA,                                'single');
    fwrite(fid, APINFO.antPhaseCtrBear,                       'single');
    fwrite(fid, APINFO.yearMidAp,                             'uint16');
    fwrite(fid, APINFO.monthMidAp,                            'uint16');
    fwrite(fid, APINFO.dayMidAp,                              'uint16');
    fwrite(fid, APINFO.hourMidAp,                             'uint16');
    fwrite(fid, APINFO.minuteMidAp,                           'uint16');
    fwrite(fid, APINFO.secondMidAp,                           'uint16');
    fwrite(fid, APINFO.flightTime,                            'uint32');
    fwrite(fid, APINFO.flightWeek,                            'uint32');
    fwrite(fid, APINFO.chirpRate,                             'single');
    fwrite(fid, APINFO.xDistToStart,                          'single');
    fwrite(fid, APINFO.momeasMode,                            'uint32');
    fwrite(fid, APINFO.radarMode,                             'uint32');
    fwrite(fid, APINFO.rfoa,                                  'single');
    fwrite(fid, APINFO.xVel,                                  'double');
    fwrite(fid, APINFO.yVel,                                  'double');
    fwrite(fid, APINFO.zVel,                                  'double');
    fwrite(fid, APINFO.apcLat,                                'double');
    fwrite(fid, APINFO.apcLon,                                'double');
    fwrite(fid, APINFO.apcAlt,                                'double');
    fwrite(fid, APINFO.keepOutViol,                           'single');
    fwrite(fid, APINFO.gimStopTwist,                          'single');
    fwrite(fid, APINFO.gimStopTilt,                           'single');
    fwrite(fid, APINFO.gimbalStopAz,                          'single');
    fwrite(fid, APINFO.apfdFactor,                            'int32');
    fwrite(fid, APINFO.fastTimeSamples,                       'uint32');
    fwrite(fid, APINFO.adSampleFreq,                          'single');
    fwrite(fid, APINFO.apertureTime,                          'single');
    fwrite(fid, APINFO.numPhaseHistories,                     'uint32');
    fwrite(fid, APINFO.lightSpeed,                            'double');
end

function APINFO = init_APINFO_5_0() %#ok<DEFNU>
    APINFO.missionText                        = char(zeros(1,50));            
    APINFO.swVerNum                           = char(zeros(1,50));            
    APINFO.radarSerNum                        = zeros(1,1,'uint32');          
    APINFO.phSource                           = zeros(1,1,'uint32');          
    APINFO.phNameLen                          = zeros(1,1,'uint16');          
    APINFO.phName                             = char(zeros(1,128));           
    APINFO.ctrFreq                            = zeros(1,1,'single');          
    APINFO.wavelength                         = zeros(1,1,'single');          
    APINFO.rxPolarization                     = zeros(1,1,'uint32');          
    APINFO.txPolarization                     = zeros(1,1,'uint32');          
    APINFO.azBeamWidth                        = zeros(1,1,'single');          
    APINFO.elBeamWidth                        = zeros(1,1,'single');          
    APINFO.grazingAngle                       = zeros(1,1,'single');          
    APINFO.squintAngle                        = zeros(1,1,'single');          
    APINFO.gta                                = zeros(1,1,'single');          
    APINFO.rngToBeamCtr                       = zeros(1,1,'single');          
    APINFO.desSquint                          = zeros(1,1,'single');          
    APINFO.desRng                             = zeros(1,1,'single');          
    APINFO.desGTA                             = zeros(1,1,'single');          
    APINFO.antPhaseCtrBear                    = zeros(1,1,'single');          
    APINFO.yearMidAp                          = zeros(1,1,'uint16');          
    APINFO.monthMidAp                         = zeros(1,1,'uint16');          
    APINFO.dayMidAp                           = zeros(1,1,'uint16');          
    APINFO.hourMidAp                          = zeros(1,1,'uint16');          
    APINFO.minuteMidAp                        = zeros(1,1,'uint16');          
    APINFO.secondMidAp                        = zeros(1,1,'uint16');          
    APINFO.flightTime                         = zeros(1,1,'uint32');          
    APINFO.flightWeek                         = zeros(1,1,'uint32');          
    APINFO.chirpRate                          = zeros(1,1,'single');          
    APINFO.xDistToStart                       = zeros(1,1,'single');          
    APINFO.momeasMode                         = zeros(1,1,'uint32');          
    APINFO.radarMode                          = zeros(1,1,'uint32');          
    APINFO.rfoa                               = zeros(1,1,'single');          
    APINFO.xVel                               = zeros(1,1,'double');          
    APINFO.yVel                               = zeros(1,1,'double');          
    APINFO.zVel                               = zeros(1,1,'double');          
    APINFO.apcLat                             = zeros(1,1,'double');          
    APINFO.apcLon                             = zeros(1,1,'double');          
    APINFO.apcAlt                             = zeros(1,1,'double');          
    APINFO.keepOutViol                        = zeros(1,1,'single');          
    APINFO.gimStopTwist                       = zeros(1,1,'single');          
    APINFO.gimStopTilt                        = zeros(1,1,'single');          
    APINFO.gimbalStopAz                       = zeros(1,1,'single');          
    APINFO.apfdFactor                         = zeros(1,1,'int32');           
    APINFO.fastTimeSamples                    = zeros(1,1,'uint32');          
    APINFO.adSampleFreq                       = zeros(1,1,'single');          
    APINFO.apertureTime                       = zeros(1,1,'single');          
    APINFO.numPhaseHistories                  = zeros(1,1,'uint32');          
    APINFO.lightSpeed                         = zeros(1,1,'double');          
    APINFO.ver_maj                            = 5;                            
    APINFO.ver_min                            = 0;                            
end

function APINFO = read_APINFO_5_1(buffer) %#ok<DEFNU>
    APINFO.missionText                        = typecast(buffer(1:50),'uint8');
    APINFO.swVerNum                           = typecast(buffer(51:100),'uint8');
    APINFO.radarSerNum                        = typecast(buffer(101:104),'uint32');
    APINFO.phSource                           = typecast(buffer(105:108),'uint32');
    APINFO.phNameLen                          = typecast(buffer(109:110),'uint16');
    APINFO.phName                             = typecast(buffer(111:238),'uint8');
    APINFO.ctrFreq                            = typecast(buffer(239:242),'single');
    APINFO.wavelength                         = typecast(buffer(243:246),'single');
    APINFO.rxPolarization                     = typecast(buffer(247:250),'uint32');
    APINFO.txPolarization                     = typecast(buffer(251:254),'uint32');
    APINFO.azBeamWidth                        = typecast(buffer(255:258),'single');
    APINFO.elBeamWidth                        = typecast(buffer(259:262),'single');
    APINFO.grazingAngle                       = typecast(buffer(263:266),'single');
    APINFO.squintAngle                        = typecast(buffer(267:270),'single');
    APINFO.gta                                = typecast(buffer(271:274),'single');
    APINFO.rngToBeamCtr                       = typecast(buffer(275:278),'single');
    APINFO.desSquint                          = typecast(buffer(279:282),'single');
    APINFO.desRng                             = typecast(buffer(283:286),'single');
    APINFO.desGTA                             = typecast(buffer(287:290),'single');
    APINFO.antPhaseCtrBear                    = typecast(buffer(291:294),'single');
    APINFO.yearMidAp                          = typecast(buffer(295:296),'uint16');
    APINFO.monthMidAp                         = typecast(buffer(297:298),'uint16');
    APINFO.dayMidAp                           = typecast(buffer(299:300),'uint16');
    APINFO.hourMidAp                          = typecast(buffer(301:302),'uint16');
    APINFO.minuteMidAp                        = typecast(buffer(303:304),'uint16');
    APINFO.secondMidAp                        = typecast(buffer(305:306),'uint16');
    APINFO.flightTime                         = typecast(buffer(307:310),'uint32');
    APINFO.flightWeek                         = typecast(buffer(311:314),'uint32');
    APINFO.chirpRate                          = typecast(buffer(315:318),'single');
    APINFO.xDistToStart                       = typecast(buffer(319:322),'single');
    APINFO.momeasMode                         = typecast(buffer(323:326),'uint32');
    APINFO.radarMode                          = typecast(buffer(327:330),'uint32');
    APINFO.rfoa                               = typecast(buffer(331:334),'single');
    APINFO.xVel                               = typecast(buffer(335:342),'double');
    APINFO.yVel                               = typecast(buffer(343:350),'double');
    APINFO.zVel                               = typecast(buffer(351:358),'double');
    APINFO.apcLat                             = typecast(buffer(359:366),'double');
    APINFO.apcLon                             = typecast(buffer(367:374),'double');
    APINFO.apcAlt                             = typecast(buffer(375:382),'double');
    APINFO.keepOutViol                        = typecast(buffer(383:386),'single');
    APINFO.gimStopTwist                       = typecast(buffer(387:390),'single');
    APINFO.gimStopTilt                        = typecast(buffer(391:394),'single');
    APINFO.gimbalStopAz                       = typecast(buffer(395:398),'single');
    APINFO.apfdFactor                         = typecast(buffer(399:402),'int32');
    APINFO.fastTimeSamples                    = typecast(buffer(403:406),'uint32');
    APINFO.adSampleFreq                       = typecast(buffer(407:410),'single');
    APINFO.apertureTime                       = typecast(buffer(411:414),'single');
    APINFO.numPhaseHistories                  = typecast(buffer(415:418),'uint32');
    APINFO.lightSpeed                         = typecast(buffer(419:426),'double');
    APINFO.delTanApAngle                      = typecast(buffer(427:430),'single');
    APINFO.ver_maj                            = 5;                            
    APINFO.ver_min                            = 1;                            
end

function APINFO = unmarshal_APINFO_5_1(APINFO) %#ok<DEFNU>
    APINFO.missionText = char(APINFO.missionText.');
    APINFO.missionText(find(APINFO.missionText==0,1,'first'):end)=0;
    APINFO.swVerNum = char(APINFO.swVerNum.');
    APINFO.swVerNum(find(APINFO.swVerNum==0,1,'first'):end)=0;
    APINFO.phName = char(APINFO.phName.');
    APINFO.phName(find(APINFO.phName==0,1,'first'):end)=0;
end

function APINFO = marshal_APINFO_5_1(APINFO) %#ok<DEFNU>
    APINFO.missionText = marshal_string(APINFO.missionText, 50);
    APINFO.swVerNum = marshal_string(APINFO.swVerNum, 50);
    APINFO.phName = marshal_string(APINFO.phName, 128);
end

function write_APINFO_5_1(fid, APINFO) %#ok<DEFNU>
    fwrite(fid, APINFO.missionText,                           'uint8');
    fwrite(fid, APINFO.swVerNum,                              'uint8');
    fwrite(fid, APINFO.radarSerNum,                           'uint32');
    fwrite(fid, APINFO.phSource,                              'uint32');
    fwrite(fid, APINFO.phNameLen,                             'uint16');
    fwrite(fid, APINFO.phName,                                'uint8');
    fwrite(fid, APINFO.ctrFreq,                               'single');
    fwrite(fid, APINFO.wavelength,                            'single');
    fwrite(fid, APINFO.rxPolarization,                        'uint32');
    fwrite(fid, APINFO.txPolarization,                        'uint32');
    fwrite(fid, APINFO.azBeamWidth,                           'single');
    fwrite(fid, APINFO.elBeamWidth,                           'single');
    fwrite(fid, APINFO.grazingAngle,                          'single');
    fwrite(fid, APINFO.squintAngle,                           'single');
    fwrite(fid, APINFO.gta,                                   'single');
    fwrite(fid, APINFO.rngToBeamCtr,                          'single');
    fwrite(fid, APINFO.desSquint,                             'single');
    fwrite(fid, APINFO.desRng,                                'single');
    fwrite(fid, APINFO.desGTA,                                'single');
    fwrite(fid, APINFO.antPhaseCtrBear,                       'single');
    fwrite(fid, APINFO.yearMidAp,                             'uint16');
    fwrite(fid, APINFO.monthMidAp,                            'uint16');
    fwrite(fid, APINFO.dayMidAp,                              'uint16');
    fwrite(fid, APINFO.hourMidAp,                             'uint16');
    fwrite(fid, APINFO.minuteMidAp,                           'uint16');
    fwrite(fid, APINFO.secondMidAp,                           'uint16');
    fwrite(fid, APINFO.flightTime,                            'uint32');
    fwrite(fid, APINFO.flightWeek,                            'uint32');
    fwrite(fid, APINFO.chirpRate,                             'single');
    fwrite(fid, APINFO.xDistToStart,                          'single');
    fwrite(fid, APINFO.momeasMode,                            'uint32');
    fwrite(fid, APINFO.radarMode,                             'uint32');
    fwrite(fid, APINFO.rfoa,                                  'single');
    fwrite(fid, APINFO.xVel,                                  'double');
    fwrite(fid, APINFO.yVel,                                  'double');
    fwrite(fid, APINFO.zVel,                                  'double');
    fwrite(fid, APINFO.apcLat,                                'double');
    fwrite(fid, APINFO.apcLon,                                'double');
    fwrite(fid, APINFO.apcAlt,                                'double');
    fwrite(fid, APINFO.keepOutViol,                           'single');
    fwrite(fid, APINFO.gimStopTwist,                          'single');
    fwrite(fid, APINFO.gimStopTilt,                           'single');
    fwrite(fid, APINFO.gimbalStopAz,                          'single');
    fwrite(fid, APINFO.apfdFactor,                            'int32');
    fwrite(fid, APINFO.fastTimeSamples,                       'uint32');
    fwrite(fid, APINFO.adSampleFreq,                          'single');
    fwrite(fid, APINFO.apertureTime,                          'single');
    fwrite(fid, APINFO.numPhaseHistories,                     'uint32');
    fwrite(fid, APINFO.lightSpeed,                            'double');
    fwrite(fid, APINFO.delTanApAngle,                         'single');
end

function APINFO = init_APINFO_5_1() %#ok<DEFNU>
    APINFO.missionText                        = char(zeros(1,50));            
    APINFO.swVerNum                           = char(zeros(1,50));            
    APINFO.radarSerNum                        = zeros(1,1,'uint32');          
    APINFO.phSource                           = zeros(1,1,'uint32');          
    APINFO.phNameLen                          = zeros(1,1,'uint16');          
    APINFO.phName                             = char(zeros(1,128));           
    APINFO.ctrFreq                            = zeros(1,1,'single');          
    APINFO.wavelength                         = zeros(1,1,'single');          
    APINFO.rxPolarization                     = zeros(1,1,'uint32');          
    APINFO.txPolarization                     = zeros(1,1,'uint32');          
    APINFO.azBeamWidth                        = zeros(1,1,'single');          
    APINFO.elBeamWidth                        = zeros(1,1,'single');          
    APINFO.grazingAngle                       = zeros(1,1,'single');          
    APINFO.squintAngle                        = zeros(1,1,'single');          
    APINFO.gta                                = zeros(1,1,'single');          
    APINFO.rngToBeamCtr                       = zeros(1,1,'single');          
    APINFO.desSquint                          = zeros(1,1,'single');          
    APINFO.desRng                             = zeros(1,1,'single');          
    APINFO.desGTA                             = zeros(1,1,'single');          
    APINFO.antPhaseCtrBear                    = zeros(1,1,'single');          
    APINFO.yearMidAp                          = zeros(1,1,'uint16');          
    APINFO.monthMidAp                         = zeros(1,1,'uint16');          
    APINFO.dayMidAp                           = zeros(1,1,'uint16');          
    APINFO.hourMidAp                          = zeros(1,1,'uint16');          
    APINFO.minuteMidAp                        = zeros(1,1,'uint16');          
    APINFO.secondMidAp                        = zeros(1,1,'uint16');          
    APINFO.flightTime                         = zeros(1,1,'uint32');          
    APINFO.flightWeek                         = zeros(1,1,'uint32');          
    APINFO.chirpRate                          = zeros(1,1,'single');          
    APINFO.xDistToStart                       = zeros(1,1,'single');          
    APINFO.momeasMode                         = zeros(1,1,'uint32');          
    APINFO.radarMode                          = zeros(1,1,'uint32');          
    APINFO.rfoa                               = zeros(1,1,'single');          
    APINFO.xVel                               = zeros(1,1,'double');          
    APINFO.yVel                               = zeros(1,1,'double');          
    APINFO.zVel                               = zeros(1,1,'double');          
    APINFO.apcLat                             = zeros(1,1,'double');          
    APINFO.apcLon                             = zeros(1,1,'double');          
    APINFO.apcAlt                             = zeros(1,1,'double');          
    APINFO.keepOutViol                        = zeros(1,1,'single');          
    APINFO.gimStopTwist                       = zeros(1,1,'single');          
    APINFO.gimStopTilt                        = zeros(1,1,'single');          
    APINFO.gimbalStopAz                       = zeros(1,1,'single');          
    APINFO.apfdFactor                         = zeros(1,1,'int32');           
    APINFO.fastTimeSamples                    = zeros(1,1,'uint32');          
    APINFO.adSampleFreq                       = zeros(1,1,'single');          
    APINFO.apertureTime                       = zeros(1,1,'single');          
    APINFO.numPhaseHistories                  = zeros(1,1,'uint32');          
    APINFO.lightSpeed                         = zeros(1,1,'double');          
    APINFO.delTanApAngle                      = zeros(1,1,'single');          
    APINFO.ver_maj                            = 5;                            
    APINFO.ver_min                            = 1;                            
end

function APINFO = read_APINFO_5_2(buffer) %#ok<DEFNU>
    APINFO.missionText                        = typecast(buffer(1:50),'uint8');
    APINFO.swVerNum                           = typecast(buffer(51:100),'uint8');
    APINFO.radarSerNum                        = typecast(buffer(101:104),'uint32');
    APINFO.phSource                           = typecast(buffer(105:108),'uint32');
    APINFO.phNameLen                          = typecast(buffer(109:110),'uint16');
    APINFO.phName                             = typecast(buffer(111:238),'uint8');
    APINFO.ctrFreq                            = typecast(buffer(239:242),'single');
    APINFO.wavelength                         = typecast(buffer(243:246),'single');
    APINFO.rxPolarization                     = typecast(buffer(247:250),'uint32');
    APINFO.txPolarization                     = typecast(buffer(251:254),'uint32');
    APINFO.azBeamWidth                        = typecast(buffer(255:258),'single');
    APINFO.elBeamWidth                        = typecast(buffer(259:262),'single');
    APINFO.grazingAngle                       = typecast(buffer(263:266),'single');
    APINFO.squintAngle                        = typecast(buffer(267:270),'single');
    APINFO.gta                                = typecast(buffer(271:274),'single');
    APINFO.rngToBeamCtr                       = typecast(buffer(275:278),'single');
    APINFO.desSquint                          = typecast(buffer(279:282),'single');
    APINFO.desRng                             = typecast(buffer(283:286),'single');
    APINFO.desGTA                             = typecast(buffer(287:290),'single');
    APINFO.antPhaseCtrBear                    = typecast(buffer(291:294),'single');
    APINFO.yearMidAp                          = typecast(buffer(295:296),'uint16');
    APINFO.monthMidAp                         = typecast(buffer(297:298),'uint16');
    APINFO.dayMidAp                           = typecast(buffer(299:300),'uint16');
    APINFO.hourMidAp                          = typecast(buffer(301:302),'uint16');
    APINFO.minuteMidAp                        = typecast(buffer(303:304),'uint16');
    APINFO.secondMidAp                        = typecast(buffer(305:306),'uint16');
    APINFO.flightTime                         = typecast(buffer(307:310),'uint32');
    APINFO.flightWeek                         = typecast(buffer(311:314),'uint32');
    APINFO.chirpRate                          = typecast(buffer(315:318),'single');
    APINFO.xDistToStart                       = typecast(buffer(319:322),'single');
    APINFO.momeasMode                         = typecast(buffer(323:326),'uint32');
    APINFO.radarMode                          = typecast(buffer(327:330),'uint32');
    APINFO.rfoa                               = typecast(buffer(331:334),'single');
    APINFO.xVel                               = typecast(buffer(335:342),'double');
    APINFO.yVel                               = typecast(buffer(343:350),'double');
    APINFO.zVel                               = typecast(buffer(351:358),'double');
    APINFO.apcLat                             = typecast(buffer(359:366),'double');
    APINFO.apcLon                             = typecast(buffer(367:374),'double');
    APINFO.apcAlt                             = typecast(buffer(375:382),'double');
    APINFO.keepOutViol                        = typecast(buffer(383:386),'single');
    APINFO.gimStopTwist                       = typecast(buffer(387:390),'single');
    APINFO.gimStopTilt                        = typecast(buffer(391:394),'single');
    APINFO.gimbalStopAz                       = typecast(buffer(395:398),'single');
    APINFO.apfdFactor                         = typecast(buffer(399:402),'int32');
    APINFO.fastTimeSamples                    = typecast(buffer(403:406),'uint32');
    APINFO.adSampleFreq                       = typecast(buffer(407:410),'single');
    APINFO.apertureTime                       = typecast(buffer(411:414),'single');
    APINFO.numPhaseHistories                  = typecast(buffer(415:418),'uint32');
    APINFO.lightSpeed                         = typecast(buffer(419:426),'double');
    APINFO.delTanApAngle                      = typecast(buffer(427:430),'single');
    APINFO.metersInSampledDoppler             = typecast(buffer(431:434),'single');
    APINFO.ver_maj                            = 5;                            
    APINFO.ver_min                            = 2;                            
end

function APINFO = unmarshal_APINFO_5_2(APINFO) %#ok<DEFNU>
    APINFO.missionText = char(APINFO.missionText.');
    APINFO.missionText(find(APINFO.missionText==0,1,'first'):end)=0;
    APINFO.swVerNum = char(APINFO.swVerNum.');
    APINFO.swVerNum(find(APINFO.swVerNum==0,1,'first'):end)=0;
    APINFO.phName = char(APINFO.phName.');
    APINFO.phName(find(APINFO.phName==0,1,'first'):end)=0;
end

function APINFO = marshal_APINFO_5_2(APINFO) %#ok<DEFNU>
    APINFO.missionText = marshal_string(APINFO.missionText, 50);
    APINFO.swVerNum = marshal_string(APINFO.swVerNum, 50);
    APINFO.phName = marshal_string(APINFO.phName, 128);
end

function write_APINFO_5_2(fid, APINFO) %#ok<DEFNU>
    fwrite(fid, APINFO.missionText,                           'uint8');
    fwrite(fid, APINFO.swVerNum,                              'uint8');
    fwrite(fid, APINFO.radarSerNum,                           'uint32');
    fwrite(fid, APINFO.phSource,                              'uint32');
    fwrite(fid, APINFO.phNameLen,                             'uint16');
    fwrite(fid, APINFO.phName,                                'uint8');
    fwrite(fid, APINFO.ctrFreq,                               'single');
    fwrite(fid, APINFO.wavelength,                            'single');
    fwrite(fid, APINFO.rxPolarization,                        'uint32');
    fwrite(fid, APINFO.txPolarization,                        'uint32');
    fwrite(fid, APINFO.azBeamWidth,                           'single');
    fwrite(fid, APINFO.elBeamWidth,                           'single');
    fwrite(fid, APINFO.grazingAngle,                          'single');
    fwrite(fid, APINFO.squintAngle,                           'single');
    fwrite(fid, APINFO.gta,                                   'single');
    fwrite(fid, APINFO.rngToBeamCtr,                          'single');
    fwrite(fid, APINFO.desSquint,                             'single');
    fwrite(fid, APINFO.desRng,                                'single');
    fwrite(fid, APINFO.desGTA,                                'single');
    fwrite(fid, APINFO.antPhaseCtrBear,                       'single');
    fwrite(fid, APINFO.yearMidAp,                             'uint16');
    fwrite(fid, APINFO.monthMidAp,                            'uint16');
    fwrite(fid, APINFO.dayMidAp,                              'uint16');
    fwrite(fid, APINFO.hourMidAp,                             'uint16');
    fwrite(fid, APINFO.minuteMidAp,                           'uint16');
    fwrite(fid, APINFO.secondMidAp,                           'uint16');
    fwrite(fid, APINFO.flightTime,                            'uint32');
    fwrite(fid, APINFO.flightWeek,                            'uint32');
    fwrite(fid, APINFO.chirpRate,                             'single');
    fwrite(fid, APINFO.xDistToStart,                          'single');
    fwrite(fid, APINFO.momeasMode,                            'uint32');
    fwrite(fid, APINFO.radarMode,                             'uint32');
    fwrite(fid, APINFO.rfoa,                                  'single');
    fwrite(fid, APINFO.xVel,                                  'double');
    fwrite(fid, APINFO.yVel,                                  'double');
    fwrite(fid, APINFO.zVel,                                  'double');
    fwrite(fid, APINFO.apcLat,                                'double');
    fwrite(fid, APINFO.apcLon,                                'double');
    fwrite(fid, APINFO.apcAlt,                                'double');
    fwrite(fid, APINFO.keepOutViol,                           'single');
    fwrite(fid, APINFO.gimStopTwist,                          'single');
    fwrite(fid, APINFO.gimStopTilt,                           'single');
    fwrite(fid, APINFO.gimbalStopAz,                          'single');
    fwrite(fid, APINFO.apfdFactor,                            'int32');
    fwrite(fid, APINFO.fastTimeSamples,                       'uint32');
    fwrite(fid, APINFO.adSampleFreq,                          'single');
    fwrite(fid, APINFO.apertureTime,                          'single');
    fwrite(fid, APINFO.numPhaseHistories,                     'uint32');
    fwrite(fid, APINFO.lightSpeed,                            'double');
    fwrite(fid, APINFO.delTanApAngle,                         'single');
    fwrite(fid, APINFO.metersInSampledDoppler,                'single');
end

function APINFO = init_APINFO_5_2() %#ok<DEFNU>
    APINFO.missionText                        = char(zeros(1,50));            
    APINFO.swVerNum                           = char(zeros(1,50));            
    APINFO.radarSerNum                        = zeros(1,1,'uint32');          
    APINFO.phSource                           = zeros(1,1,'uint32');          
    APINFO.phNameLen                          = zeros(1,1,'uint16');          
    APINFO.phName                             = char(zeros(1,128));           
    APINFO.ctrFreq                            = zeros(1,1,'single');          
    APINFO.wavelength                         = zeros(1,1,'single');          
    APINFO.rxPolarization                     = zeros(1,1,'uint32');          
    APINFO.txPolarization                     = zeros(1,1,'uint32');          
    APINFO.azBeamWidth                        = zeros(1,1,'single');          
    APINFO.elBeamWidth                        = zeros(1,1,'single');          
    APINFO.grazingAngle                       = zeros(1,1,'single');          
    APINFO.squintAngle                        = zeros(1,1,'single');          
    APINFO.gta                                = zeros(1,1,'single');          
    APINFO.rngToBeamCtr                       = zeros(1,1,'single');          
    APINFO.desSquint                          = zeros(1,1,'single');          
    APINFO.desRng                             = zeros(1,1,'single');          
    APINFO.desGTA                             = zeros(1,1,'single');          
    APINFO.antPhaseCtrBear                    = zeros(1,1,'single');          
    APINFO.yearMidAp                          = zeros(1,1,'uint16');          
    APINFO.monthMidAp                         = zeros(1,1,'uint16');          
    APINFO.dayMidAp                           = zeros(1,1,'uint16');          
    APINFO.hourMidAp                          = zeros(1,1,'uint16');          
    APINFO.minuteMidAp                        = zeros(1,1,'uint16');          
    APINFO.secondMidAp                        = zeros(1,1,'uint16');          
    APINFO.flightTime                         = zeros(1,1,'uint32');          
    APINFO.flightWeek                         = zeros(1,1,'uint32');          
    APINFO.chirpRate                          = zeros(1,1,'single');          
    APINFO.xDistToStart                       = zeros(1,1,'single');          
    APINFO.momeasMode                         = zeros(1,1,'uint32');          
    APINFO.radarMode                          = zeros(1,1,'uint32');          
    APINFO.rfoa                               = zeros(1,1,'single');          
    APINFO.xVel                               = zeros(1,1,'double');          
    APINFO.yVel                               = zeros(1,1,'double');          
    APINFO.zVel                               = zeros(1,1,'double');          
    APINFO.apcLat                             = zeros(1,1,'double');          
    APINFO.apcLon                             = zeros(1,1,'double');          
    APINFO.apcAlt                             = zeros(1,1,'double');          
    APINFO.keepOutViol                        = zeros(1,1,'single');          
    APINFO.gimStopTwist                       = zeros(1,1,'single');          
    APINFO.gimStopTilt                        = zeros(1,1,'single');          
    APINFO.gimbalStopAz                       = zeros(1,1,'single');          
    APINFO.apfdFactor                         = zeros(1,1,'int32');           
    APINFO.fastTimeSamples                    = zeros(1,1,'uint32');          
    APINFO.adSampleFreq                       = zeros(1,1,'single');          
    APINFO.apertureTime                       = zeros(1,1,'single');          
    APINFO.numPhaseHistories                  = zeros(1,1,'uint32');          
    APINFO.lightSpeed                         = zeros(1,1,'double');          
    APINFO.delTanApAngle                      = zeros(1,1,'single');          
    APINFO.metersInSampledDoppler             = zeros(1,1,'single');          
    APINFO.ver_maj                            = 5;                            
    APINFO.ver_min                            = 2;                            
end

function CCDINFO = read_CCDINFO_1_0(buffer) %#ok<DEFNU>
    CCDINFO.avgCoherence                      = typecast(buffer(1:4),'single');
    CCDINFO.bulkRegX                          = typecast(buffer(5:8),'single');
    CCDINFO.bulkRegY                          = typecast(buffer(9:12),'single');
    CCDINFO.flightTimeRef1                    = typecast(buffer(13:16),'uint32');
    CCDINFO.flightTimeRef2                    = typecast(buffer(17:20),'uint32');
    CCDINFO.flightWeekRef1                    = typecast(buffer(21:24),'uint32');
    CCDINFO.flightWeekRef2                    = typecast(buffer(25:28),'uint32');
    CCDINFO.ref1FileNameLen                   = typecast(buffer(29:30),'uint16');
    CCDINFO.ref2FileNameLen                   = typecast(buffer(31:32),'uint16');
    CCDINFO.ref1FileName                      = typecast(buffer(33:288),'uint8');
    CCDINFO.ref2FileName                      = typecast(buffer(289:544),'uint8');
    CCDINFO.ver_maj                           = 1;                            
    CCDINFO.ver_min                           = 0;                            
end

function CCDINFO = unmarshal_CCDINFO_1_0(CCDINFO) %#ok<DEFNU>
    CCDINFO.ref1FileName = char(CCDINFO.ref1FileName.');
    CCDINFO.ref1FileName(find(CCDINFO.ref1FileName==0,1,'first'):end)=0;
    CCDINFO.ref2FileName = char(CCDINFO.ref2FileName.');
    CCDINFO.ref2FileName(find(CCDINFO.ref2FileName==0,1,'first'):end)=0;
end

function CCDINFO = marshal_CCDINFO_1_0(CCDINFO) %#ok<DEFNU>
    CCDINFO.ref1FileName = marshal_string(CCDINFO.ref1FileName, 256);
    CCDINFO.ref2FileName = marshal_string(CCDINFO.ref2FileName, 256);
end

function write_CCDINFO_1_0(fid, CCDINFO) %#ok<DEFNU>
    fwrite(fid, CCDINFO.avgCoherence,                         'single');
    fwrite(fid, CCDINFO.bulkRegX,                             'single');
    fwrite(fid, CCDINFO.bulkRegY,                             'single');
    fwrite(fid, CCDINFO.flightTimeRef1,                       'uint32');
    fwrite(fid, CCDINFO.flightTimeRef2,                       'uint32');
    fwrite(fid, CCDINFO.flightWeekRef1,                       'uint32');
    fwrite(fid, CCDINFO.flightWeekRef2,                       'uint32');
    fwrite(fid, CCDINFO.ref1FileNameLen,                      'uint16');
    fwrite(fid, CCDINFO.ref2FileNameLen,                      'uint16');
    fwrite(fid, CCDINFO.ref1FileName,                         'uint8');
    fwrite(fid, CCDINFO.ref2FileName,                         'uint8');
end

function CCDINFO = init_CCDINFO_1_0() %#ok<DEFNU>
    CCDINFO.avgCoherence                      = zeros(1,1,'single');          
    CCDINFO.bulkRegX                          = zeros(1,1,'single');          
    CCDINFO.bulkRegY                          = zeros(1,1,'single');          
    CCDINFO.flightTimeRef1                    = zeros(1,1,'uint32');          
    CCDINFO.flightTimeRef2                    = zeros(1,1,'uint32');          
    CCDINFO.flightWeekRef1                    = zeros(1,1,'uint32');          
    CCDINFO.flightWeekRef2                    = zeros(1,1,'uint32');          
    CCDINFO.ref1FileNameLen                   = zeros(1,1,'uint16');          
    CCDINFO.ref2FileNameLen                   = zeros(1,1,'uint16');          
    CCDINFO.ref1FileName                      = char(zeros(1,256));           
    CCDINFO.ref2FileName                      = char(zeros(1,256));           
    CCDINFO.ver_maj                           = 1;                            
    CCDINFO.ver_min                           = 0;                            
end

function CCDINFO = read_CCDINFO_1_1(buffer) %#ok<DEFNU>
    CCDINFO.avgCoherence                      = typecast(buffer(1:4),'single');
    CCDINFO.bulkRegX                          = typecast(buffer(5:8),'single');
    CCDINFO.bulkRegY                          = typecast(buffer(9:12),'single');
    CCDINFO.flightTimeRef1                    = typecast(buffer(13:16),'uint32');
    CCDINFO.flightTimeRef2                    = typecast(buffer(17:20),'uint32');
    CCDINFO.flightWeekRef1                    = typecast(buffer(21:24),'uint32');
    CCDINFO.flightWeekRef2                    = typecast(buffer(25:28),'uint32');
    CCDINFO.ref1FileNameLen                   = typecast(buffer(29:30),'uint16');
    CCDINFO.ref2FileNameLen                   = typecast(buffer(31:32),'uint16');
    CCDINFO.ref1FileName                      = typecast(buffer(33:288),'uint8');
    CCDINFO.ref2FileName                      = typecast(buffer(289:544),'uint8');
    CCDINFO.origRangePixels                   = typecast(buffer(545:548),'uint32');
    CCDINFO.origAzPixels                      = typecast(buffer(549:552),'uint32');
    CCDINFO.ver_maj                           = 1;                            
    CCDINFO.ver_min                           = 1;                            
end

function CCDINFO = unmarshal_CCDINFO_1_1(CCDINFO) %#ok<DEFNU>
    CCDINFO.ref1FileName = char(CCDINFO.ref1FileName.');
    CCDINFO.ref1FileName(find(CCDINFO.ref1FileName==0,1,'first'):end)=0;
    CCDINFO.ref2FileName = char(CCDINFO.ref2FileName.');
    CCDINFO.ref2FileName(find(CCDINFO.ref2FileName==0,1,'first'):end)=0;
end

function CCDINFO = marshal_CCDINFO_1_1(CCDINFO) %#ok<DEFNU>
    CCDINFO.ref1FileName = marshal_string(CCDINFO.ref1FileName, 256);
    CCDINFO.ref2FileName = marshal_string(CCDINFO.ref2FileName, 256);
end

function write_CCDINFO_1_1(fid, CCDINFO) %#ok<DEFNU>
    fwrite(fid, CCDINFO.avgCoherence,                         'single');
    fwrite(fid, CCDINFO.bulkRegX,                             'single');
    fwrite(fid, CCDINFO.bulkRegY,                             'single');
    fwrite(fid, CCDINFO.flightTimeRef1,                       'uint32');
    fwrite(fid, CCDINFO.flightTimeRef2,                       'uint32');
    fwrite(fid, CCDINFO.flightWeekRef1,                       'uint32');
    fwrite(fid, CCDINFO.flightWeekRef2,                       'uint32');
    fwrite(fid, CCDINFO.ref1FileNameLen,                      'uint16');
    fwrite(fid, CCDINFO.ref2FileNameLen,                      'uint16');
    fwrite(fid, CCDINFO.ref1FileName,                         'uint8');
    fwrite(fid, CCDINFO.ref2FileName,                         'uint8');
    fwrite(fid, CCDINFO.origRangePixels,                      'uint32');
    fwrite(fid, CCDINFO.origAzPixels,                         'uint32');
end

function CCDINFO = init_CCDINFO_1_1() %#ok<DEFNU>
    CCDINFO.avgCoherence                      = zeros(1,1,'single');          
    CCDINFO.bulkRegX                          = zeros(1,1,'single');          
    CCDINFO.bulkRegY                          = zeros(1,1,'single');          
    CCDINFO.flightTimeRef1                    = zeros(1,1,'uint32');          
    CCDINFO.flightTimeRef2                    = zeros(1,1,'uint32');          
    CCDINFO.flightWeekRef1                    = zeros(1,1,'uint32');          
    CCDINFO.flightWeekRef2                    = zeros(1,1,'uint32');          
    CCDINFO.ref1FileNameLen                   = zeros(1,1,'uint16');          
    CCDINFO.ref2FileNameLen                   = zeros(1,1,'uint16');          
    CCDINFO.ref1FileName                      = char(zeros(1,256));           
    CCDINFO.ref2FileName                      = char(zeros(1,256));           
    CCDINFO.origRangePixels                   = zeros(1,1,'uint32');          
    CCDINFO.origAzPixels                      = zeros(1,1,'uint32');          
    CCDINFO.ver_maj                           = 1;                            
    CCDINFO.ver_min                           = 1;                            
end

function CHIPINFO = read_CHIPINFO_1_0(buffer) %#ok<DEFNU>
    CHIPINFO.zoomLevel                        = typecast(buffer(1:4),'single');
    CHIPINFO.MCPLat                           = typecast(buffer(5:12),'double');
    CHIPINFO.MCPLon                           = typecast(buffer(13:20),'double');
    CHIPINFO.MCPAlt                           = typecast(buffer(21:28),'double');
    CHIPINFO.chipUpperLeftCorner_x            = typecast(buffer(29:32),'uint32');
    CHIPINFO.chipUpperLeftCorner_y            = typecast(buffer(33:36),'uint32');
    CHIPINFO.rangePixels_original             = typecast(buffer(37:40),'uint32');
    CHIPINFO.azPixels_original                = typecast(buffer(41:44),'uint32');
    CHIPINFO.ver_maj                          = 1;                            
    CHIPINFO.ver_min                          = 0;                            
end

function CHIPINFO = unmarshal_CHIPINFO_1_0(CHIPINFO) %#ok<DEFNU>
end

function CHIPINFO = marshal_CHIPINFO_1_0(CHIPINFO) %#ok<DEFNU>
end

function write_CHIPINFO_1_0(fid, CHIPINFO) %#ok<DEFNU>
    fwrite(fid, CHIPINFO.zoomLevel,                           'single');
    fwrite(fid, CHIPINFO.MCPLat,                              'double');
    fwrite(fid, CHIPINFO.MCPLon,                              'double');
    fwrite(fid, CHIPINFO.MCPAlt,                              'double');
    fwrite(fid, CHIPINFO.chipUpperLeftCorner_x,               'uint32');
    fwrite(fid, CHIPINFO.chipUpperLeftCorner_y,               'uint32');
    fwrite(fid, CHIPINFO.rangePixels_original,                'uint32');
    fwrite(fid, CHIPINFO.azPixels_original,                   'uint32');
end

function CHIPINFO = init_CHIPINFO_1_0() %#ok<DEFNU>
    CHIPINFO.zoomLevel                        = zeros(1,1,'single');          
    CHIPINFO.MCPLat                           = zeros(1,1,'double');          
    CHIPINFO.MCPLon                           = zeros(1,1,'double');          
    CHIPINFO.MCPAlt                           = zeros(1,1,'double');          
    CHIPINFO.chipUpperLeftCorner_x            = zeros(1,1,'uint32');          
    CHIPINFO.chipUpperLeftCorner_y            = zeros(1,1,'uint32');          
    CHIPINFO.rangePixels_original             = zeros(1,1,'uint32');          
    CHIPINFO.azPixels_original                = zeros(1,1,'uint32');          
    CHIPINFO.ver_maj                          = 1;                            
    CHIPINFO.ver_min                          = 0;                            
end

function COMPRESSINFO = read_COMPRESSINFO_1_0(buffer) %#ok<DEFNU>
    COMPRESSINFO.uncompressedSize             = typecast(buffer(1:4),'int32');
    COMPRESSINFO.compressionVal               = typecast(buffer(5:8),'single');
    COMPRESSINFO.jpegLUT                      = typecast(buffer(9:12),'uint32');
    COMPRESSINFO.ver_maj                      = 1;                            
    COMPRESSINFO.ver_min                      = 0;                            
end

function COMPRESSINFO = unmarshal_COMPRESSINFO_1_0(COMPRESSINFO) %#ok<DEFNU>
end

function COMPRESSINFO = marshal_COMPRESSINFO_1_0(COMPRESSINFO) %#ok<DEFNU>
end

function write_COMPRESSINFO_1_0(fid, COMPRESSINFO) %#ok<DEFNU>
    fwrite(fid, COMPRESSINFO.uncompressedSize,                'int32');
    fwrite(fid, COMPRESSINFO.compressionVal,                  'single');
    fwrite(fid, COMPRESSINFO.jpegLUT,                         'uint32');
end

function COMPRESSINFO = init_COMPRESSINFO_1_0() %#ok<DEFNU>
    COMPRESSINFO.uncompressedSize             = zeros(1,1,'int32');           
    COMPRESSINFO.compressionVal               = zeros(1,1,'single');          
    COMPRESSINFO.jpegLUT                      = zeros(1,1,'uint32');          
    COMPRESSINFO.ver_maj                      = 1;                            
    COMPRESSINFO.ver_min                      = 0;                            
end

function COMPRESSINFO = read_COMPRESSINFO_1_1(buffer) %#ok<DEFNU>
    COMPRESSINFO.uncompressedSize             = typecast(buffer(1:4),'int32');
    COMPRESSINFO.compressionVal               = typecast(buffer(5:8),'single');
    COMPRESSINFO.jpegLUT                      = typecast(buffer(9:12),'uint32');
    COMPRESSINFO.jpegOffset                   = typecast(buffer(13:16),'uint32');
    COMPRESSINFO.suggestedLUT                 = typecast(buffer(17:20),'single');
    COMPRESSINFO.ver_maj                      = 1;                            
    COMPRESSINFO.ver_min                      = 1;                            
end

function COMPRESSINFO = unmarshal_COMPRESSINFO_1_1(COMPRESSINFO) %#ok<DEFNU>
end

function COMPRESSINFO = marshal_COMPRESSINFO_1_1(COMPRESSINFO) %#ok<DEFNU>
end

function write_COMPRESSINFO_1_1(fid, COMPRESSINFO) %#ok<DEFNU>
    fwrite(fid, COMPRESSINFO.uncompressedSize,                'int32');
    fwrite(fid, COMPRESSINFO.compressionVal,                  'single');
    fwrite(fid, COMPRESSINFO.jpegLUT,                         'uint32');
    fwrite(fid, COMPRESSINFO.jpegOffset,                      'uint32');
    fwrite(fid, COMPRESSINFO.suggestedLUT,                    'single');
end

function COMPRESSINFO = init_COMPRESSINFO_1_1() %#ok<DEFNU>
    COMPRESSINFO.uncompressedSize             = zeros(1,1,'int32');           
    COMPRESSINFO.compressionVal               = zeros(1,1,'single');          
    COMPRESSINFO.jpegLUT                      = zeros(1,1,'uint32');          
    COMPRESSINFO.jpegOffset                   = zeros(1,1,'uint32');          
    COMPRESSINFO.suggestedLUT                 = zeros(1,1,'single');          
    COMPRESSINFO.ver_maj                      = 1;                            
    COMPRESSINFO.ver_min                      = 1;                            
end

function GEOINFO = read_GEOINFO_1_0(buffer) %#ok<DEFNU>
    GEOINFO.imagePlane                        = typecast(buffer(1:4),'int32');
    GEOINFO.rangePixSpacing                   = typecast(buffer(5:8),'single');
    GEOINFO.desiredGrazAng                    = typecast(buffer(9:12),'single');
    GEOINFO.azPixSpacing                      = typecast(buffer(13:16),'single');
    GEOINFO.patchCtrLat                       = typecast(buffer(17:24),'double');
    GEOINFO.patchCtrLong                      = typecast(buffer(25:32),'double');
    GEOINFO.patchCtrAlt                       = typecast(buffer(33:40),'double');
    GEOINFO.pixLocImCtrRow                    = typecast(buffer(41:44),'uint32');
    GEOINFO.pixLocImCtrCol                    = typecast(buffer(45:48),'uint32');
    GEOINFO.imgRotAngle                       = typecast(buffer(49:52),'single');
    GEOINFO.ver_maj                           = 1;                            
    GEOINFO.ver_min                           = 0;                            
end

function GEOINFO = unmarshal_GEOINFO_1_0(GEOINFO) %#ok<DEFNU>
end

function GEOINFO = marshal_GEOINFO_1_0(GEOINFO) %#ok<DEFNU>
end

function write_GEOINFO_1_0(fid, GEOINFO) %#ok<DEFNU>
    fwrite(fid, GEOINFO.imagePlane,                           'int32');
    fwrite(fid, GEOINFO.rangePixSpacing,                      'single');
    fwrite(fid, GEOINFO.desiredGrazAng,                       'single');
    fwrite(fid, GEOINFO.azPixSpacing,                         'single');
    fwrite(fid, GEOINFO.patchCtrLat,                          'double');
    fwrite(fid, GEOINFO.patchCtrLong,                         'double');
    fwrite(fid, GEOINFO.patchCtrAlt,                          'double');
    fwrite(fid, GEOINFO.pixLocImCtrRow,                       'uint32');
    fwrite(fid, GEOINFO.pixLocImCtrCol,                       'uint32');
    fwrite(fid, GEOINFO.imgRotAngle,                          'single');
end

function GEOINFO = init_GEOINFO_1_0() %#ok<DEFNU>
    GEOINFO.imagePlane                        = zeros(1,1,'int32');           
    GEOINFO.rangePixSpacing                   = zeros(1,1,'single');          
    GEOINFO.desiredGrazAng                    = zeros(1,1,'single');          
    GEOINFO.azPixSpacing                      = zeros(1,1,'single');          
    GEOINFO.patchCtrLat                       = zeros(1,1,'double');          
    GEOINFO.patchCtrLong                      = zeros(1,1,'double');          
    GEOINFO.patchCtrAlt                       = zeros(1,1,'double');          
    GEOINFO.pixLocImCtrRow                    = zeros(1,1,'uint32');          
    GEOINFO.pixLocImCtrCol                    = zeros(1,1,'uint32');          
    GEOINFO.imgRotAngle                       = zeros(1,1,'single');          
    GEOINFO.ver_maj                           = 1;                            
    GEOINFO.ver_min                           = 0;                            
end

function GEOINFO = read_GEOINFO_1_1(buffer) %#ok<DEFNU>
    GEOINFO.imagePlane                        = typecast(buffer(1:4),'int32');
    GEOINFO.rangePixSpacing                   = typecast(buffer(5:8),'single');
    GEOINFO.desiredGrazAng                    = typecast(buffer(9:12),'single');
    GEOINFO.azPixSpacing                      = typecast(buffer(13:16),'single');
    GEOINFO.patchCtrLat                       = typecast(buffer(17:24),'double');
    GEOINFO.patchCtrLong                      = typecast(buffer(25:32),'double');
    GEOINFO.patchCtrAlt                       = typecast(buffer(33:40),'double');
    GEOINFO.pixLocImCtrRow                    = typecast(buffer(41:44),'uint32');
    GEOINFO.pixLocImCtrCol                    = typecast(buffer(45:48),'uint32');
    GEOINFO.imgRotAngle                       = typecast(buffer(49:52),'single');
    GEOINFO.ver_maj                           = 1;                            
    GEOINFO.ver_min                           = 1;                            
end

function GEOINFO = unmarshal_GEOINFO_1_1(GEOINFO) %#ok<DEFNU>
end

function GEOINFO = marshal_GEOINFO_1_1(GEOINFO) %#ok<DEFNU>
end

function write_GEOINFO_1_1(fid, GEOINFO) %#ok<DEFNU>
    fwrite(fid, GEOINFO.imagePlane,                           'int32');
    fwrite(fid, GEOINFO.rangePixSpacing,                      'single');
    fwrite(fid, GEOINFO.desiredGrazAng,                       'single');
    fwrite(fid, GEOINFO.azPixSpacing,                         'single');
    fwrite(fid, GEOINFO.patchCtrLat,                          'double');
    fwrite(fid, GEOINFO.patchCtrLong,                         'double');
    fwrite(fid, GEOINFO.patchCtrAlt,                          'double');
    fwrite(fid, GEOINFO.pixLocImCtrRow,                       'uint32');
    fwrite(fid, GEOINFO.pixLocImCtrCol,                       'uint32');
    fwrite(fid, GEOINFO.imgRotAngle,                          'single');
end

function GEOINFO = init_GEOINFO_1_1() %#ok<DEFNU>
    GEOINFO.imagePlane                        = zeros(1,1,'int32');           
    GEOINFO.rangePixSpacing                   = zeros(1,1,'single');          
    GEOINFO.desiredGrazAng                    = zeros(1,1,'single');          
    GEOINFO.azPixSpacing                      = zeros(1,1,'single');          
    GEOINFO.patchCtrLat                       = zeros(1,1,'double');          
    GEOINFO.patchCtrLong                      = zeros(1,1,'double');          
    GEOINFO.patchCtrAlt                       = zeros(1,1,'double');          
    GEOINFO.pixLocImCtrRow                    = zeros(1,1,'uint32');          
    GEOINFO.pixLocImCtrCol                    = zeros(1,1,'uint32');          
    GEOINFO.imgRotAngle                       = zeros(1,1,'single');          
    GEOINFO.ver_maj                           = 1;                            
    GEOINFO.ver_min                           = 1;                            
end

function GMTIINFO = read_GMTIINFO_1_0(buffer) %#ok<DEFNU>
    GMTIINFO.mti_calmin                       = typecast(buffer(1:4),'single');
    GMTIINFO.mti_calscale                     = typecast(buffer(5:8),'single');
    GMTIINFO.ver_maj                          = 1;                            
    GMTIINFO.ver_min                          = 0;                            
end

function GMTIINFO = unmarshal_GMTIINFO_1_0(GMTIINFO) %#ok<DEFNU>
end

function GMTIINFO = marshal_GMTIINFO_1_0(GMTIINFO) %#ok<DEFNU>
end

function write_GMTIINFO_1_0(fid, GMTIINFO) %#ok<DEFNU>
    fwrite(fid, GMTIINFO.mti_calmin,                          'single');
    fwrite(fid, GMTIINFO.mti_calscale,                        'single');
end

function GMTIINFO = init_GMTIINFO_1_0() %#ok<DEFNU>
    GMTIINFO.mti_calmin                       = zeros(1,1,'single');          
    GMTIINFO.mti_calscale                     = zeros(1,1,'single');          
    GMTIINFO.ver_maj                          = 1;                            
    GMTIINFO.ver_min                          = 0;                            
end

function GSATIMG = read_GSATIMG_1_6(buffer) %#ok<DEFNU>
    GSATIMG.Version_Minor                     = typecast(buffer(1:2),'uint16');
    GSATIMG.Version_Major                     = typecast(buffer(3:4),'uint16');
    GSATIMG.Length                            = typecast(buffer(5:8),'uint32');
    GSATIMG.Creator_Length                    = typecast(buffer(9:10),'uint16');
    GSATIMG.Creator                           = typecast(buffer(11:34),'uint8');
    GSATIMG.DateTime                          = typecast(buffer(35:46),'uint16');
    GSATIMG.Endian                            = typecast(buffer(47:48),'uint16');
    GSATIMG.BytesPerPixel                     = typecast(buffer(49:52),'uint32');
    GSATIMG.FrameCnt                          = typecast(buffer(53:56),'uint32');
    GSATIMG.ImageType                         = typecast(buffer(57:60),'uint32');
    GSATIMG.RowMajor                          = typecast(buffer(61:64),'uint32');
    GSATIMG.RgCnt                             = typecast(buffer(65:68),'uint32');
    GSATIMG.AzCnt                             = typecast(buffer(69:72),'uint32');
    GSATIMG.ScaleExponent                     = typecast(buffer(73:76),'int32');
    GSATIMG.ScaleMantissa                     = typecast(buffer(77:80),'int32');
    GSATIMG.OffsetExponent                    = typecast(buffer(81:84),'int32');
    GSATIMG.OffsetMantissa                    = typecast(buffer(85:88),'int32');
    GSATIMG.Comment_Length                    = typecast(buffer(89:90),'uint16');
    GSATIMG.Comment                           = typecast(buffer(91:256),'uint8');
    GSATIMG.ImagePlane                        = typecast(buffer(257:260),'uint32');
    GSATIMG.RgPixelSz                         = typecast(buffer(261:264),'uint32');
    GSATIMG.AzPixelSz                         = typecast(buffer(265:268),'uint32');
    GSATIMG.AzOverlap                         = typecast(buffer(269:272),'int32');
    GSATIMG.SRPLat                            = typecast(buffer(273:276),'int32');
    GSATIMG.SRPLong                           = typecast(buffer(277:280),'int32');
    GSATIMG.SRPAlt                            = typecast(buffer(281:284),'int32');
    GSATIMG.RFOA                              = typecast(buffer(285:288),'int32');
    GSATIMG.XtoSRP                            = typecast(buffer(289:292),'int32');
    GSATIMG.PhaseName_Length                  = typecast(buffer(293:294),'uint16');
    GSATIMG.PhaseName                         = typecast(buffer(295:422),'uint8');
    GSATIMG.ImageName_Length                  = typecast(buffer(423:424),'uint16');
    GSATIMG.ImageName                         = typecast(buffer(425:552),'uint8');
    GSATIMG.LookCnt                           = typecast(buffer(553:556),'uint32');
    GSATIMG.ParamRefAp                        = typecast(buffer(557:560),'uint32');
    GSATIMG.ParamRefPos                       = typecast(buffer(561:564),'uint32');
    GSATIMG.GrzAngle                          = typecast(buffer(565:568),'uint32');
    GSATIMG.Squint                            = typecast(buffer(569:572),'int32');
    GSATIMG.GTA                               = typecast(buffer(573:576),'int32');
    GSATIMG.RgBeamCtr                         = typecast(buffer(577:580),'uint32');
    GSATIMG.FlightTime                        = typecast(buffer(581:584),'uint32');
    GSATIMG.RgChirpRate                       = typecast(buffer(585:588),'single');
    GSATIMG.XtoStart                          = typecast(buffer(589:592),'int32');
    GSATIMG.MoCompMode                        = typecast(buffer(593:596),'uint32');
    GSATIMG.V_X                               = typecast(buffer(597:600),'uint32');
    GSATIMG.APCLat                            = typecast(buffer(601:604),'int32');
    GSATIMG.APCLong                           = typecast(buffer(605:608),'int32');
    GSATIMG.APCAlt                            = typecast(buffer(609:612),'int32');
    GSATIMG.Calparm                           = typecast(buffer(613:616),'uint32');
    GSATIMG.LogicalBlkAdrr                    = typecast(buffer(617:620),'uint32');
    GSATIMG.AzResolution                      = typecast(buffer(621:624),'uint32');
    GSATIMG.RgResolution                      = typecast(buffer(625:628),'uint32');
    GSATIMG.DesSigmaN                         = typecast(buffer(629:632),'int32');
    GSATIMG.DesGrazAngle                      = typecast(buffer(633:636),'uint32');
    GSATIMG.DesSquint                         = typecast(buffer(637:640),'int32');
    GSATIMG.DesRng                            = typecast(buffer(641:644),'uint32');
    GSATIMG.SceneTrackAngle                   = typecast(buffer(645:648),'int32');
    GSATIMG.UserParam                         = typecast(buffer(649:696),'uint8');
    GSATIMG.CoarseSNR                         = typecast(buffer(697:700),'int32');
    GSATIMG.CoarseAzSubSamp                   = typecast(buffer(701:704),'int32');
    GSATIMG.CoarseRngSubSamp                  = typecast(buffer(705:708),'int32');
    GSATIMG.MaxAzShift                        = typecast(buffer(709:712),'int32');
    GSATIMG.MaxRngShift                       = typecast(buffer(713:716),'int32');
    GSATIMG.CoarseDltAz                       = typecast(buffer(717:720),'int32');
    GSATIMG.CoarseDltRng                      = typecast(buffer(721:724),'int32');
    GSATIMG.TotProcs                          = typecast(buffer(725:728),'int32');
    GSATIMG.TptBoxCMode                       = typecast(buffer(729:732),'int32');
    GSATIMG.SNRThresh                         = typecast(buffer(733:736),'int32');
    GSATIMG.RngSize                           = typecast(buffer(737:740),'int32');
    GSATIMG.MapBoxSize                        = typecast(buffer(741:744),'int32');
    GSATIMG.BoxSize                           = typecast(buffer(745:748),'int32');
    GSATIMG.BoxSpc                            = typecast(buffer(749:752),'int32');
    GSATIMG.TotTPts                           = typecast(buffer(753:756),'int32');
    GSATIMG.GoodTPts                          = typecast(buffer(757:760),'int32');
    GSATIMG.RndSeed                           = typecast(buffer(761:764),'int32');
    GSATIMG.RngShift                          = typecast(buffer(765:768),'int32');
    GSATIMG.AzShift                           = typecast(buffer(769:772),'int32');
    GSATIMG.SumXRamp                          = typecast(buffer(773:776),'int32');
    GSATIMG.SumYRamp                          = typecast(buffer(777:780),'int32');
    GSATIMG.Cy9kTapeBlock                     = typecast(buffer(781:784),'uint32');
    GSATIMG.NominalCenterFreq                 = typecast(buffer(785:788),'single');
    GSATIMG.ImageFlags                        = typecast(buffer(789:792),'uint32');
    GSATIMG.LineNumber                        = typecast(buffer(793:796),'uint32');
    GSATIMG.PatchNumber                       = typecast(buffer(797:800),'uint32');
    GSATIMG.Lambda0                           = typecast(buffer(801:804),'single');
    GSATIMG.SRngPixSpace                      = typecast(buffer(805:808),'single');
    GSATIMG.DoppPixSpace                      = typecast(buffer(809:812),'single');
    GSATIMG.DoppOffset                        = typecast(buffer(813:816),'single');
    GSATIMG.DoppRngScale                      = typecast(buffer(817:820),'single');
    GSATIMG.MuxTimeDelay                      = typecast(buffer(821:824),'single');
    GSATIMG.APCXECEF                          = typecast(buffer(825:832),'double');
    GSATIMG.APCYECEF                          = typecast(buffer(833:840),'double');
    GSATIMG.APCZECEF                          = typecast(buffer(841:848),'double');
    GSATIMG.VxECEF                            = typecast(buffer(849:852),'single');
    GSATIMG.VyECEF                            = typecast(buffer(853:856),'single');
    GSATIMG.VzECEF                            = typecast(buffer(857:860),'single');
    GSATIMG.PhaseCal                          = typecast(buffer(861:864),'single');
    GSATIMG.SRPxECEF                          = typecast(buffer(865:872),'double');
    GSATIMG.SRPyECEF                          = typecast(buffer(873:880),'double');
    GSATIMG.SRPzECEF                          = typecast(buffer(881:888),'double');
    GSATIMG.Res5                              = typecast(buffer(889:952),'uint8');
    GSATIMG.ver_maj                           = 1;                            
    GSATIMG.ver_min                           = 6;                            
end

function GSATIMG = unmarshal_GSATIMG_1_6(GSATIMG) %#ok<DEFNU>
    GSATIMG.Creator = char(GSATIMG.Creator.');
    GSATIMG.Creator(find(GSATIMG.Creator==0,1,'first'):end)=0;
    GSATIMG.Comment = char(GSATIMG.Comment.');
    GSATIMG.Comment(find(GSATIMG.Comment==0,1,'first'):end)=0;
    GSATIMG.PhaseName = char(GSATIMG.PhaseName.');
    GSATIMG.PhaseName(find(GSATIMG.PhaseName==0,1,'first'):end)=0;
    GSATIMG.ImageName = char(GSATIMG.ImageName.');
    GSATIMG.ImageName(find(GSATIMG.ImageName==0,1,'first'):end)=0;
    GSATIMG.Res5 = char(GSATIMG.Res5.');
    GSATIMG.Res5(find(GSATIMG.Res5==0,1,'first'):end)=0;
end

function GSATIMG = marshal_GSATIMG_1_6(GSATIMG) %#ok<DEFNU>
    GSATIMG.Creator = marshal_string(GSATIMG.Creator, 24);
    GSATIMG.Comment = marshal_string(GSATIMG.Comment, 166);
    GSATIMG.PhaseName = marshal_string(GSATIMG.PhaseName, 128);
    GSATIMG.ImageName = marshal_string(GSATIMG.ImageName, 128);
    GSATIMG.Res5 = marshal_string(GSATIMG.Res5, 64);
end

function write_GSATIMG_1_6(fid, GSATIMG) %#ok<DEFNU>
    fwrite(fid, GSATIMG.Version_Minor,                        'uint16');
    fwrite(fid, GSATIMG.Version_Major,                        'uint16');
    fwrite(fid, GSATIMG.Length,                               'uint32');
    fwrite(fid, GSATIMG.Creator_Length,                       'uint16');
    fwrite(fid, GSATIMG.Creator,                              'uint8');
    fwrite(fid, GSATIMG.DateTime,                             'uint16');
    fwrite(fid, GSATIMG.Endian,                               'uint16');
    fwrite(fid, GSATIMG.BytesPerPixel,                        'uint32');
    fwrite(fid, GSATIMG.FrameCnt,                             'uint32');
    fwrite(fid, GSATIMG.ImageType,                            'uint32');
    fwrite(fid, GSATIMG.RowMajor,                             'uint32');
    fwrite(fid, GSATIMG.RgCnt,                                'uint32');
    fwrite(fid, GSATIMG.AzCnt,                                'uint32');
    fwrite(fid, GSATIMG.ScaleExponent,                        'int32');
    fwrite(fid, GSATIMG.ScaleMantissa,                        'int32');
    fwrite(fid, GSATIMG.OffsetExponent,                       'int32');
    fwrite(fid, GSATIMG.OffsetMantissa,                       'int32');
    fwrite(fid, GSATIMG.Comment_Length,                       'uint16');
    fwrite(fid, GSATIMG.Comment,                              'uint8');
    fwrite(fid, GSATIMG.ImagePlane,                           'uint32');
    fwrite(fid, GSATIMG.RgPixelSz,                            'uint32');
    fwrite(fid, GSATIMG.AzPixelSz,                            'uint32');
    fwrite(fid, GSATIMG.AzOverlap,                            'int32');
    fwrite(fid, GSATIMG.SRPLat,                               'int32');
    fwrite(fid, GSATIMG.SRPLong,                              'int32');
    fwrite(fid, GSATIMG.SRPAlt,                               'int32');
    fwrite(fid, GSATIMG.RFOA,                                 'int32');
    fwrite(fid, GSATIMG.XtoSRP,                               'int32');
    fwrite(fid, GSATIMG.PhaseName_Length,                     'uint16');
    fwrite(fid, GSATIMG.PhaseName,                            'uint8');
    fwrite(fid, GSATIMG.ImageName_Length,                     'uint16');
    fwrite(fid, GSATIMG.ImageName,                            'uint8');
    fwrite(fid, GSATIMG.LookCnt,                              'uint32');
    fwrite(fid, GSATIMG.ParamRefAp,                           'uint32');
    fwrite(fid, GSATIMG.ParamRefPos,                          'uint32');
    fwrite(fid, GSATIMG.GrzAngle,                             'uint32');
    fwrite(fid, GSATIMG.Squint,                               'int32');
    fwrite(fid, GSATIMG.GTA,                                  'int32');
    fwrite(fid, GSATIMG.RgBeamCtr,                            'uint32');
    fwrite(fid, GSATIMG.FlightTime,                           'uint32');
    fwrite(fid, GSATIMG.RgChirpRate,                          'single');
    fwrite(fid, GSATIMG.XtoStart,                             'int32');
    fwrite(fid, GSATIMG.MoCompMode,                           'uint32');
    fwrite(fid, GSATIMG.V_X,                                  'uint32');
    fwrite(fid, GSATIMG.APCLat,                               'int32');
    fwrite(fid, GSATIMG.APCLong,                              'int32');
    fwrite(fid, GSATIMG.APCAlt,                               'int32');
    fwrite(fid, GSATIMG.Calparm,                              'uint32');
    fwrite(fid, GSATIMG.LogicalBlkAdrr,                       'uint32');
    fwrite(fid, GSATIMG.AzResolution,                         'uint32');
    fwrite(fid, GSATIMG.RgResolution,                         'uint32');
    fwrite(fid, GSATIMG.DesSigmaN,                            'int32');
    fwrite(fid, GSATIMG.DesGrazAngle,                         'uint32');
    fwrite(fid, GSATIMG.DesSquint,                            'int32');
    fwrite(fid, GSATIMG.DesRng,                               'uint32');
    fwrite(fid, GSATIMG.SceneTrackAngle,                      'int32');
    fwrite(fid, GSATIMG.UserParam,                            'uint8');
    fwrite(fid, GSATIMG.CoarseSNR,                            'int32');
    fwrite(fid, GSATIMG.CoarseAzSubSamp,                      'int32');
    fwrite(fid, GSATIMG.CoarseRngSubSamp,                     'int32');
    fwrite(fid, GSATIMG.MaxAzShift,                           'int32');
    fwrite(fid, GSATIMG.MaxRngShift,                          'int32');
    fwrite(fid, GSATIMG.CoarseDltAz,                          'int32');
    fwrite(fid, GSATIMG.CoarseDltRng,                         'int32');
    fwrite(fid, GSATIMG.TotProcs,                             'int32');
    fwrite(fid, GSATIMG.TptBoxCMode,                          'int32');
    fwrite(fid, GSATIMG.SNRThresh,                            'int32');
    fwrite(fid, GSATIMG.RngSize,                              'int32');
    fwrite(fid, GSATIMG.MapBoxSize,                           'int32');
    fwrite(fid, GSATIMG.BoxSize,                              'int32');
    fwrite(fid, GSATIMG.BoxSpc,                               'int32');
    fwrite(fid, GSATIMG.TotTPts,                              'int32');
    fwrite(fid, GSATIMG.GoodTPts,                             'int32');
    fwrite(fid, GSATIMG.RndSeed,                              'int32');
    fwrite(fid, GSATIMG.RngShift,                             'int32');
    fwrite(fid, GSATIMG.AzShift,                              'int32');
    fwrite(fid, GSATIMG.SumXRamp,                             'int32');
    fwrite(fid, GSATIMG.SumYRamp,                             'int32');
    fwrite(fid, GSATIMG.Cy9kTapeBlock,                        'uint32');
    fwrite(fid, GSATIMG.NominalCenterFreq,                    'single');
    fwrite(fid, GSATIMG.ImageFlags,                           'uint32');
    fwrite(fid, GSATIMG.LineNumber,                           'uint32');
    fwrite(fid, GSATIMG.PatchNumber,                          'uint32');
    fwrite(fid, GSATIMG.Lambda0,                              'single');
    fwrite(fid, GSATIMG.SRngPixSpace,                         'single');
    fwrite(fid, GSATIMG.DoppPixSpace,                         'single');
    fwrite(fid, GSATIMG.DoppOffset,                           'single');
    fwrite(fid, GSATIMG.DoppRngScale,                         'single');
    fwrite(fid, GSATIMG.MuxTimeDelay,                         'single');
    fwrite(fid, GSATIMG.APCXECEF,                             'double');
    fwrite(fid, GSATIMG.APCYECEF,                             'double');
    fwrite(fid, GSATIMG.APCZECEF,                             'double');
    fwrite(fid, GSATIMG.VxECEF,                               'single');
    fwrite(fid, GSATIMG.VyECEF,                               'single');
    fwrite(fid, GSATIMG.VzECEF,                               'single');
    fwrite(fid, GSATIMG.PhaseCal,                             'single');
    fwrite(fid, GSATIMG.SRPxECEF,                             'double');
    fwrite(fid, GSATIMG.SRPyECEF,                             'double');
    fwrite(fid, GSATIMG.SRPzECEF,                             'double');
    fwrite(fid, GSATIMG.Res5,                                 'uint8');
end

function GSATIMG = init_GSATIMG_1_6() %#ok<DEFNU>
    GSATIMG.Version_Minor                     = zeros(1,1,'uint16');          
    GSATIMG.Version_Major                     = zeros(1,1,'uint16');          
    GSATIMG.Length                            = zeros(1,1,'uint32');          
    GSATIMG.Creator_Length                    = zeros(1,1,'uint16');          
    GSATIMG.Creator                           = char(zeros(1,24));            
    GSATIMG.DateTime                          = zeros(1,6,'uint16');          
    GSATIMG.Endian                            = zeros(1,1,'uint16');          
    GSATIMG.BytesPerPixel                     = zeros(1,1,'uint32');          
    GSATIMG.FrameCnt                          = zeros(1,1,'uint32');          
    GSATIMG.ImageType                         = zeros(1,1,'uint32');          
    GSATIMG.RowMajor                          = zeros(1,1,'uint32');          
    GSATIMG.RgCnt                             = zeros(1,1,'uint32');          
    GSATIMG.AzCnt                             = zeros(1,1,'uint32');          
    GSATIMG.ScaleExponent                     = zeros(1,1,'int32');           
    GSATIMG.ScaleMantissa                     = zeros(1,1,'int32');           
    GSATIMG.OffsetExponent                    = zeros(1,1,'int32');           
    GSATIMG.OffsetMantissa                    = zeros(1,1,'int32');           
    GSATIMG.Comment_Length                    = zeros(1,1,'uint16');          
    GSATIMG.Comment                           = char(zeros(1,166));           
    GSATIMG.ImagePlane                        = zeros(1,1,'uint32');          
    GSATIMG.RgPixelSz                         = zeros(1,1,'uint32');          
    GSATIMG.AzPixelSz                         = zeros(1,1,'uint32');          
    GSATIMG.AzOverlap                         = zeros(1,1,'int32');           
    GSATIMG.SRPLat                            = zeros(1,1,'int32');           
    GSATIMG.SRPLong                           = zeros(1,1,'int32');           
    GSATIMG.SRPAlt                            = zeros(1,1,'int32');           
    GSATIMG.RFOA                              = zeros(1,1,'int32');           
    GSATIMG.XtoSRP                            = zeros(1,1,'int32');           
    GSATIMG.PhaseName_Length                  = zeros(1,1,'uint16');          
    GSATIMG.PhaseName                         = char(zeros(1,128));           
    GSATIMG.ImageName_Length                  = zeros(1,1,'uint16');          
    GSATIMG.ImageName                         = char(zeros(1,128));           
    GSATIMG.LookCnt                           = zeros(1,1,'uint32');          
    GSATIMG.ParamRefAp                        = zeros(1,1,'uint32');          
    GSATIMG.ParamRefPos                       = zeros(1,1,'uint32');          
    GSATIMG.GrzAngle                          = zeros(1,1,'uint32');          
    GSATIMG.Squint                            = zeros(1,1,'int32');           
    GSATIMG.GTA                               = zeros(1,1,'int32');           
    GSATIMG.RgBeamCtr                         = zeros(1,1,'uint32');          
    GSATIMG.FlightTime                        = zeros(1,1,'uint32');          
    GSATIMG.RgChirpRate                       = zeros(1,1,'single');          
    GSATIMG.XtoStart                          = zeros(1,1,'int32');           
    GSATIMG.MoCompMode                        = zeros(1,1,'uint32');          
    GSATIMG.V_X                               = zeros(1,1,'uint32');          
    GSATIMG.APCLat                            = zeros(1,1,'int32');           
    GSATIMG.APCLong                           = zeros(1,1,'int32');           
    GSATIMG.APCAlt                            = zeros(1,1,'int32');           
    GSATIMG.Calparm                           = zeros(1,1,'uint32');          
    GSATIMG.LogicalBlkAdrr                    = zeros(1,1,'uint32');          
    GSATIMG.AzResolution                      = zeros(1,1,'uint32');          
    GSATIMG.RgResolution                      = zeros(1,1,'uint32');          
    GSATIMG.DesSigmaN                         = zeros(1,1,'int32');           
    GSATIMG.DesGrazAngle                      = zeros(1,1,'uint32');          
    GSATIMG.DesSquint                         = zeros(1,1,'int32');           
    GSATIMG.DesRng                            = zeros(1,1,'uint32');          
    GSATIMG.SceneTrackAngle                   = zeros(1,1,'int32');           
    GSATIMG.UserParam                         = zeros(1,48,'uint8');          
    GSATIMG.CoarseSNR                         = zeros(1,1,'int32');           
    GSATIMG.CoarseAzSubSamp                   = zeros(1,1,'int32');           
    GSATIMG.CoarseRngSubSamp                  = zeros(1,1,'int32');           
    GSATIMG.MaxAzShift                        = zeros(1,1,'int32');           
    GSATIMG.MaxRngShift                       = zeros(1,1,'int32');           
    GSATIMG.CoarseDltAz                       = zeros(1,1,'int32');           
    GSATIMG.CoarseDltRng                      = zeros(1,1,'int32');           
    GSATIMG.TotProcs                          = zeros(1,1,'int32');           
    GSATIMG.TptBoxCMode                       = zeros(1,1,'int32');           
    GSATIMG.SNRThresh                         = zeros(1,1,'int32');           
    GSATIMG.RngSize                           = zeros(1,1,'int32');           
    GSATIMG.MapBoxSize                        = zeros(1,1,'int32');           
    GSATIMG.BoxSize                           = zeros(1,1,'int32');           
    GSATIMG.BoxSpc                            = zeros(1,1,'int32');           
    GSATIMG.TotTPts                           = zeros(1,1,'int32');           
    GSATIMG.GoodTPts                          = zeros(1,1,'int32');           
    GSATIMG.RndSeed                           = zeros(1,1,'int32');           
    GSATIMG.RngShift                          = zeros(1,1,'int32');           
    GSATIMG.AzShift                           = zeros(1,1,'int32');           
    GSATIMG.SumXRamp                          = zeros(1,1,'int32');           
    GSATIMG.SumYRamp                          = zeros(1,1,'int32');           
    GSATIMG.Cy9kTapeBlock                     = zeros(1,1,'uint32');          
    GSATIMG.NominalCenterFreq                 = zeros(1,1,'single');          
    GSATIMG.ImageFlags                        = zeros(1,1,'uint32');          
    GSATIMG.LineNumber                        = zeros(1,1,'uint32');          
    GSATIMG.PatchNumber                       = zeros(1,1,'uint32');          
    GSATIMG.Lambda0                           = zeros(1,1,'single');          
    GSATIMG.SRngPixSpace                      = zeros(1,1,'single');          
    GSATIMG.DoppPixSpace                      = zeros(1,1,'single');          
    GSATIMG.DoppOffset                        = zeros(1,1,'single');          
    GSATIMG.DoppRngScale                      = zeros(1,1,'single');          
    GSATIMG.MuxTimeDelay                      = zeros(1,1,'single');          
    GSATIMG.APCXECEF                          = zeros(1,1,'double');          
    GSATIMG.APCYECEF                          = zeros(1,1,'double');          
    GSATIMG.APCZECEF                          = zeros(1,1,'double');          
    GSATIMG.VxECEF                            = zeros(1,1,'single');          
    GSATIMG.VyECEF                            = zeros(1,1,'single');          
    GSATIMG.VzECEF                            = zeros(1,1,'single');          
    GSATIMG.PhaseCal                          = zeros(1,1,'single');          
    GSATIMG.SRPxECEF                          = zeros(1,1,'double');          
    GSATIMG.SRPyECEF                          = zeros(1,1,'double');          
    GSATIMG.SRPzECEF                          = zeros(1,1,'double');          
    GSATIMG.Res5                              = char(zeros(1,64));            
    GSATIMG.ver_maj                           = 1;                            
    GSATIMG.ver_min                           = 6;                            
end

function GSATIMG = read_GSATIMG_1_8(buffer) %#ok<DEFNU>
    GSATIMG.Version_Minor                     = typecast(buffer(1:2),'uint16');
    GSATIMG.Version_Major                     = typecast(buffer(3:4),'uint16');
    GSATIMG.Length                            = typecast(buffer(5:8),'uint32');
    GSATIMG.Creator_Length                    = typecast(buffer(9:10),'uint16');
    GSATIMG.Creator                           = typecast(buffer(11:34),'uint8');
    GSATIMG.DateTime                          = typecast(buffer(35:46),'uint16');
    GSATIMG.Endian                            = typecast(buffer(47:48),'uint16');
    GSATIMG.BytesPerPixel                     = typecast(buffer(49:52),'single');
    GSATIMG.FrameCnt                          = typecast(buffer(53:56),'uint32');
    GSATIMG.ImageType                         = typecast(buffer(57:60),'uint32');
    GSATIMG.RowMajor                          = typecast(buffer(61:64),'uint32');
    GSATIMG.RgCnt                             = typecast(buffer(65:68),'uint32');
    GSATIMG.AzCnt                             = typecast(buffer(69:72),'uint32');
    GSATIMG.ScaleExponent                     = typecast(buffer(73:76),'int32');
    GSATIMG.ScaleMantissa                     = typecast(buffer(77:80),'int32');
    GSATIMG.OffsetExponent                    = typecast(buffer(81:84),'int32');
    GSATIMG.OffsetMantissa                    = typecast(buffer(85:88),'int32');
    GSATIMG.Res2                              = typecast(buffer(89:120),'uint8');
    GSATIMG.Comment_Length                    = typecast(buffer(121:122),'uint16');
    GSATIMG.Comment                           = typecast(buffer(123:288),'uint8');
    GSATIMG.ImagePlane                        = typecast(buffer(289:292),'uint32');
    GSATIMG.RgPixelSz                         = typecast(buffer(293:296),'uint32');
    GSATIMG.AzPixelSz                         = typecast(buffer(297:300),'uint32');
    GSATIMG.AzOverlap                         = typecast(buffer(301:304),'int32');
    GSATIMG.SRPLat                            = typecast(buffer(305:308),'int32');
    GSATIMG.SRPLong                           = typecast(buffer(309:312),'int32');
    GSATIMG.SRPAlt                            = typecast(buffer(313:316),'int32');
    GSATIMG.RFOA                              = typecast(buffer(317:320),'int32');
    GSATIMG.XtoSRP                            = typecast(buffer(321:324),'int32');
    GSATIMG.Res3                              = typecast(buffer(325:356),'uint8');
    GSATIMG.PhaseName_Length                  = typecast(buffer(357:358),'uint16');
    GSATIMG.PhaseName                         = typecast(buffer(359:486),'uint8');
    GSATIMG.ImageName_Length                  = typecast(buffer(487:488),'uint16');
    GSATIMG.ImageName                         = typecast(buffer(489:616),'uint8');
    GSATIMG.LookCnt                           = typecast(buffer(617:620),'uint32');
    GSATIMG.ParamRefAp                        = typecast(buffer(621:624),'uint32');
    GSATIMG.ParamRefPos                       = typecast(buffer(625:628),'uint32');
    GSATIMG.GrzAngle                          = typecast(buffer(629:632),'uint32');
    GSATIMG.Squint                            = typecast(buffer(633:636),'int32');
    GSATIMG.GTA                               = typecast(buffer(637:640),'int32');
    GSATIMG.RgBeamCtr                         = typecast(buffer(641:644),'uint32');
    GSATIMG.FlightTime                        = typecast(buffer(645:648),'uint32');
    GSATIMG.RgChirpRate                       = typecast(buffer(649:652),'single');
    GSATIMG.XtoStart                          = typecast(buffer(653:656),'int32');
    GSATIMG.MoCompMode                        = typecast(buffer(657:660),'uint32');
    GSATIMG.V_X                               = typecast(buffer(661:664),'uint32');
    GSATIMG.APCLat                            = typecast(buffer(665:668),'int32');
    GSATIMG.APCLong                           = typecast(buffer(669:672),'int32');
    GSATIMG.APCAlt                            = typecast(buffer(673:676),'int32');
    GSATIMG.Calparm                           = typecast(buffer(677:680),'uint32');
    GSATIMG.LogicalBlkAdrr                    = typecast(buffer(681:684),'uint32');
    GSATIMG.AzResolution                      = typecast(buffer(685:688),'uint32');
    GSATIMG.RgResolution                      = typecast(buffer(689:692),'uint32');
    GSATIMG.DesSigmaN                         = typecast(buffer(693:696),'int32');
    GSATIMG.DesGrazAngle                      = typecast(buffer(697:700),'uint32');
    GSATIMG.DesSquint                         = typecast(buffer(701:704),'int32');
    GSATIMG.DesRng                            = typecast(buffer(705:708),'uint32');
    GSATIMG.SceneTrackAngle                   = typecast(buffer(709:712),'int32');
    GSATIMG.UserParam                         = typecast(buffer(713:760),'uint8');
    GSATIMG.CoarseSNR                         = typecast(buffer(761:764),'int32');
    GSATIMG.CoarseAzSubSamp                   = typecast(buffer(765:768),'int32');
    GSATIMG.CoarseRngSubSamp                  = typecast(buffer(769:772),'int32');
    GSATIMG.MaxAzShift                        = typecast(buffer(773:776),'int32');
    GSATIMG.MaxRngShift                       = typecast(buffer(777:780),'int32');
    GSATIMG.CoarseDltAz                       = typecast(buffer(781:784),'int32');
    GSATIMG.CoarseDltRng                      = typecast(buffer(785:788),'int32');
    GSATIMG.TotProcs                          = typecast(buffer(789:792),'int32');
    GSATIMG.TptBoxCMode                       = typecast(buffer(793:796),'int32');
    GSATIMG.SNRThresh                         = typecast(buffer(797:800),'int32');
    GSATIMG.RngSize                           = typecast(buffer(801:804),'int32');
    GSATIMG.MapBoxSize                        = typecast(buffer(805:808),'int32');
    GSATIMG.BoxSize                           = typecast(buffer(809:812),'int32');
    GSATIMG.BoxSpc                            = typecast(buffer(813:816),'int32');
    GSATIMG.TotTPts                           = typecast(buffer(817:820),'int32');
    GSATIMG.GoodTPts                          = typecast(buffer(821:824),'int32');
    GSATIMG.RndSeed                           = typecast(buffer(825:828),'int32');
    GSATIMG.RngShift                          = typecast(buffer(829:832),'int32');
    GSATIMG.AzShift                           = typecast(buffer(833:836),'int32');
    GSATIMG.SumXRamp                          = typecast(buffer(837:840),'int32');
    GSATIMG.SumYRamp                          = typecast(buffer(841:844),'int32');
    GSATIMG.Cy9kTapeBlock                     = typecast(buffer(845:848),'uint32');
    GSATIMG.NominalCenterFreq                 = typecast(buffer(849:852),'single');
    GSATIMG.ImageFlags                        = typecast(buffer(853:856),'uint32');
    GSATIMG.LineNumber                        = typecast(buffer(857:860),'uint32');
    GSATIMG.PatchNumber                       = typecast(buffer(861:864),'uint32');
    GSATIMG.Lambda0                           = typecast(buffer(865:868),'single');
    GSATIMG.SRngPixSpace                      = typecast(buffer(869:872),'single');
    GSATIMG.DoppPixSpace                      = typecast(buffer(873:876),'single');
    GSATIMG.DoppOffset                        = typecast(buffer(877:880),'single');
    GSATIMG.DoppRngScale                      = typecast(buffer(881:884),'single');
    GSATIMG.MuxTimeDelay                      = typecast(buffer(885:888),'single');
    GSATIMG.APCXECEF                          = typecast(buffer(889:896),'double');
    GSATIMG.APCYECEF                          = typecast(buffer(897:904),'double');
    GSATIMG.APCZECEF                          = typecast(buffer(905:912),'double');
    GSATIMG.VxECEF                            = typecast(buffer(913:916),'single');
    GSATIMG.VyECEF                            = typecast(buffer(917:920),'single');
    GSATIMG.VzECEF                            = typecast(buffer(921:924),'single');
    GSATIMG.PhaseCal                          = typecast(buffer(925:928),'single');
    GSATIMG.SRPxECEF                          = typecast(buffer(929:936),'double');
    GSATIMG.SRPyECEF                          = typecast(buffer(937:944),'double');
    GSATIMG.SRPzECEF                          = typecast(buffer(945:952),'double');
    GSATIMG.Res5                              = typecast(buffer(953:1016),'uint8');
    GSATIMG.HeaderLen1                        = typecast(buffer(1017:1020),'uint32');
    GSATIMG.ImgDate.Year                      = typecast(buffer(1021:1022),'uint16');
    GSATIMG.ImgDate.Month                     = typecast(buffer(1023:1024),'uint16');
    GSATIMG.ImgDate.Day                       = typecast(buffer(1025:1026),'uint16');
    GSATIMG.ImgDate.Hour                      = typecast(buffer(1027:1028),'uint16');
    GSATIMG.ImgDate.Minute                    = typecast(buffer(1029:1030),'uint16');
    GSATIMG.ImgDate.Second                    = typecast(buffer(1031:1032),'uint16');
    GSATIMG.ImgDate.ver_maj                   = 1;                            
    GSATIMG.ImgDate.ver_min                   = 8;                            
    GSATIMG.CompFileName                      = typecast(buffer(1033:1160),'uint8');
    GSATIMG.RefFileName                       = typecast(buffer(1161:1288),'uint8');
    GSATIMG.IEPlatform                        = typecast(buffer(1289:1312),'uint8');
    GSATIMG.IEProcID                          = typecast(buffer(1313:1324),'uint8');
    GSATIMG.IERadarModel                      = typecast(buffer(1325:1336),'uint8');
    GSATIMG.IERadarID                         = typecast(buffer(1337:1340),'uint32');
    GSATIMG.IESWID                            = typecast(buffer(1341:1364),'uint8');
    GSATIMG.IFPlatform                        = typecast(buffer(1365:1388),'uint8');
    GSATIMG.IFProcID                          = typecast(buffer(1389:1400),'uint8');
    GSATIMG.IFRadarModel                      = typecast(buffer(1401:1412),'uint8');
    GSATIMG.IFRadarID                         = typecast(buffer(1413:1416),'uint32');
    GSATIMG.IFSWID                            = typecast(buffer(1417:1440),'uint8');
    GSATIMG.IFAlgo                            = typecast(buffer(1441:1448),'uint8');
    GSATIMG.PHPlatform                        = typecast(buffer(1449:1472),'uint8');
    GSATIMG.PHProcID                          = typecast(buffer(1473:1484),'uint8');
    GSATIMG.PHRadarModel                      = typecast(buffer(1485:1496),'uint8');
    GSATIMG.PHRadarID                         = typecast(buffer(1497:1500),'uint32');
    GSATIMG.PHSWID                            = typecast(buffer(1501:1524),'uint8');
    GSATIMG.PHDataRcd                         = typecast(buffer(1525:1528),'int32');
    GSATIMG.ProcProduct                       = typecast(buffer(1529:1532),'int32');
    GSATIMG.MissionText                       = typecast(buffer(1533:1540),'uint8');
    GSATIMG.PHSource                          = typecast(buffer(1541:1544),'int32');
    GSATIMG.GPSWeek                           = typecast(buffer(1545:1548),'uint32');
    GSATIMG.DataCollectReqH                   = typecast(buffer(1549:1562),'uint8');
    GSATIMG.Res6                              = typecast(buffer(1563:1564),'uint8');
    GSATIMG.GridName                          = typecast(buffer(1565:1588),'uint8');
    GSATIMG.PixValLinearity                   = typecast(buffer(1589:1592),'int32');
    GSATIMG.ComplexOrReal                     = typecast(buffer(1593:1596),'int32');
    GSATIMG.BitsPerMagnitude                  = typecast(buffer(1597:1598),'uint16');
    GSATIMG.BitsPerPhase                      = typecast(buffer(1599:1600),'uint16');
    GSATIMG.ComplexOrderType                  = typecast(buffer(1601:1604),'int32');
    GSATIMG.PixDataType                       = typecast(buffer(1605:1608),'int32');
    GSATIMG.ImageLength                       = typecast(buffer(1609:1612),'int32');
    GSATIMG.ImageCmpScheme                    = typecast(buffer(1613:1616),'int32');
    GSATIMG.APBO                              = typecast(buffer(1617:1620),'single');
    GSATIMG.AsaPitch                          = typecast(buffer(1621:1624),'single');
    GSATIMG.AsaSquint                         = typecast(buffer(1625:1628),'single');
    GSATIMG.DsaPitch                          = typecast(buffer(1629:1632),'single');
    GSATIMG.IRA                               = typecast(buffer(1633:1636),'single');
    GSATIMG.RxPolarization                    = typecast(buffer(1637:1644),'single');
    GSATIMG.TxPolarization                    = typecast(buffer(1645:1652),'single');
    GSATIMG.VxAvg                             = typecast(buffer(1653:1656),'single');
    GSATIMG.VyAvg                             = typecast(buffer(1657:1660),'single');
    GSATIMG.VzAvg                             = typecast(buffer(1661:1664),'single');
    GSATIMG.APCxAvg                           = typecast(buffer(1665:1668),'single');
    GSATIMG.APCyAvg                           = typecast(buffer(1669:1672),'single');
    GSATIMG.APCzAvg                           = typecast(buffer(1673:1676),'single');
    GSATIMG.AveragingTime                     = typecast(buffer(1677:1680),'single');
    GSATIMG.Dgta                              = typecast(buffer(1681:1684),'single');
    GSATIMG.VelocY                            = typecast(buffer(1685:1688),'uint32');
    GSATIMG.VelocZ                            = typecast(buffer(1689:1692),'uint32');
    GSATIMG.Ba                                = typecast(buffer(1693:1696),'single');
    GSATIMG.Be                                = typecast(buffer(1697:1700),'single');
    GSATIMG.AzGeomCorr                        = typecast(buffer(1701:1704),'int32');
    GSATIMG.RngGeomCorr                       = typecast(buffer(1705:1708),'int32');
    GSATIMG.AzWinFacBW                        = typecast(buffer(1709:1712),'single');
    GSATIMG.RngWinFacBW                       = typecast(buffer(1713:1716),'single');
    GSATIMG.AzWinID                           = typecast(buffer(1717:1764),'uint8');
    GSATIMG.RngWinID                          = typecast(buffer(1765:1812),'uint8');
    GSATIMG.KeepOutViolPrcnt                  = typecast(buffer(1813:1816),'single');
    GSATIMG.AzCoeff                           = typecast(buffer(1817:1840),'single');
    GSATIMG.PosUncertDown                     = typecast(buffer(1841:1844),'single');
    GSATIMG.PosUncertE                        = typecast(buffer(1845:1848),'single');
    GSATIMG.PosUncertN                        = typecast(buffer(1849:1852),'single');
    GSATIMG.NavAidingType                     = typecast(buffer(1853:1856),'int32');
    GSATIMG.TwoDNLPhaseCoeffs                 = typecast(buffer(1857:1896),'single');
    GSATIMG.ClutterSNRThresh                  = typecast(buffer(1897:1900),'single');
    GSATIMG.ElevationCoeff                    = typecast(buffer(1901:1936),'single');
    GSATIMG.MonopulseCoeff                    = typecast(buffer(1937:1984),'single');
    GSATIMG.TwistPntErrPrcnt                  = typecast(buffer(1985:1988),'single');
    GSATIMG.TiltPntErrPrcnt                   = typecast(buffer(1989:1992),'single');
    GSATIMG.AzPntErrPrcnt                     = typecast(buffer(1993:1996),'single');
    GSATIMG.SigmaN                            = typecast(buffer(1997:2000),'uint32');
    GSATIMG.TakeNum                           = typecast(buffer(2001:2004),'int32');
    GSATIMG.IFSARFlags                        = typecast(buffer(2005:2024),'int32');
    GSATIMG.MuThreshold                       = typecast(buffer(2025:2028),'single');
    GSATIMG.GffAppType                        = typecast(buffer(2029:2032),'int32');
    GSATIMG.Res7                              = typecast(buffer(2033:2040),'uint8');
    GSATIMG.ver_maj                           = 1;                            
    GSATIMG.ver_min                           = 8;                            
end

function GSATIMG = unmarshal_GSATIMG_1_8(GSATIMG) %#ok<DEFNU>
    GSATIMG.Creator = char(GSATIMG.Creator.');
    GSATIMG.Creator(find(GSATIMG.Creator==0,1,'first'):end)=0;
    GSATIMG.Res2 = char(GSATIMG.Res2.');
    GSATIMG.Res2(find(GSATIMG.Res2==0,1,'first'):end)=0;
    GSATIMG.Comment = char(GSATIMG.Comment.');
    GSATIMG.Comment(find(GSATIMG.Comment==0,1,'first'):end)=0;
    GSATIMG.Res3 = char(GSATIMG.Res3.');
    GSATIMG.Res3(find(GSATIMG.Res3==0,1,'first'):end)=0;
    GSATIMG.PhaseName = char(GSATIMG.PhaseName.');
    GSATIMG.PhaseName(find(GSATIMG.PhaseName==0,1,'first'):end)=0;
    GSATIMG.ImageName = char(GSATIMG.ImageName.');
    GSATIMG.ImageName(find(GSATIMG.ImageName==0,1,'first'):end)=0;
    GSATIMG.Res5 = char(GSATIMG.Res5.');
    GSATIMG.Res5(find(GSATIMG.Res5==0,1,'first'):end)=0;
    GSATIMG.CompFileName = char(GSATIMG.CompFileName.');
    GSATIMG.CompFileName(find(GSATIMG.CompFileName==0,1,'first'):end)=0;
    GSATIMG.RefFileName = char(GSATIMG.RefFileName.');
    GSATIMG.RefFileName(find(GSATIMG.RefFileName==0,1,'first'):end)=0;
    GSATIMG.IEPlatform = char(GSATIMG.IEPlatform.');
    GSATIMG.IEPlatform(find(GSATIMG.IEPlatform==0,1,'first'):end)=0;
    GSATIMG.IEProcID = char(GSATIMG.IEProcID.');
    GSATIMG.IEProcID(find(GSATIMG.IEProcID==0,1,'first'):end)=0;
    GSATIMG.IERadarModel = char(GSATIMG.IERadarModel.');
    GSATIMG.IERadarModel(find(GSATIMG.IERadarModel==0,1,'first'):end)=0;
    GSATIMG.IESWID = char(GSATIMG.IESWID.');
    GSATIMG.IESWID(find(GSATIMG.IESWID==0,1,'first'):end)=0;
    GSATIMG.IFPlatform = char(GSATIMG.IFPlatform.');
    GSATIMG.IFPlatform(find(GSATIMG.IFPlatform==0,1,'first'):end)=0;
    GSATIMG.IFProcID = char(GSATIMG.IFProcID.');
    GSATIMG.IFProcID(find(GSATIMG.IFProcID==0,1,'first'):end)=0;
    GSATIMG.IFRadarModel = char(GSATIMG.IFRadarModel.');
    GSATIMG.IFRadarModel(find(GSATIMG.IFRadarModel==0,1,'first'):end)=0;
    GSATIMG.IFSWID = char(GSATIMG.IFSWID.');
    GSATIMG.IFSWID(find(GSATIMG.IFSWID==0,1,'first'):end)=0;
    GSATIMG.IFAlgo = char(GSATIMG.IFAlgo.');
    GSATIMG.IFAlgo(find(GSATIMG.IFAlgo==0,1,'first'):end)=0;
    GSATIMG.PHPlatform = char(GSATIMG.PHPlatform.');
    GSATIMG.PHPlatform(find(GSATIMG.PHPlatform==0,1,'first'):end)=0;
    GSATIMG.PHProcID = char(GSATIMG.PHProcID.');
    GSATIMG.PHProcID(find(GSATIMG.PHProcID==0,1,'first'):end)=0;
    GSATIMG.PHRadarModel = char(GSATIMG.PHRadarModel.');
    GSATIMG.PHRadarModel(find(GSATIMG.PHRadarModel==0,1,'first'):end)=0;
    GSATIMG.PHSWID = char(GSATIMG.PHSWID.');
    GSATIMG.PHSWID(find(GSATIMG.PHSWID==0,1,'first'):end)=0;
    GSATIMG.MissionText = char(GSATIMG.MissionText.');
    GSATIMG.MissionText(find(GSATIMG.MissionText==0,1,'first'):end)=0;
    GSATIMG.DataCollectReqH = char(GSATIMG.DataCollectReqH.');
    GSATIMG.DataCollectReqH(find(GSATIMG.DataCollectReqH==0,1,'first'):end)=0;
    GSATIMG.Res6 = char(GSATIMG.Res6.');
    GSATIMG.Res6(find(GSATIMG.Res6==0,1,'first'):end)=0;
    GSATIMG.GridName = char(GSATIMG.GridName.');
    GSATIMG.GridName(find(GSATIMG.GridName==0,1,'first'):end)=0;
    GSATIMG.AzWinID = char(GSATIMG.AzWinID.');
    GSATIMG.AzWinID(find(GSATIMG.AzWinID==0,1,'first'):end)=0;
    GSATIMG.RngWinID = char(GSATIMG.RngWinID.');
    GSATIMG.RngWinID(find(GSATIMG.RngWinID==0,1,'first'):end)=0;
    GSATIMG.Res7 = char(GSATIMG.Res7.');
    GSATIMG.Res7(find(GSATIMG.Res7==0,1,'first'):end)=0;
end

function GSATIMG = marshal_GSATIMG_1_8(GSATIMG) %#ok<DEFNU>
    GSATIMG.Creator = marshal_string(GSATIMG.Creator, 24);
    GSATIMG.Res2 = marshal_string(GSATIMG.Res2, 32);
    GSATIMG.Comment = marshal_string(GSATIMG.Comment, 166);
    GSATIMG.Res3 = marshal_string(GSATIMG.Res3, 32);
    GSATIMG.PhaseName = marshal_string(GSATIMG.PhaseName, 128);
    GSATIMG.ImageName = marshal_string(GSATIMG.ImageName, 128);
    GSATIMG.Res5 = marshal_string(GSATIMG.Res5, 64);
    GSATIMG.CompFileName = marshal_string(GSATIMG.CompFileName, 128);
    GSATIMG.RefFileName = marshal_string(GSATIMG.RefFileName, 128);
    GSATIMG.IEPlatform = marshal_string(GSATIMG.IEPlatform, 24);
    GSATIMG.IEProcID = marshal_string(GSATIMG.IEProcID, 12);
    GSATIMG.IERadarModel = marshal_string(GSATIMG.IERadarModel, 12);
    GSATIMG.IESWID = marshal_string(GSATIMG.IESWID, 24);
    GSATIMG.IFPlatform = marshal_string(GSATIMG.IFPlatform, 24);
    GSATIMG.IFProcID = marshal_string(GSATIMG.IFProcID, 12);
    GSATIMG.IFRadarModel = marshal_string(GSATIMG.IFRadarModel, 12);
    GSATIMG.IFSWID = marshal_string(GSATIMG.IFSWID, 24);
    GSATIMG.IFAlgo = marshal_string(GSATIMG.IFAlgo, 8);
    GSATIMG.PHPlatform = marshal_string(GSATIMG.PHPlatform, 24);
    GSATIMG.PHProcID = marshal_string(GSATIMG.PHProcID, 12);
    GSATIMG.PHRadarModel = marshal_string(GSATIMG.PHRadarModel, 12);
    GSATIMG.PHSWID = marshal_string(GSATIMG.PHSWID, 24);
    GSATIMG.MissionText = marshal_string(GSATIMG.MissionText, 8);
    GSATIMG.DataCollectReqH = marshal_string(GSATIMG.DataCollectReqH, 14);
    GSATIMG.Res6 = marshal_string(GSATIMG.Res6, 2);
    GSATIMG.GridName = marshal_string(GSATIMG.GridName, 24);
    GSATIMG.AzWinID = marshal_string(GSATIMG.AzWinID, 48);
    GSATIMG.RngWinID = marshal_string(GSATIMG.RngWinID, 48);
    GSATIMG.Res7 = marshal_string(GSATIMG.Res7, 8);
end

function write_GSATIMG_1_8(fid, GSATIMG) %#ok<DEFNU>
    fwrite(fid, GSATIMG.Version_Minor,                        'uint16');
    fwrite(fid, GSATIMG.Version_Major,                        'uint16');
    fwrite(fid, GSATIMG.Length,                               'uint32');
    fwrite(fid, GSATIMG.Creator_Length,                       'uint16');
    fwrite(fid, GSATIMG.Creator,                              'uint8');
    fwrite(fid, GSATIMG.DateTime,                             'uint16');
    fwrite(fid, GSATIMG.Endian,                               'uint16');
    fwrite(fid, GSATIMG.BytesPerPixel,                        'single');
    fwrite(fid, GSATIMG.FrameCnt,                             'uint32');
    fwrite(fid, GSATIMG.ImageType,                            'uint32');
    fwrite(fid, GSATIMG.RowMajor,                             'uint32');
    fwrite(fid, GSATIMG.RgCnt,                                'uint32');
    fwrite(fid, GSATIMG.AzCnt,                                'uint32');
    fwrite(fid, GSATIMG.ScaleExponent,                        'int32');
    fwrite(fid, GSATIMG.ScaleMantissa,                        'int32');
    fwrite(fid, GSATIMG.OffsetExponent,                       'int32');
    fwrite(fid, GSATIMG.OffsetMantissa,                       'int32');
    fwrite(fid, GSATIMG.Res2,                                 'uint8');
    fwrite(fid, GSATIMG.Comment_Length,                       'uint16');
    fwrite(fid, GSATIMG.Comment,                              'uint8');
    fwrite(fid, GSATIMG.ImagePlane,                           'uint32');
    fwrite(fid, GSATIMG.RgPixelSz,                            'uint32');
    fwrite(fid, GSATIMG.AzPixelSz,                            'uint32');
    fwrite(fid, GSATIMG.AzOverlap,                            'int32');
    fwrite(fid, GSATIMG.SRPLat,                               'int32');
    fwrite(fid, GSATIMG.SRPLong,                              'int32');
    fwrite(fid, GSATIMG.SRPAlt,                               'int32');
    fwrite(fid, GSATIMG.RFOA,                                 'int32');
    fwrite(fid, GSATIMG.XtoSRP,                               'int32');
    fwrite(fid, GSATIMG.Res3,                                 'uint8');
    fwrite(fid, GSATIMG.PhaseName_Length,                     'uint16');
    fwrite(fid, GSATIMG.PhaseName,                            'uint8');
    fwrite(fid, GSATIMG.ImageName_Length,                     'uint16');
    fwrite(fid, GSATIMG.ImageName,                            'uint8');
    fwrite(fid, GSATIMG.LookCnt,                              'uint32');
    fwrite(fid, GSATIMG.ParamRefAp,                           'uint32');
    fwrite(fid, GSATIMG.ParamRefPos,                          'uint32');
    fwrite(fid, GSATIMG.GrzAngle,                             'uint32');
    fwrite(fid, GSATIMG.Squint,                               'int32');
    fwrite(fid, GSATIMG.GTA,                                  'int32');
    fwrite(fid, GSATIMG.RgBeamCtr,                            'uint32');
    fwrite(fid, GSATIMG.FlightTime,                           'uint32');
    fwrite(fid, GSATIMG.RgChirpRate,                          'single');
    fwrite(fid, GSATIMG.XtoStart,                             'int32');
    fwrite(fid, GSATIMG.MoCompMode,                           'uint32');
    fwrite(fid, GSATIMG.V_X,                                  'uint32');
    fwrite(fid, GSATIMG.APCLat,                               'int32');
    fwrite(fid, GSATIMG.APCLong,                              'int32');
    fwrite(fid, GSATIMG.APCAlt,                               'int32');
    fwrite(fid, GSATIMG.Calparm,                              'uint32');
    fwrite(fid, GSATIMG.LogicalBlkAdrr,                       'uint32');
    fwrite(fid, GSATIMG.AzResolution,                         'uint32');
    fwrite(fid, GSATIMG.RgResolution,                         'uint32');
    fwrite(fid, GSATIMG.DesSigmaN,                            'int32');
    fwrite(fid, GSATIMG.DesGrazAngle,                         'uint32');
    fwrite(fid, GSATIMG.DesSquint,                            'int32');
    fwrite(fid, GSATIMG.DesRng,                               'uint32');
    fwrite(fid, GSATIMG.SceneTrackAngle,                      'int32');
    fwrite(fid, GSATIMG.UserParam,                            'uint8');
    fwrite(fid, GSATIMG.CoarseSNR,                            'int32');
    fwrite(fid, GSATIMG.CoarseAzSubSamp,                      'int32');
    fwrite(fid, GSATIMG.CoarseRngSubSamp,                     'int32');
    fwrite(fid, GSATIMG.MaxAzShift,                           'int32');
    fwrite(fid, GSATIMG.MaxRngShift,                          'int32');
    fwrite(fid, GSATIMG.CoarseDltAz,                          'int32');
    fwrite(fid, GSATIMG.CoarseDltRng,                         'int32');
    fwrite(fid, GSATIMG.TotProcs,                             'int32');
    fwrite(fid, GSATIMG.TptBoxCMode,                          'int32');
    fwrite(fid, GSATIMG.SNRThresh,                            'int32');
    fwrite(fid, GSATIMG.RngSize,                              'int32');
    fwrite(fid, GSATIMG.MapBoxSize,                           'int32');
    fwrite(fid, GSATIMG.BoxSize,                              'int32');
    fwrite(fid, GSATIMG.BoxSpc,                               'int32');
    fwrite(fid, GSATIMG.TotTPts,                              'int32');
    fwrite(fid, GSATIMG.GoodTPts,                             'int32');
    fwrite(fid, GSATIMG.RndSeed,                              'int32');
    fwrite(fid, GSATIMG.RngShift,                             'int32');
    fwrite(fid, GSATIMG.AzShift,                              'int32');
    fwrite(fid, GSATIMG.SumXRamp,                             'int32');
    fwrite(fid, GSATIMG.SumYRamp,                             'int32');
    fwrite(fid, GSATIMG.Cy9kTapeBlock,                        'uint32');
    fwrite(fid, GSATIMG.NominalCenterFreq,                    'single');
    fwrite(fid, GSATIMG.ImageFlags,                           'uint32');
    fwrite(fid, GSATIMG.LineNumber,                           'uint32');
    fwrite(fid, GSATIMG.PatchNumber,                          'uint32');
    fwrite(fid, GSATIMG.Lambda0,                              'single');
    fwrite(fid, GSATIMG.SRngPixSpace,                         'single');
    fwrite(fid, GSATIMG.DoppPixSpace,                         'single');
    fwrite(fid, GSATIMG.DoppOffset,                           'single');
    fwrite(fid, GSATIMG.DoppRngScale,                         'single');
    fwrite(fid, GSATIMG.MuxTimeDelay,                         'single');
    fwrite(fid, GSATIMG.APCXECEF,                             'double');
    fwrite(fid, GSATIMG.APCYECEF,                             'double');
    fwrite(fid, GSATIMG.APCZECEF,                             'double');
    fwrite(fid, GSATIMG.VxECEF,                               'single');
    fwrite(fid, GSATIMG.VyECEF,                               'single');
    fwrite(fid, GSATIMG.VzECEF,                               'single');
    fwrite(fid, GSATIMG.PhaseCal,                             'single');
    fwrite(fid, GSATIMG.SRPxECEF,                             'double');
    fwrite(fid, GSATIMG.SRPyECEF,                             'double');
    fwrite(fid, GSATIMG.SRPzECEF,                             'double');
    fwrite(fid, GSATIMG.Res5,                                 'uint8');
    fwrite(fid, GSATIMG.HeaderLen1,                           'uint32');
    fwrite(fid, GSATIMG.ImgDate.Year,                         'uint16');
    fwrite(fid, GSATIMG.ImgDate.Month,                        'uint16');
    fwrite(fid, GSATIMG.ImgDate.Day,                          'uint16');
    fwrite(fid, GSATIMG.ImgDate.Hour,                         'uint16');
    fwrite(fid, GSATIMG.ImgDate.Minute,                       'uint16');
    fwrite(fid, GSATIMG.ImgDate.Second,                       'uint16');
    fwrite(fid, GSATIMG.CompFileName,                         'uint8');
    fwrite(fid, GSATIMG.RefFileName,                          'uint8');
    fwrite(fid, GSATIMG.IEPlatform,                           'uint8');
    fwrite(fid, GSATIMG.IEProcID,                             'uint8');
    fwrite(fid, GSATIMG.IERadarModel,                         'uint8');
    fwrite(fid, GSATIMG.IERadarID,                            'uint32');
    fwrite(fid, GSATIMG.IESWID,                               'uint8');
    fwrite(fid, GSATIMG.IFPlatform,                           'uint8');
    fwrite(fid, GSATIMG.IFProcID,                             'uint8');
    fwrite(fid, GSATIMG.IFRadarModel,                         'uint8');
    fwrite(fid, GSATIMG.IFRadarID,                            'uint32');
    fwrite(fid, GSATIMG.IFSWID,                               'uint8');
    fwrite(fid, GSATIMG.IFAlgo,                               'uint8');
    fwrite(fid, GSATIMG.PHPlatform,                           'uint8');
    fwrite(fid, GSATIMG.PHProcID,                             'uint8');
    fwrite(fid, GSATIMG.PHRadarModel,                         'uint8');
    fwrite(fid, GSATIMG.PHRadarID,                            'uint32');
    fwrite(fid, GSATIMG.PHSWID,                               'uint8');
    fwrite(fid, GSATIMG.PHDataRcd,                            'int32');
    fwrite(fid, GSATIMG.ProcProduct,                          'int32');
    fwrite(fid, GSATIMG.MissionText,                          'uint8');
    fwrite(fid, GSATIMG.PHSource,                             'int32');
    fwrite(fid, GSATIMG.GPSWeek,                              'uint32');
    fwrite(fid, GSATIMG.DataCollectReqH,                      'uint8');
    fwrite(fid, GSATIMG.Res6,                                 'uint8');
    fwrite(fid, GSATIMG.GridName,                             'uint8');
    fwrite(fid, GSATIMG.PixValLinearity,                      'int32');
    fwrite(fid, GSATIMG.ComplexOrReal,                        'int32');
    fwrite(fid, GSATIMG.BitsPerMagnitude,                     'uint16');
    fwrite(fid, GSATIMG.BitsPerPhase,                         'uint16');
    fwrite(fid, GSATIMG.ComplexOrderType,                     'int32');
    fwrite(fid, GSATIMG.PixDataType,                          'int32');
    fwrite(fid, GSATIMG.ImageLength,                          'int32');
    fwrite(fid, GSATIMG.ImageCmpScheme,                       'int32');
    fwrite(fid, GSATIMG.APBO,                                 'single');
    fwrite(fid, GSATIMG.AsaPitch,                             'single');
    fwrite(fid, GSATIMG.AsaSquint,                            'single');
    fwrite(fid, GSATIMG.DsaPitch,                             'single');
    fwrite(fid, GSATIMG.IRA,                                  'single');
    fwrite(fid, GSATIMG.RxPolarization,                       'single');
    fwrite(fid, GSATIMG.TxPolarization,                       'single');
    fwrite(fid, GSATIMG.VxAvg,                                'single');
    fwrite(fid, GSATIMG.VyAvg,                                'single');
    fwrite(fid, GSATIMG.VzAvg,                                'single');
    fwrite(fid, GSATIMG.APCxAvg,                              'single');
    fwrite(fid, GSATIMG.APCyAvg,                              'single');
    fwrite(fid, GSATIMG.APCzAvg,                              'single');
    fwrite(fid, GSATIMG.AveragingTime,                        'single');
    fwrite(fid, GSATIMG.Dgta,                                 'single');
    fwrite(fid, GSATIMG.VelocY,                               'uint32');
    fwrite(fid, GSATIMG.VelocZ,                               'uint32');
    fwrite(fid, GSATIMG.Ba,                                   'single');
    fwrite(fid, GSATIMG.Be,                                   'single');
    fwrite(fid, GSATIMG.AzGeomCorr,                           'int32');
    fwrite(fid, GSATIMG.RngGeomCorr,                          'int32');
    fwrite(fid, GSATIMG.AzWinFacBW,                           'single');
    fwrite(fid, GSATIMG.RngWinFacBW,                          'single');
    fwrite(fid, GSATIMG.AzWinID,                              'uint8');
    fwrite(fid, GSATIMG.RngWinID,                             'uint8');
    fwrite(fid, GSATIMG.KeepOutViolPrcnt,                     'single');
    fwrite(fid, GSATIMG.AzCoeff,                              'single');
    fwrite(fid, GSATIMG.PosUncertDown,                        'single');
    fwrite(fid, GSATIMG.PosUncertE,                           'single');
    fwrite(fid, GSATIMG.PosUncertN,                           'single');
    fwrite(fid, GSATIMG.NavAidingType,                        'int32');
    fwrite(fid, GSATIMG.TwoDNLPhaseCoeffs,                    'single');
    fwrite(fid, GSATIMG.ClutterSNRThresh,                     'single');
    fwrite(fid, GSATIMG.ElevationCoeff,                       'single');
    fwrite(fid, GSATIMG.MonopulseCoeff,                       'single');
    fwrite(fid, GSATIMG.TwistPntErrPrcnt,                     'single');
    fwrite(fid, GSATIMG.TiltPntErrPrcnt,                      'single');
    fwrite(fid, GSATIMG.AzPntErrPrcnt,                        'single');
    fwrite(fid, GSATIMG.SigmaN,                               'uint32');
    fwrite(fid, GSATIMG.TakeNum,                              'int32');
    fwrite(fid, GSATIMG.IFSARFlags,                           'int32');
    fwrite(fid, GSATIMG.MuThreshold,                          'single');
    fwrite(fid, GSATIMG.GffAppType,                           'int32');
    fwrite(fid, GSATIMG.Res7,                                 'uint8');
end

function GSATIMG = init_GSATIMG_1_8() %#ok<DEFNU>
    GSATIMG.Version_Minor                     = zeros(1,1,'uint16');          
    GSATIMG.Version_Major                     = zeros(1,1,'uint16');          
    GSATIMG.Length                            = zeros(1,1,'uint32');          
    GSATIMG.Creator_Length                    = zeros(1,1,'uint16');          
    GSATIMG.Creator                           = char(zeros(1,24));            
    GSATIMG.DateTime                          = zeros(1,6,'uint16');          
    GSATIMG.Endian                            = zeros(1,1,'uint16');          
    GSATIMG.BytesPerPixel                     = zeros(1,1,'single');          
    GSATIMG.FrameCnt                          = zeros(1,1,'uint32');          
    GSATIMG.ImageType                         = zeros(1,1,'uint32');          
    GSATIMG.RowMajor                          = zeros(1,1,'uint32');          
    GSATIMG.RgCnt                             = zeros(1,1,'uint32');          
    GSATIMG.AzCnt                             = zeros(1,1,'uint32');          
    GSATIMG.ScaleExponent                     = zeros(1,1,'int32');           
    GSATIMG.ScaleMantissa                     = zeros(1,1,'int32');           
    GSATIMG.OffsetExponent                    = zeros(1,1,'int32');           
    GSATIMG.OffsetMantissa                    = zeros(1,1,'int32');           
    GSATIMG.Res2                              = char(zeros(1,32));            
    GSATIMG.Comment_Length                    = zeros(1,1,'uint16');          
    GSATIMG.Comment                           = char(zeros(1,166));           
    GSATIMG.ImagePlane                        = zeros(1,1,'uint32');          
    GSATIMG.RgPixelSz                         = zeros(1,1,'uint32');          
    GSATIMG.AzPixelSz                         = zeros(1,1,'uint32');          
    GSATIMG.AzOverlap                         = zeros(1,1,'int32');           
    GSATIMG.SRPLat                            = zeros(1,1,'int32');           
    GSATIMG.SRPLong                           = zeros(1,1,'int32');           
    GSATIMG.SRPAlt                            = zeros(1,1,'int32');           
    GSATIMG.RFOA                              = zeros(1,1,'int32');           
    GSATIMG.XtoSRP                            = zeros(1,1,'int32');           
    GSATIMG.Res3                              = char(zeros(1,32));            
    GSATIMG.PhaseName_Length                  = zeros(1,1,'uint16');          
    GSATIMG.PhaseName                         = char(zeros(1,128));           
    GSATIMG.ImageName_Length                  = zeros(1,1,'uint16');          
    GSATIMG.ImageName                         = char(zeros(1,128));           
    GSATIMG.LookCnt                           = zeros(1,1,'uint32');          
    GSATIMG.ParamRefAp                        = zeros(1,1,'uint32');          
    GSATIMG.ParamRefPos                       = zeros(1,1,'uint32');          
    GSATIMG.GrzAngle                          = zeros(1,1,'uint32');          
    GSATIMG.Squint                            = zeros(1,1,'int32');           
    GSATIMG.GTA                               = zeros(1,1,'int32');           
    GSATIMG.RgBeamCtr                         = zeros(1,1,'uint32');          
    GSATIMG.FlightTime                        = zeros(1,1,'uint32');          
    GSATIMG.RgChirpRate                       = zeros(1,1,'single');          
    GSATIMG.XtoStart                          = zeros(1,1,'int32');           
    GSATIMG.MoCompMode                        = zeros(1,1,'uint32');          
    GSATIMG.V_X                               = zeros(1,1,'uint32');          
    GSATIMG.APCLat                            = zeros(1,1,'int32');           
    GSATIMG.APCLong                           = zeros(1,1,'int32');           
    GSATIMG.APCAlt                            = zeros(1,1,'int32');           
    GSATIMG.Calparm                           = zeros(1,1,'uint32');          
    GSATIMG.LogicalBlkAdrr                    = zeros(1,1,'uint32');          
    GSATIMG.AzResolution                      = zeros(1,1,'uint32');          
    GSATIMG.RgResolution                      = zeros(1,1,'uint32');          
    GSATIMG.DesSigmaN                         = zeros(1,1,'int32');           
    GSATIMG.DesGrazAngle                      = zeros(1,1,'uint32');          
    GSATIMG.DesSquint                         = zeros(1,1,'int32');           
    GSATIMG.DesRng                            = zeros(1,1,'uint32');          
    GSATIMG.SceneTrackAngle                   = zeros(1,1,'int32');           
    GSATIMG.UserParam                         = zeros(1,48,'uint8');          
    GSATIMG.CoarseSNR                         = zeros(1,1,'int32');           
    GSATIMG.CoarseAzSubSamp                   = zeros(1,1,'int32');           
    GSATIMG.CoarseRngSubSamp                  = zeros(1,1,'int32');           
    GSATIMG.MaxAzShift                        = zeros(1,1,'int32');           
    GSATIMG.MaxRngShift                       = zeros(1,1,'int32');           
    GSATIMG.CoarseDltAz                       = zeros(1,1,'int32');           
    GSATIMG.CoarseDltRng                      = zeros(1,1,'int32');           
    GSATIMG.TotProcs                          = zeros(1,1,'int32');           
    GSATIMG.TptBoxCMode                       = zeros(1,1,'int32');           
    GSATIMG.SNRThresh                         = zeros(1,1,'int32');           
    GSATIMG.RngSize                           = zeros(1,1,'int32');           
    GSATIMG.MapBoxSize                        = zeros(1,1,'int32');           
    GSATIMG.BoxSize                           = zeros(1,1,'int32');           
    GSATIMG.BoxSpc                            = zeros(1,1,'int32');           
    GSATIMG.TotTPts                           = zeros(1,1,'int32');           
    GSATIMG.GoodTPts                          = zeros(1,1,'int32');           
    GSATIMG.RndSeed                           = zeros(1,1,'int32');           
    GSATIMG.RngShift                          = zeros(1,1,'int32');           
    GSATIMG.AzShift                           = zeros(1,1,'int32');           
    GSATIMG.SumXRamp                          = zeros(1,1,'int32');           
    GSATIMG.SumYRamp                          = zeros(1,1,'int32');           
    GSATIMG.Cy9kTapeBlock                     = zeros(1,1,'uint32');          
    GSATIMG.NominalCenterFreq                 = zeros(1,1,'single');          
    GSATIMG.ImageFlags                        = zeros(1,1,'uint32');          
    GSATIMG.LineNumber                        = zeros(1,1,'uint32');          
    GSATIMG.PatchNumber                       = zeros(1,1,'uint32');          
    GSATIMG.Lambda0                           = zeros(1,1,'single');          
    GSATIMG.SRngPixSpace                      = zeros(1,1,'single');          
    GSATIMG.DoppPixSpace                      = zeros(1,1,'single');          
    GSATIMG.DoppOffset                        = zeros(1,1,'single');          
    GSATIMG.DoppRngScale                      = zeros(1,1,'single');          
    GSATIMG.MuxTimeDelay                      = zeros(1,1,'single');          
    GSATIMG.APCXECEF                          = zeros(1,1,'double');          
    GSATIMG.APCYECEF                          = zeros(1,1,'double');          
    GSATIMG.APCZECEF                          = zeros(1,1,'double');          
    GSATIMG.VxECEF                            = zeros(1,1,'single');          
    GSATIMG.VyECEF                            = zeros(1,1,'single');          
    GSATIMG.VzECEF                            = zeros(1,1,'single');          
    GSATIMG.PhaseCal                          = zeros(1,1,'single');          
    GSATIMG.SRPxECEF                          = zeros(1,1,'double');          
    GSATIMG.SRPyECEF                          = zeros(1,1,'double');          
    GSATIMG.SRPzECEF                          = zeros(1,1,'double');          
    GSATIMG.Res5                              = char(zeros(1,64));            
    GSATIMG.HeaderLen1                        = zeros(1,1,'uint32');          
    GSATIMG.ImgDate.Year                      = zeros(1,1,'uint16');          
    GSATIMG.ImgDate.Month                     = zeros(1,1,'uint16');          
    GSATIMG.ImgDate.Day                       = zeros(1,1,'uint16');          
    GSATIMG.ImgDate.Hour                      = zeros(1,1,'uint16');          
    GSATIMG.ImgDate.Minute                    = zeros(1,1,'uint16');          
    GSATIMG.ImgDate.Second                    = zeros(1,1,'uint16');          
    GSATIMG.ImgDate.ver_maj                   = 1;                            
    GSATIMG.ImgDate.ver_min                   = 8;                            
    GSATIMG.CompFileName                      = char(zeros(1,128));           
    GSATIMG.RefFileName                       = char(zeros(1,128));           
    GSATIMG.IEPlatform                        = char(zeros(1,24));            
    GSATIMG.IEProcID                          = char(zeros(1,12));            
    GSATIMG.IERadarModel                      = char(zeros(1,12));            
    GSATIMG.IERadarID                         = zeros(1,1,'uint32');          
    GSATIMG.IESWID                            = char(zeros(1,24));            
    GSATIMG.IFPlatform                        = char(zeros(1,24));            
    GSATIMG.IFProcID                          = char(zeros(1,12));            
    GSATIMG.IFRadarModel                      = char(zeros(1,12));            
    GSATIMG.IFRadarID                         = zeros(1,1,'uint32');          
    GSATIMG.IFSWID                            = char(zeros(1,24));            
    GSATIMG.IFAlgo                            = char(zeros(1,8));             
    GSATIMG.PHPlatform                        = char(zeros(1,24));            
    GSATIMG.PHProcID                          = char(zeros(1,12));            
    GSATIMG.PHRadarModel                      = char(zeros(1,12));            
    GSATIMG.PHRadarID                         = zeros(1,1,'uint32');          
    GSATIMG.PHSWID                            = char(zeros(1,24));            
    GSATIMG.PHDataRcd                         = zeros(1,1,'int32');           
    GSATIMG.ProcProduct                       = zeros(1,1,'int32');           
    GSATIMG.MissionText                       = char(zeros(1,8));             
    GSATIMG.PHSource                          = zeros(1,1,'int32');           
    GSATIMG.GPSWeek                           = zeros(1,1,'uint32');          
    GSATIMG.DataCollectReqH                   = char(zeros(1,14));            
    GSATIMG.Res6                              = char(zeros(1,2));             
    GSATIMG.GridName                          = char(zeros(1,24));            
    GSATIMG.PixValLinearity                   = zeros(1,1,'int32');           
    GSATIMG.ComplexOrReal                     = zeros(1,1,'int32');           
    GSATIMG.BitsPerMagnitude                  = zeros(1,1,'uint16');          
    GSATIMG.BitsPerPhase                      = zeros(1,1,'uint16');          
    GSATIMG.ComplexOrderType                  = zeros(1,1,'int32');           
    GSATIMG.PixDataType                       = zeros(1,1,'int32');           
    GSATIMG.ImageLength                       = zeros(1,1,'int32');           
    GSATIMG.ImageCmpScheme                    = zeros(1,1,'int32');           
    GSATIMG.APBO                              = zeros(1,1,'single');          
    GSATIMG.AsaPitch                          = zeros(1,1,'single');          
    GSATIMG.AsaSquint                         = zeros(1,1,'single');          
    GSATIMG.DsaPitch                          = zeros(1,1,'single');          
    GSATIMG.IRA                               = zeros(1,1,'single');          
    GSATIMG.RxPolarization                    = zeros(1,2,'single');          
    GSATIMG.TxPolarization                    = zeros(1,2,'single');          
    GSATIMG.VxAvg                             = zeros(1,1,'single');          
    GSATIMG.VyAvg                             = zeros(1,1,'single');          
    GSATIMG.VzAvg                             = zeros(1,1,'single');          
    GSATIMG.APCxAvg                           = zeros(1,1,'single');          
    GSATIMG.APCyAvg                           = zeros(1,1,'single');          
    GSATIMG.APCzAvg                           = zeros(1,1,'single');          
    GSATIMG.AveragingTime                     = zeros(1,1,'single');          
    GSATIMG.Dgta                              = zeros(1,1,'single');          
    GSATIMG.VelocY                            = zeros(1,1,'uint32');          
    GSATIMG.VelocZ                            = zeros(1,1,'uint32');          
    GSATIMG.Ba                                = zeros(1,1,'single');          
    GSATIMG.Be                                = zeros(1,1,'single');          
    GSATIMG.AzGeomCorr                        = zeros(1,1,'int32');           
    GSATIMG.RngGeomCorr                       = zeros(1,1,'int32');           
    GSATIMG.AzWinFacBW                        = zeros(1,1,'single');          
    GSATIMG.RngWinFacBW                       = zeros(1,1,'single');          
    GSATIMG.AzWinID                           = char(zeros(1,48));            
    GSATIMG.RngWinID                          = char(zeros(1,48));            
    GSATIMG.KeepOutViolPrcnt                  = zeros(1,1,'single');          
    GSATIMG.AzCoeff                           = zeros(1,6,'single');          
    GSATIMG.PosUncertDown                     = zeros(1,1,'single');          
    GSATIMG.PosUncertE                        = zeros(1,1,'single');          
    GSATIMG.PosUncertN                        = zeros(1,1,'single');          
    GSATIMG.NavAidingType                     = zeros(1,1,'int32');           
    GSATIMG.TwoDNLPhaseCoeffs                 = zeros(1,10,'single');         
    GSATIMG.ClutterSNRThresh                  = zeros(1,1,'single');          
    GSATIMG.ElevationCoeff                    = zeros(1,9,'single');          
    GSATIMG.MonopulseCoeff                    = zeros(1,12,'single');         
    GSATIMG.TwistPntErrPrcnt                  = zeros(1,1,'single');          
    GSATIMG.TiltPntErrPrcnt                   = zeros(1,1,'single');          
    GSATIMG.AzPntErrPrcnt                     = zeros(1,1,'single');          
    GSATIMG.SigmaN                            = zeros(1,1,'uint32');          
    GSATIMG.TakeNum                           = zeros(1,1,'int32');           
    GSATIMG.IFSARFlags                        = zeros(1,5,'int32');           
    GSATIMG.MuThreshold                       = zeros(1,1,'single');          
    GSATIMG.GffAppType                        = zeros(1,1,'int32');           
    GSATIMG.Res7                              = char(zeros(1,8));             
    GSATIMG.ver_maj                           = 1;                            
    GSATIMG.ver_min                           = 8;                            
end

function GSATIMG = read_GSATIMG_2_5(buffer) %#ok<DEFNU>
    GSATIMG.endian                            = typecast(buffer(1:4),'uint32');
    GSATIMG.imageCreatorLen                   = typecast(buffer(5:6),'uint16');
    GSATIMG.imageCreator                      = typecast(buffer(7:30),'uint8');
    GSATIMG.rangePixels                       = typecast(buffer(31:34),'uint32');
    GSATIMG.azPixels                          = typecast(buffer(35:38),'uint32');
    GSATIMG.pixOrder                          = typecast(buffer(39:42),'uint32');
    GSATIMG.imageLengthBytes                  = typecast(buffer(43:46),'uint32');
    GSATIMG.imageCompressionScheme            = typecast(buffer(47:50),'uint32');
    GSATIMG.pixDataType                       = typecast(buffer(51:54),'uint32');
    GSATIMG.pixelFormat.compFormat(1).bitSize = typecast(buffer(55:56),'uint16');
    GSATIMG.pixelFormat.compFormat(1).dataType = typecast(buffer(57:60),'uint32');
    GSATIMG.pixelFormat.compFormat(1).ver_maj = 2;                            
    GSATIMG.pixelFormat.compFormat(1).ver_min = 5;                            
    GSATIMG.pixelFormat.compFormat(2).bitSize = typecast(buffer(61:62),'uint16');
    GSATIMG.pixelFormat.compFormat(2).dataType = typecast(buffer(63:66),'uint32');
    GSATIMG.pixelFormat.compFormat(2).ver_maj = 2;                            
    GSATIMG.pixelFormat.compFormat(2).ver_min = 5;                            
    GSATIMG.pixelFormat.cmplxDomain           = typecast(buffer(67:70),'uint32');
    GSATIMG.pixelFormat.numComponents         = typecast(buffer(71:74),'int32');
    GSATIMG.pixelFormat.ver_maj               = 2;                            
    GSATIMG.pixelFormat.ver_min               = 5;                            
    GSATIMG.pixValLin                         = typecast(buffer(75:78),'int32');
    GSATIMG.autoScaleFac                      = typecast(buffer(79:82),'single');
    GSATIMG.ver_maj                           = 2;                            
    GSATIMG.ver_min                           = 5;                            
end

function GSATIMG = unmarshal_GSATIMG_2_5(GSATIMG) %#ok<DEFNU>
    GSATIMG.imageCreator = char(GSATIMG.imageCreator.');
    GSATIMG.imageCreator(find(GSATIMG.imageCreator==0,1,'first'):end)=0;
end

function GSATIMG = marshal_GSATIMG_2_5(GSATIMG) %#ok<DEFNU>
    GSATIMG.imageCreator = marshal_string(GSATIMG.imageCreator, 24);
end

function write_GSATIMG_2_5(fid, GSATIMG) %#ok<DEFNU>
    fwrite(fid, GSATIMG.endian,                               'uint32');
    fwrite(fid, GSATIMG.imageCreatorLen,                      'uint16');
    fwrite(fid, GSATIMG.imageCreator,                         'uint8');
    fwrite(fid, GSATIMG.rangePixels,                          'uint32');
    fwrite(fid, GSATIMG.azPixels,                             'uint32');
    fwrite(fid, GSATIMG.pixOrder,                             'uint32');
    fwrite(fid, GSATIMG.imageLengthBytes,                     'uint32');
    fwrite(fid, GSATIMG.imageCompressionScheme,               'uint32');
    fwrite(fid, GSATIMG.pixDataType,                          'uint32');
    fwrite(fid, GSATIMG.pixelFormat.compFormat(1).bitSize,    'uint16');
    fwrite(fid, GSATIMG.pixelFormat.compFormat(1).dataType,   'uint32');
    if (length(GSATIMG.pixelFormat.compFormat) == 2)
    fwrite(fid, GSATIMG.pixelFormat.compFormat(2).bitSize,    'uint16');
    fwrite(fid, GSATIMG.pixelFormat.compFormat(2).dataType,   'uint32');
    else
    fwrite(fid, GSATIMG.pixelFormat.compFormat(1).bitSize*0,    'uint16');
    fwrite(fid, GSATIMG.pixelFormat.compFormat(1).dataType*0,   'uint32');
    end
    fwrite(fid, GSATIMG.pixelFormat.cmplxDomain,              'uint32');
    fwrite(fid, GSATIMG.pixelFormat.numComponents,            'int32');
    fwrite(fid, GSATIMG.pixValLin,                            'int32');
    fwrite(fid, GSATIMG.autoScaleFac,                         'single');
end

function GSATIMG = init_GSATIMG_2_5() %#ok<DEFNU>
    GSATIMG.endian                            = zeros(1,1,'uint32');          
    GSATIMG.imageCreatorLen                   = zeros(1,1,'uint16');          
    GSATIMG.imageCreator                      = char(zeros(1,24));            
    GSATIMG.rangePixels                       = zeros(1,1,'uint32');          
    GSATIMG.azPixels                          = zeros(1,1,'uint32');          
    GSATIMG.pixOrder                          = zeros(1,1,'uint32');          
    GSATIMG.imageLengthBytes                  = zeros(1,1,'uint32');          
    GSATIMG.imageCompressionScheme            = zeros(1,1,'uint32');          
    GSATIMG.pixDataType                       = zeros(1,1,'uint32');          
    GSATIMG.pixelFormat.compFormat(1).bitSize = zeros(1,1,'uint16');          
    GSATIMG.pixelFormat.compFormat(1).dataType = zeros(1,1,'uint32');          
    GSATIMG.pixelFormat.compFormat(1).ver_maj = 2;                            
    GSATIMG.pixelFormat.compFormat(1).ver_min = 5;                            
    GSATIMG.pixelFormat.compFormat(2).bitSize = zeros(1,1,'uint16');          
    GSATIMG.pixelFormat.compFormat(2).dataType = zeros(1,1,'uint32');          
    GSATIMG.pixelFormat.compFormat(2).ver_maj = 2;                            
    GSATIMG.pixelFormat.compFormat(2).ver_min = 5;                            
    GSATIMG.pixelFormat.cmplxDomain           = zeros(1,1,'uint32');          
    GSATIMG.pixelFormat.numComponents         = zeros(1,1,'int32');           
    GSATIMG.pixelFormat.ver_maj               = 2;                            
    GSATIMG.pixelFormat.ver_min               = 5;                            
    GSATIMG.pixValLin                         = zeros(1,1,'int32');           
    GSATIMG.autoScaleFac                      = zeros(1,1,'single');          
    GSATIMG.ver_maj                           = 2;                            
    GSATIMG.ver_min                           = 5;                            
end

function IFINFO = read_IFINFO_1_0(buffer) %#ok<DEFNU>
    IFINFO.procProduct                        = typecast(buffer(1:4),'uint32');
    IFINFO.imgFileNameLen                     = typecast(buffer(5:6),'uint16');
    IFINFO.imgFileName                        = typecast(buffer(7:134),'uint8');
    IFINFO.azResolution                       = typecast(buffer(135:138),'single');
    IFINFO.rngResolution                      = typecast(buffer(139:142),'single');
    IFINFO.imgCalParam                        = typecast(buffer(143:146),'single');
    IFINFO.sigmaN                             = typecast(buffer(147:150),'single');
    IFINFO.sampLocDCRow                       = typecast(buffer(151:154),'int32');
    IFINFO.sampLocDCCol                       = typecast(buffer(155:158),'int32');
    IFINFO.ifAlgo                             = typecast(buffer(159:166),'uint8');
    IFINFO.imgFlag                            = typecast(buffer(167:170),'int32');
    IFINFO.azCoeff                            = typecast(buffer(171:194),'single');
    IFINFO.elCoeff                            = typecast(buffer(195:230),'single');
    IFINFO.azGeoCorrect                       = typecast(buffer(231:234),'int32');
    IFINFO.rngGeoCorrect                      = typecast(buffer(235:238),'int32');
    IFINFO.wndBwFactAz                        = typecast(buffer(239:242),'single');
    IFINFO.wndBwFactRng                       = typecast(buffer(243:246),'single');
    IFINFO.wndFncIdAz                         = typecast(buffer(247:294),'uint8');
    IFINFO.wndFncIdRng                        = typecast(buffer(295:342),'uint8');
    IFINFO.cmtLen                             = typecast(buffer(343:344),'uint16');
    IFINFO.cmtText                            = typecast(buffer(345:510),'uint8');
    IFINFO.autoFocusInfo                      = typecast(buffer(511:514),'int32');
    IFINFO.ver_maj                            = 1;                            
    IFINFO.ver_min                            = 0;                            
end

function IFINFO = unmarshal_IFINFO_1_0(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = char(IFINFO.imgFileName.');
    IFINFO.imgFileName(find(IFINFO.imgFileName==0,1,'first'):end)=0;
    IFINFO.ifAlgo = char(IFINFO.ifAlgo.');
    IFINFO.ifAlgo(find(IFINFO.ifAlgo==0,1,'first'):end)=0;
    IFINFO.wndFncIdAz = char(IFINFO.wndFncIdAz.');
    IFINFO.wndFncIdAz(find(IFINFO.wndFncIdAz==0,1,'first'):end)=0;
    IFINFO.wndFncIdRng = char(IFINFO.wndFncIdRng.');
    IFINFO.wndFncIdRng(find(IFINFO.wndFncIdRng==0,1,'first'):end)=0;
    IFINFO.cmtText = char(IFINFO.cmtText.');
    IFINFO.cmtText(find(IFINFO.cmtText==0,1,'first'):end)=0;
end

function IFINFO = marshal_IFINFO_1_0(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = marshal_string(IFINFO.imgFileName, 128);
    IFINFO.ifAlgo = marshal_string(IFINFO.ifAlgo, 8);
    IFINFO.wndFncIdAz = marshal_string(IFINFO.wndFncIdAz, 48);
    IFINFO.wndFncIdRng = marshal_string(IFINFO.wndFncIdRng, 48);
    IFINFO.cmtText = marshal_string(IFINFO.cmtText, 166);
end

function write_IFINFO_1_0(fid, IFINFO) %#ok<DEFNU>
    fwrite(fid, IFINFO.procProduct,                           'uint32');
    fwrite(fid, IFINFO.imgFileNameLen,                        'uint16');
    fwrite(fid, IFINFO.imgFileName,                           'uint8');
    fwrite(fid, IFINFO.azResolution,                          'single');
    fwrite(fid, IFINFO.rngResolution,                         'single');
    fwrite(fid, IFINFO.imgCalParam,                           'single');
    fwrite(fid, IFINFO.sigmaN,                                'single');
    fwrite(fid, IFINFO.sampLocDCRow,                          'int32');
    fwrite(fid, IFINFO.sampLocDCCol,                          'int32');
    fwrite(fid, IFINFO.ifAlgo,                                'uint8');
    fwrite(fid, IFINFO.imgFlag,                               'int32');
    fwrite(fid, IFINFO.azCoeff,                               'single');
    fwrite(fid, IFINFO.elCoeff,                               'single');
    fwrite(fid, IFINFO.azGeoCorrect,                          'int32');
    fwrite(fid, IFINFO.rngGeoCorrect,                         'int32');
    fwrite(fid, IFINFO.wndBwFactAz,                           'single');
    fwrite(fid, IFINFO.wndBwFactRng,                          'single');
    fwrite(fid, IFINFO.wndFncIdAz,                            'uint8');
    fwrite(fid, IFINFO.wndFncIdRng,                           'uint8');
    fwrite(fid, IFINFO.cmtLen,                                'uint16');
    fwrite(fid, IFINFO.cmtText,                               'uint8');
    fwrite(fid, IFINFO.autoFocusInfo,                         'int32');
end

function IFINFO = init_IFINFO_1_0() %#ok<DEFNU>
    IFINFO.procProduct                        = zeros(1,1,'uint32');          
    IFINFO.imgFileNameLen                     = zeros(1,1,'uint16');          
    IFINFO.imgFileName                        = char(zeros(1,128));           
    IFINFO.azResolution                       = zeros(1,1,'single');          
    IFINFO.rngResolution                      = zeros(1,1,'single');          
    IFINFO.imgCalParam                        = zeros(1,1,'single');          
    IFINFO.sigmaN                             = zeros(1,1,'single');          
    IFINFO.sampLocDCRow                       = zeros(1,1,'int32');           
    IFINFO.sampLocDCCol                       = zeros(1,1,'int32');           
    IFINFO.ifAlgo                             = char(zeros(1,8));             
    IFINFO.imgFlag                            = zeros(1,1,'int32');           
    IFINFO.azCoeff                            = zeros(1,6,'single');          
    IFINFO.elCoeff                            = zeros(1,9,'single');          
    IFINFO.azGeoCorrect                       = zeros(1,1,'int32');           
    IFINFO.rngGeoCorrect                      = zeros(1,1,'int32');           
    IFINFO.wndBwFactAz                        = zeros(1,1,'single');          
    IFINFO.wndBwFactRng                       = zeros(1,1,'single');          
    IFINFO.wndFncIdAz                         = char(zeros(1,48));            
    IFINFO.wndFncIdRng                        = char(zeros(1,48));            
    IFINFO.cmtLen                             = zeros(1,1,'uint16');          
    IFINFO.cmtText                            = char(zeros(1,166));           
    IFINFO.autoFocusInfo                      = zeros(1,1,'int32');           
    IFINFO.ver_maj                            = 1;                            
    IFINFO.ver_min                            = 0;                            
end

function IFINFO = read_IFINFO_2_0(buffer) %#ok<DEFNU>
    IFINFO.procProduct                        = typecast(buffer(1:4),'uint32');
    IFINFO.imgFileNameLen                     = typecast(buffer(5:6),'uint16');
    IFINFO.imgFileName                        = typecast(buffer(7:134),'uint8');
    IFINFO.azResolution                       = typecast(buffer(135:138),'single');
    IFINFO.rngResolution                      = typecast(buffer(139:142),'single');
    IFINFO.imgCalParam                        = typecast(buffer(143:146),'single');
    IFINFO.sigmaN                             = typecast(buffer(147:150),'single');
    IFINFO.sampLocDCRow                       = typecast(buffer(151:154),'int32');
    IFINFO.sampLocDCCol                       = typecast(buffer(155:158),'int32');
    IFINFO.ifAlgo                             = typecast(buffer(159:166),'uint8');
    IFINFO.imgFlag                            = typecast(buffer(167:170),'int32');
    IFINFO.azCoeff                            = typecast(buffer(171:194),'single');
    IFINFO.rngCoeff                           = typecast(buffer(195:230),'single');
    IFINFO.azGeoCorrect                       = typecast(buffer(231:234),'int32');
    IFINFO.rngGeoCorrect                      = typecast(buffer(235:238),'int32');
    IFINFO.wndBwFactAz                        = typecast(buffer(239:242),'single');
    IFINFO.wndBwFactRng                       = typecast(buffer(243:246),'single');
    IFINFO.wndFncIdAz                         = typecast(buffer(247:294),'uint8');
    IFINFO.wndFncIdRng                        = typecast(buffer(295:342),'uint8');
    IFINFO.cmtLen                             = typecast(buffer(343:344),'uint16');
    IFINFO.cmtText                            = typecast(buffer(345:510),'uint8');
    IFINFO.autoFocusInfo                      = typecast(buffer(511:514),'int32');
    IFINFO.rngFFTSize                         = typecast(buffer(515:518),'int32');
    IFINFO.RangePaneFilterCoeff               = typecast(buffer(519:562),'single');
    IFINFO.AzPreFilterCoeff                   = typecast(buffer(563:582),'single');
    IFINFO.ver_maj                            = 2;                            
    IFINFO.ver_min                            = 0;                            
end

function IFINFO = unmarshal_IFINFO_2_0(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = char(IFINFO.imgFileName.');
    IFINFO.imgFileName(find(IFINFO.imgFileName==0,1,'first'):end)=0;
    IFINFO.ifAlgo = char(IFINFO.ifAlgo.');
    IFINFO.ifAlgo(find(IFINFO.ifAlgo==0,1,'first'):end)=0;
    IFINFO.wndFncIdAz = char(IFINFO.wndFncIdAz.');
    IFINFO.wndFncIdAz(find(IFINFO.wndFncIdAz==0,1,'first'):end)=0;
    IFINFO.wndFncIdRng = char(IFINFO.wndFncIdRng.');
    IFINFO.wndFncIdRng(find(IFINFO.wndFncIdRng==0,1,'first'):end)=0;
    IFINFO.cmtText = char(IFINFO.cmtText.');
    IFINFO.cmtText(find(IFINFO.cmtText==0,1,'first'):end)=0;
end

function IFINFO = marshal_IFINFO_2_0(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = marshal_string(IFINFO.imgFileName, 128);
    IFINFO.ifAlgo = marshal_string(IFINFO.ifAlgo, 8);
    IFINFO.wndFncIdAz = marshal_string(IFINFO.wndFncIdAz, 48);
    IFINFO.wndFncIdRng = marshal_string(IFINFO.wndFncIdRng, 48);
    IFINFO.cmtText = marshal_string(IFINFO.cmtText, 166);
end

function write_IFINFO_2_0(fid, IFINFO) %#ok<DEFNU>
    fwrite(fid, IFINFO.procProduct,                           'uint32');
    fwrite(fid, IFINFO.imgFileNameLen,                        'uint16');
    fwrite(fid, IFINFO.imgFileName,                           'uint8');
    fwrite(fid, IFINFO.azResolution,                          'single');
    fwrite(fid, IFINFO.rngResolution,                         'single');
    fwrite(fid, IFINFO.imgCalParam,                           'single');
    fwrite(fid, IFINFO.sigmaN,                                'single');
    fwrite(fid, IFINFO.sampLocDCRow,                          'int32');
    fwrite(fid, IFINFO.sampLocDCCol,                          'int32');
    fwrite(fid, IFINFO.ifAlgo,                                'uint8');
    fwrite(fid, IFINFO.imgFlag,                               'int32');
    fwrite(fid, IFINFO.azCoeff,                               'single');
    fwrite(fid, IFINFO.rngCoeff,                              'single');
    fwrite(fid, IFINFO.azGeoCorrect,                          'int32');
    fwrite(fid, IFINFO.rngGeoCorrect,                         'int32');
    fwrite(fid, IFINFO.wndBwFactAz,                           'single');
    fwrite(fid, IFINFO.wndBwFactRng,                          'single');
    fwrite(fid, IFINFO.wndFncIdAz,                            'uint8');
    fwrite(fid, IFINFO.wndFncIdRng,                           'uint8');
    fwrite(fid, IFINFO.cmtLen,                                'uint16');
    fwrite(fid, IFINFO.cmtText,                               'uint8');
    fwrite(fid, IFINFO.autoFocusInfo,                         'int32');
    fwrite(fid, IFINFO.rngFFTSize,                            'int32');
    fwrite(fid, IFINFO.RangePaneFilterCoeff,                  'single');
    fwrite(fid, IFINFO.AzPreFilterCoeff,                      'single');
end

function IFINFO = init_IFINFO_2_0() %#ok<DEFNU>
    IFINFO.procProduct                        = zeros(1,1,'uint32');          
    IFINFO.imgFileNameLen                     = zeros(1,1,'uint16');          
    IFINFO.imgFileName                        = char(zeros(1,128));           
    IFINFO.azResolution                       = zeros(1,1,'single');          
    IFINFO.rngResolution                      = zeros(1,1,'single');          
    IFINFO.imgCalParam                        = zeros(1,1,'single');          
    IFINFO.sigmaN                             = zeros(1,1,'single');          
    IFINFO.sampLocDCRow                       = zeros(1,1,'int32');           
    IFINFO.sampLocDCCol                       = zeros(1,1,'int32');           
    IFINFO.ifAlgo                             = char(zeros(1,8));             
    IFINFO.imgFlag                            = zeros(1,1,'int32');           
    IFINFO.azCoeff                            = zeros(1,6,'single');          
    IFINFO.rngCoeff                           = zeros(1,9,'single');          
    IFINFO.azGeoCorrect                       = zeros(1,1,'int32');           
    IFINFO.rngGeoCorrect                      = zeros(1,1,'int32');           
    IFINFO.wndBwFactAz                        = zeros(1,1,'single');          
    IFINFO.wndBwFactRng                       = zeros(1,1,'single');          
    IFINFO.wndFncIdAz                         = char(zeros(1,48));            
    IFINFO.wndFncIdRng                        = char(zeros(1,48));            
    IFINFO.cmtLen                             = zeros(1,1,'uint16');          
    IFINFO.cmtText                            = char(zeros(1,166));           
    IFINFO.autoFocusInfo                      = zeros(1,1,'int32');           
    IFINFO.rngFFTSize                         = zeros(1,1,'int32');           
    IFINFO.RangePaneFilterCoeff               = zeros(1,11,'single');         
    IFINFO.AzPreFilterCoeff                   = zeros(1,5,'single');          
    IFINFO.ver_maj                            = 2;                            
    IFINFO.ver_min                            = 0;                            
end

function IFINFO = read_IFINFO_2_1(buffer) %#ok<DEFNU>
    IFINFO.procProduct                        = typecast(buffer(1:4),'uint32');
    IFINFO.imgFileNameLen                     = typecast(buffer(5:6),'uint16');
    IFINFO.imgFileName                        = typecast(buffer(7:134),'uint8');
    IFINFO.azResolution                       = typecast(buffer(135:138),'single');
    IFINFO.rngResolution                      = typecast(buffer(139:142),'single');
    IFINFO.imgCalParam                        = typecast(buffer(143:146),'single');
    IFINFO.sigmaN                             = typecast(buffer(147:150),'single');
    IFINFO.sampLocDCRow                       = typecast(buffer(151:154),'int32');
    IFINFO.sampLocDCCol                       = typecast(buffer(155:158),'int32');
    IFINFO.ifAlgo                             = typecast(buffer(159:166),'uint8');
    IFINFO.imgFlag                            = typecast(buffer(167:170),'int32');
    IFINFO.azCoeff                            = typecast(buffer(171:194),'single');
    IFINFO.elCoeff                            = typecast(buffer(195:230),'single');
    IFINFO.azGeoCorrect                       = typecast(buffer(231:234),'int32');
    IFINFO.rngGeoCorrect                      = typecast(buffer(235:238),'int32');
    IFINFO.wndBwFactAz                        = typecast(buffer(239:242),'single');
    IFINFO.wndBwFactRng                       = typecast(buffer(243:246),'single');
    IFINFO.wndFncIdAz                         = typecast(buffer(247:294),'uint8');
    IFINFO.wndFncIdRng                        = typecast(buffer(295:342),'uint8');
    IFINFO.cmtLen                             = typecast(buffer(343:344),'uint16');
    IFINFO.cmtText                            = typecast(buffer(345:510),'uint8');
    IFINFO.autoFocusInfo                      = typecast(buffer(511:514),'int32');
    IFINFO.rngFFTSize                         = typecast(buffer(515:518),'int32');
    IFINFO.RangePaneFilterCoeff               = typecast(buffer(519:562),'single');
    IFINFO.AzPreFilterCoeff                   = typecast(buffer(563:582),'single');
    IFINFO.ver_maj                            = 2;                            
    IFINFO.ver_min                            = 1;                            
end

function IFINFO = unmarshal_IFINFO_2_1(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = char(IFINFO.imgFileName.');
    IFINFO.imgFileName(find(IFINFO.imgFileName==0,1,'first'):end)=0;
    IFINFO.ifAlgo = char(IFINFO.ifAlgo.');
    IFINFO.ifAlgo(find(IFINFO.ifAlgo==0,1,'first'):end)=0;
    IFINFO.wndFncIdAz = char(IFINFO.wndFncIdAz.');
    IFINFO.wndFncIdAz(find(IFINFO.wndFncIdAz==0,1,'first'):end)=0;
    IFINFO.wndFncIdRng = char(IFINFO.wndFncIdRng.');
    IFINFO.wndFncIdRng(find(IFINFO.wndFncIdRng==0,1,'first'):end)=0;
    IFINFO.cmtText = char(IFINFO.cmtText.');
    IFINFO.cmtText(find(IFINFO.cmtText==0,1,'first'):end)=0;
end

function IFINFO = marshal_IFINFO_2_1(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = marshal_string(IFINFO.imgFileName, 128);
    IFINFO.ifAlgo = marshal_string(IFINFO.ifAlgo, 8);
    IFINFO.wndFncIdAz = marshal_string(IFINFO.wndFncIdAz, 48);
    IFINFO.wndFncIdRng = marshal_string(IFINFO.wndFncIdRng, 48);
    IFINFO.cmtText = marshal_string(IFINFO.cmtText, 166);
end

function write_IFINFO_2_1(fid, IFINFO) %#ok<DEFNU>
    fwrite(fid, IFINFO.procProduct,                           'uint32');
    fwrite(fid, IFINFO.imgFileNameLen,                        'uint16');
    fwrite(fid, IFINFO.imgFileName,                           'uint8');
    fwrite(fid, IFINFO.azResolution,                          'single');
    fwrite(fid, IFINFO.rngResolution,                         'single');
    fwrite(fid, IFINFO.imgCalParam,                           'single');
    fwrite(fid, IFINFO.sigmaN,                                'single');
    fwrite(fid, IFINFO.sampLocDCRow,                          'int32');
    fwrite(fid, IFINFO.sampLocDCCol,                          'int32');
    fwrite(fid, IFINFO.ifAlgo,                                'uint8');
    fwrite(fid, IFINFO.imgFlag,                               'int32');
    fwrite(fid, IFINFO.azCoeff,                               'single');
    fwrite(fid, IFINFO.elCoeff,                               'single');
    fwrite(fid, IFINFO.azGeoCorrect,                          'int32');
    fwrite(fid, IFINFO.rngGeoCorrect,                         'int32');
    fwrite(fid, IFINFO.wndBwFactAz,                           'single');
    fwrite(fid, IFINFO.wndBwFactRng,                          'single');
    fwrite(fid, IFINFO.wndFncIdAz,                            'uint8');
    fwrite(fid, IFINFO.wndFncIdRng,                           'uint8');
    fwrite(fid, IFINFO.cmtLen,                                'uint16');
    fwrite(fid, IFINFO.cmtText,                               'uint8');
    fwrite(fid, IFINFO.autoFocusInfo,                         'int32');
    fwrite(fid, IFINFO.rngFFTSize,                            'int32');
    fwrite(fid, IFINFO.RangePaneFilterCoeff,                  'single');
    fwrite(fid, IFINFO.AzPreFilterCoeff,                      'single');
end

function IFINFO = init_IFINFO_2_1() %#ok<DEFNU>
    IFINFO.procProduct                        = zeros(1,1,'uint32');          
    IFINFO.imgFileNameLen                     = zeros(1,1,'uint16');          
    IFINFO.imgFileName                        = char(zeros(1,128));           
    IFINFO.azResolution                       = zeros(1,1,'single');          
    IFINFO.rngResolution                      = zeros(1,1,'single');          
    IFINFO.imgCalParam                        = zeros(1,1,'single');          
    IFINFO.sigmaN                             = zeros(1,1,'single');          
    IFINFO.sampLocDCRow                       = zeros(1,1,'int32');           
    IFINFO.sampLocDCCol                       = zeros(1,1,'int32');           
    IFINFO.ifAlgo                             = char(zeros(1,8));             
    IFINFO.imgFlag                            = zeros(1,1,'int32');           
    IFINFO.azCoeff                            = zeros(1,6,'single');          
    IFINFO.elCoeff                            = zeros(1,9,'single');          
    IFINFO.azGeoCorrect                       = zeros(1,1,'int32');           
    IFINFO.rngGeoCorrect                      = zeros(1,1,'int32');           
    IFINFO.wndBwFactAz                        = zeros(1,1,'single');          
    IFINFO.wndBwFactRng                       = zeros(1,1,'single');          
    IFINFO.wndFncIdAz                         = char(zeros(1,48));            
    IFINFO.wndFncIdRng                        = char(zeros(1,48));            
    IFINFO.cmtLen                             = zeros(1,1,'uint16');          
    IFINFO.cmtText                            = char(zeros(1,166));           
    IFINFO.autoFocusInfo                      = zeros(1,1,'int32');           
    IFINFO.rngFFTSize                         = zeros(1,1,'int32');           
    IFINFO.RangePaneFilterCoeff               = zeros(1,11,'single');         
    IFINFO.AzPreFilterCoeff                   = zeros(1,5,'single');          
    IFINFO.ver_maj                            = 2;                            
    IFINFO.ver_min                            = 1;                            
end

function IFINFO = read_IFINFO_2_2(buffer) %#ok<DEFNU>
    IFINFO.procProduct                        = typecast(buffer(1:4),'uint32');
    IFINFO.imgFileNameLen                     = typecast(buffer(5:6),'uint16');
    IFINFO.imgFileName                        = typecast(buffer(7:134),'uint8');
    IFINFO.azResolution                       = typecast(buffer(135:138),'single');
    IFINFO.rngResolution                      = typecast(buffer(139:142),'single');
    IFINFO.imgCalParam                        = typecast(buffer(143:146),'single');
    IFINFO.sigmaN                             = typecast(buffer(147:150),'single');
    IFINFO.sampLocDCRow                       = typecast(buffer(151:154),'int32');
    IFINFO.sampLocDCCol                       = typecast(buffer(155:158),'int32');
    IFINFO.ifAlgo                             = typecast(buffer(159:166),'uint8');
    IFINFO.imgFlag                            = typecast(buffer(167:170),'int32');
    IFINFO.azCoeff                            = typecast(buffer(171:194),'single');
    IFINFO.elCoeff                            = typecast(buffer(195:230),'single');
    IFINFO.azGeoCorrect                       = typecast(buffer(231:234),'int32');
    IFINFO.rngGeoCorrect                      = typecast(buffer(235:238),'int32');
    IFINFO.wndBwFactAz                        = typecast(buffer(239:242),'single');
    IFINFO.wndBwFactRng                       = typecast(buffer(243:246),'single');
    IFINFO.wndFncIdAz                         = typecast(buffer(247:294),'uint8');
    IFINFO.wndFncIdRng                        = typecast(buffer(295:342),'uint8');
    IFINFO.cmtLen                             = typecast(buffer(343:344),'uint16');
    IFINFO.cmtText                            = typecast(buffer(345:510),'uint8');
    IFINFO.autoFocusInfo                      = typecast(buffer(511:514),'int32');
    IFINFO.rngFFTSize                         = typecast(buffer(515:518),'int32');
    IFINFO.RangePaneFilterCoeff               = typecast(buffer(519:562),'single');
    IFINFO.AzPreFilterCoeff                   = typecast(buffer(563:582),'single');
    IFINFO.ver_maj                            = 2;                            
    IFINFO.ver_min                            = 2;                            
end

function IFINFO = unmarshal_IFINFO_2_2(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = char(IFINFO.imgFileName.');
    IFINFO.imgFileName(find(IFINFO.imgFileName==0,1,'first'):end)=0;
    IFINFO.ifAlgo = char(IFINFO.ifAlgo.');
    IFINFO.ifAlgo(find(IFINFO.ifAlgo==0,1,'first'):end)=0;
    IFINFO.wndFncIdAz = char(IFINFO.wndFncIdAz.');
    IFINFO.wndFncIdAz(find(IFINFO.wndFncIdAz==0,1,'first'):end)=0;
    IFINFO.wndFncIdRng = char(IFINFO.wndFncIdRng.');
    IFINFO.wndFncIdRng(find(IFINFO.wndFncIdRng==0,1,'first'):end)=0;
    IFINFO.cmtText = char(IFINFO.cmtText.');
    IFINFO.cmtText(find(IFINFO.cmtText==0,1,'first'):end)=0;
end

function IFINFO = marshal_IFINFO_2_2(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = marshal_string(IFINFO.imgFileName, 128);
    IFINFO.ifAlgo = marshal_string(IFINFO.ifAlgo, 8);
    IFINFO.wndFncIdAz = marshal_string(IFINFO.wndFncIdAz, 48);
    IFINFO.wndFncIdRng = marshal_string(IFINFO.wndFncIdRng, 48);
    IFINFO.cmtText = marshal_string(IFINFO.cmtText, 166);
end

function write_IFINFO_2_2(fid, IFINFO) %#ok<DEFNU>
    fwrite(fid, IFINFO.procProduct,                           'uint32');
    fwrite(fid, IFINFO.imgFileNameLen,                        'uint16');
    fwrite(fid, IFINFO.imgFileName,                           'uint8');
    fwrite(fid, IFINFO.azResolution,                          'single');
    fwrite(fid, IFINFO.rngResolution,                         'single');
    fwrite(fid, IFINFO.imgCalParam,                           'single');
    fwrite(fid, IFINFO.sigmaN,                                'single');
    fwrite(fid, IFINFO.sampLocDCRow,                          'int32');
    fwrite(fid, IFINFO.sampLocDCCol,                          'int32');
    fwrite(fid, IFINFO.ifAlgo,                                'uint8');
    fwrite(fid, IFINFO.imgFlag,                               'int32');
    fwrite(fid, IFINFO.azCoeff,                               'single');
    fwrite(fid, IFINFO.elCoeff,                               'single');
    fwrite(fid, IFINFO.azGeoCorrect,                          'int32');
    fwrite(fid, IFINFO.rngGeoCorrect,                         'int32');
    fwrite(fid, IFINFO.wndBwFactAz,                           'single');
    fwrite(fid, IFINFO.wndBwFactRng,                          'single');
    fwrite(fid, IFINFO.wndFncIdAz,                            'uint8');
    fwrite(fid, IFINFO.wndFncIdRng,                           'uint8');
    fwrite(fid, IFINFO.cmtLen,                                'uint16');
    fwrite(fid, IFINFO.cmtText,                               'uint8');
    fwrite(fid, IFINFO.autoFocusInfo,                         'int32');
    fwrite(fid, IFINFO.rngFFTSize,                            'int32');
    fwrite(fid, IFINFO.RangePaneFilterCoeff,                  'single');
    fwrite(fid, IFINFO.AzPreFilterCoeff,                      'single');
end

function IFINFO = init_IFINFO_2_2() %#ok<DEFNU>
    IFINFO.procProduct                        = zeros(1,1,'uint32');          
    IFINFO.imgFileNameLen                     = zeros(1,1,'uint16');          
    IFINFO.imgFileName                        = char(zeros(1,128));           
    IFINFO.azResolution                       = zeros(1,1,'single');          
    IFINFO.rngResolution                      = zeros(1,1,'single');          
    IFINFO.imgCalParam                        = zeros(1,1,'single');          
    IFINFO.sigmaN                             = zeros(1,1,'single');          
    IFINFO.sampLocDCRow                       = zeros(1,1,'int32');           
    IFINFO.sampLocDCCol                       = zeros(1,1,'int32');           
    IFINFO.ifAlgo                             = char(zeros(1,8));             
    IFINFO.imgFlag                            = zeros(1,1,'int32');           
    IFINFO.azCoeff                            = zeros(1,6,'single');          
    IFINFO.elCoeff                            = zeros(1,9,'single');          
    IFINFO.azGeoCorrect                       = zeros(1,1,'int32');           
    IFINFO.rngGeoCorrect                      = zeros(1,1,'int32');           
    IFINFO.wndBwFactAz                        = zeros(1,1,'single');          
    IFINFO.wndBwFactRng                       = zeros(1,1,'single');          
    IFINFO.wndFncIdAz                         = char(zeros(1,48));            
    IFINFO.wndFncIdRng                        = char(zeros(1,48));            
    IFINFO.cmtLen                             = zeros(1,1,'uint16');          
    IFINFO.cmtText                            = char(zeros(1,166));           
    IFINFO.autoFocusInfo                      = zeros(1,1,'int32');           
    IFINFO.rngFFTSize                         = zeros(1,1,'int32');           
    IFINFO.RangePaneFilterCoeff               = zeros(1,11,'single');         
    IFINFO.AzPreFilterCoeff                   = zeros(1,5,'single');          
    IFINFO.ver_maj                            = 2;                            
    IFINFO.ver_min                            = 2;                            
end

function IFINFO = read_IFINFO_3_0(buffer) %#ok<DEFNU>
    IFINFO.procProduct                        = typecast(buffer(1:4),'uint32');
    IFINFO.imgFileNameLen                     = typecast(buffer(5:6),'uint16');
    IFINFO.imgFileName                        = typecast(buffer(7:134),'uint8');
    IFINFO.azResolution                       = typecast(buffer(135:138),'single');
    IFINFO.rngResolution                      = typecast(buffer(139:142),'single');
    IFINFO.imgCalParam                        = typecast(buffer(143:146),'single');
    IFINFO.sigmaN                             = typecast(buffer(147:150),'single');
    IFINFO.sampLocDCRow                       = typecast(buffer(151:154),'int32');
    IFINFO.sampLocDCCol                       = typecast(buffer(155:158),'int32');
    IFINFO.ifAlgo                             = typecast(buffer(159:166),'uint8');
    IFINFO.imgFlag                            = typecast(buffer(167:170),'int32');
    IFINFO.azCoeff                            = typecast(buffer(171:194),'single');
    IFINFO.elCoeff                            = typecast(buffer(195:230),'single');
    IFINFO.azGeoCorrect                       = typecast(buffer(231:234),'int32');
    IFINFO.rngGeoCorrect                      = typecast(buffer(235:238),'int32');
    IFINFO.wndBwFactAz                        = typecast(buffer(239:242),'single');
    IFINFO.wndBwFactRng                       = typecast(buffer(243:246),'single');
    IFINFO.wndFncIdAz                         = typecast(buffer(247:294),'uint8');
    IFINFO.wndFncIdRng                        = typecast(buffer(295:342),'uint8');
    IFINFO.cmtLen                             = typecast(buffer(343:344),'uint16');
    IFINFO.cmtText                            = typecast(buffer(345:510),'uint8');
    IFINFO.autoFocusInfo                      = typecast(buffer(511:514),'int32');
    IFINFO.rngFFTSize                         = typecast(buffer(515:518),'int32');
    IFINFO.RangePaneFilterCoeff               = typecast(buffer(519:562),'single');
    IFINFO.AzPreFilterCoeff                   = typecast(buffer(563:582),'single');
    IFINFO.afPeakQuadComp                     = typecast(buffer(583:586),'single');
    IFINFO.ver_maj                            = 3;                            
    IFINFO.ver_min                            = 0;                            
end

function IFINFO = unmarshal_IFINFO_3_0(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = char(IFINFO.imgFileName.');
    IFINFO.imgFileName(find(IFINFO.imgFileName==0,1,'first'):end)=0;
    IFINFO.ifAlgo = char(IFINFO.ifAlgo.');
    IFINFO.ifAlgo(find(IFINFO.ifAlgo==0,1,'first'):end)=0;
    IFINFO.wndFncIdAz = char(IFINFO.wndFncIdAz.');
    IFINFO.wndFncIdAz(find(IFINFO.wndFncIdAz==0,1,'first'):end)=0;
    IFINFO.wndFncIdRng = char(IFINFO.wndFncIdRng.');
    IFINFO.wndFncIdRng(find(IFINFO.wndFncIdRng==0,1,'first'):end)=0;
    IFINFO.cmtText = char(IFINFO.cmtText.');
    IFINFO.cmtText(find(IFINFO.cmtText==0,1,'first'):end)=0;
end

function IFINFO = marshal_IFINFO_3_0(IFINFO) %#ok<DEFNU>
    IFINFO.imgFileName = marshal_string(IFINFO.imgFileName, 128);
    IFINFO.ifAlgo = marshal_string(IFINFO.ifAlgo, 8);
    IFINFO.wndFncIdAz = marshal_string(IFINFO.wndFncIdAz, 48);
    IFINFO.wndFncIdRng = marshal_string(IFINFO.wndFncIdRng, 48);
    IFINFO.cmtText = marshal_string(IFINFO.cmtText, 166);
end

function write_IFINFO_3_0(fid, IFINFO) %#ok<DEFNU>
    fwrite(fid, IFINFO.procProduct,                           'uint32');
    fwrite(fid, IFINFO.imgFileNameLen,                        'uint16');
    fwrite(fid, IFINFO.imgFileName,                           'uint8');
    fwrite(fid, IFINFO.azResolution,                          'single');
    fwrite(fid, IFINFO.rngResolution,                         'single');
    fwrite(fid, IFINFO.imgCalParam,                           'single');
    fwrite(fid, IFINFO.sigmaN,                                'single');
    fwrite(fid, IFINFO.sampLocDCRow,                          'int32');
    fwrite(fid, IFINFO.sampLocDCCol,                          'int32');
    fwrite(fid, IFINFO.ifAlgo,                                'uint8');
    fwrite(fid, IFINFO.imgFlag,                               'int32');
    fwrite(fid, IFINFO.azCoeff,                               'single');
    fwrite(fid, IFINFO.elCoeff,                               'single');
    fwrite(fid, IFINFO.azGeoCorrect,                          'int32');
    fwrite(fid, IFINFO.rngGeoCorrect,                         'int32');
    fwrite(fid, IFINFO.wndBwFactAz,                           'single');
    fwrite(fid, IFINFO.wndBwFactRng,                          'single');
    fwrite(fid, IFINFO.wndFncIdAz,                            'uint8');
    fwrite(fid, IFINFO.wndFncIdRng,                           'uint8');
    fwrite(fid, IFINFO.cmtLen,                                'uint16');
    fwrite(fid, IFINFO.cmtText,                               'uint8');
    fwrite(fid, IFINFO.autoFocusInfo,                         'int32');
    fwrite(fid, IFINFO.rngFFTSize,                            'int32');
    fwrite(fid, IFINFO.RangePaneFilterCoeff,                  'single');
    fwrite(fid, IFINFO.AzPreFilterCoeff,                      'single');
    fwrite(fid, IFINFO.afPeakQuadComp,                        'single');
end

function IFINFO = init_IFINFO_3_0() %#ok<DEFNU>
    IFINFO.procProduct                        = zeros(1,1,'uint32');          
    IFINFO.imgFileNameLen                     = zeros(1,1,'uint16');          
    IFINFO.imgFileName                        = char(zeros(1,128));           
    IFINFO.azResolution                       = zeros(1,1,'single');          
    IFINFO.rngResolution                      = zeros(1,1,'single');          
    IFINFO.imgCalParam                        = zeros(1,1,'single');          
    IFINFO.sigmaN                             = zeros(1,1,'single');          
    IFINFO.sampLocDCRow                       = zeros(1,1,'int32');           
    IFINFO.sampLocDCCol                       = zeros(1,1,'int32');           
    IFINFO.ifAlgo                             = char(zeros(1,8));             
    IFINFO.imgFlag                            = zeros(1,1,'int32');           
    IFINFO.azCoeff                            = zeros(1,6,'single');          
    IFINFO.elCoeff                            = zeros(1,9,'single');          
    IFINFO.azGeoCorrect                       = zeros(1,1,'int32');           
    IFINFO.rngGeoCorrect                      = zeros(1,1,'int32');           
    IFINFO.wndBwFactAz                        = zeros(1,1,'single');          
    IFINFO.wndBwFactRng                       = zeros(1,1,'single');          
    IFINFO.wndFncIdAz                         = char(zeros(1,48));            
    IFINFO.wndFncIdRng                        = char(zeros(1,48));            
    IFINFO.cmtLen                             = zeros(1,1,'uint16');          
    IFINFO.cmtText                            = char(zeros(1,166));           
    IFINFO.autoFocusInfo                      = zeros(1,1,'int32');           
    IFINFO.rngFFTSize                         = zeros(1,1,'int32');           
    IFINFO.RangePaneFilterCoeff               = zeros(1,11,'single');         
    IFINFO.AzPreFilterCoeff                   = zeros(1,5,'single');          
    IFINFO.afPeakQuadComp                     = zeros(1,1,'single');          
    IFINFO.ver_maj                            = 3;                            
    IFINFO.ver_min                            = 0;                            
end

function MOMEASINFO = read_MOMEASINFO_1_0(buffer) %#ok<DEFNU>
    MOMEASINFO.posUncertDown                  = typecast(buffer(1:4),'single');
    MOMEASINFO.posUncertE                     = typecast(buffer(5:8),'single');
    MOMEASINFO.posUncertN                     = typecast(buffer(9:12),'single');
    MOMEASINFO.navAidingType                  = typecast(buffer(13:16),'int32');
    MOMEASINFO.ver_maj                        = 1;                            
    MOMEASINFO.ver_min                        = 0;                            
end

function MOMEASINFO = unmarshal_MOMEASINFO_1_0(MOMEASINFO) %#ok<DEFNU>
end

function MOMEASINFO = marshal_MOMEASINFO_1_0(MOMEASINFO) %#ok<DEFNU>
end

function write_MOMEASINFO_1_0(fid, MOMEASINFO) %#ok<DEFNU>
    fwrite(fid, MOMEASINFO.posUncertDown,                     'single');
    fwrite(fid, MOMEASINFO.posUncertE,                        'single');
    fwrite(fid, MOMEASINFO.posUncertN,                        'single');
    fwrite(fid, MOMEASINFO.navAidingType,                     'int32');
end

function MOMEASINFO = init_MOMEASINFO_1_0() %#ok<DEFNU>
    MOMEASINFO.posUncertDown                  = zeros(1,1,'single');          
    MOMEASINFO.posUncertE                     = zeros(1,1,'single');          
    MOMEASINFO.posUncertN                     = zeros(1,1,'single');          
    MOMEASINFO.navAidingType                  = zeros(1,1,'int32');           
    MOMEASINFO.ver_maj                        = 1;                            
    MOMEASINFO.ver_min                        = 0;                            
end

function MOMEASINFO = read_MOMEASINFO_2_0(buffer) %#ok<DEFNU>
    MOMEASINFO.posUncertDown                  = typecast(buffer(1:4),'single');
    MOMEASINFO.posUncertE                     = typecast(buffer(5:8),'single');
    MOMEASINFO.posUncertN                     = typecast(buffer(9:12),'single');
    MOMEASINFO.navAidingType                  = typecast(buffer(13:16),'int32');
    MOMEASINFO.gpsReceiverUsed                = typecast(buffer(17:20),'int32');
    MOMEASINFO.receiverKeyed                  = typecast(buffer(21:32),'int32');
    MOMEASINFO.differentialCorrection         = typecast(buffer(33:44),'int32');
    MOMEASINFO.P1_std                         = typecast(buffer(45:52),'double');
    MOMEASINFO.P2_std                         = typecast(buffer(53:60),'double');
    MOMEASINFO.P3_std                         = typecast(buffer(61:68),'double');
    MOMEASINFO.V1_std                         = typecast(buffer(69:76),'double');
    MOMEASINFO.V2_std                         = typecast(buffer(77:84),'double');
    MOMEASINFO.V3_std                         = typecast(buffer(85:92),'double');
    MOMEASINFO.ver_maj                        = 2;                            
    MOMEASINFO.ver_min                        = 0;                            
end

function MOMEASINFO = unmarshal_MOMEASINFO_2_0(MOMEASINFO) %#ok<DEFNU>
end

function MOMEASINFO = marshal_MOMEASINFO_2_0(MOMEASINFO) %#ok<DEFNU>
end

function write_MOMEASINFO_2_0(fid, MOMEASINFO) %#ok<DEFNU>
    fwrite(fid, MOMEASINFO.posUncertDown,                     'single');
    fwrite(fid, MOMEASINFO.posUncertE,                        'single');
    fwrite(fid, MOMEASINFO.posUncertN,                        'single');
    fwrite(fid, MOMEASINFO.navAidingType,                     'int32');
    fwrite(fid, MOMEASINFO.gpsReceiverUsed,                   'int32');
    fwrite(fid, MOMEASINFO.receiverKeyed,                     'int32');
    fwrite(fid, MOMEASINFO.differentialCorrection,            'int32');
    fwrite(fid, MOMEASINFO.P1_std,                            'double');
    fwrite(fid, MOMEASINFO.P2_std,                            'double');
    fwrite(fid, MOMEASINFO.P3_std,                            'double');
    fwrite(fid, MOMEASINFO.V1_std,                            'double');
    fwrite(fid, MOMEASINFO.V2_std,                            'double');
    fwrite(fid, MOMEASINFO.V3_std,                            'double');
end

function MOMEASINFO = init_MOMEASINFO_2_0() %#ok<DEFNU>
    MOMEASINFO.posUncertDown                  = zeros(1,1,'single');          
    MOMEASINFO.posUncertE                     = zeros(1,1,'single');          
    MOMEASINFO.posUncertN                     = zeros(1,1,'single');          
    MOMEASINFO.navAidingType                  = zeros(1,1,'int32');           
    MOMEASINFO.gpsReceiverUsed                = zeros(1,1,'int32');           
    MOMEASINFO.receiverKeyed                  = zeros(1,3,'int32');           
    MOMEASINFO.differentialCorrection         = zeros(1,3,'int32');           
    MOMEASINFO.P1_std                         = zeros(1,1,'double');          
    MOMEASINFO.P2_std                         = zeros(1,1,'double');          
    MOMEASINFO.P3_std                         = zeros(1,1,'double');          
    MOMEASINFO.V1_std                         = zeros(1,1,'double');          
    MOMEASINFO.V2_std                         = zeros(1,1,'double');          
    MOMEASINFO.V3_std                         = zeros(1,1,'double');          
    MOMEASINFO.ver_maj                        = 2;                            
    MOMEASINFO.ver_min                        = 0;                            
end

function MULTILOOKINFO = read_MULTILOOKINFO_1_0(buffer) %#ok<DEFNU>
    MULTILOOKINFO.method                      = typecast(buffer(1:4),'uint32');
    MULTILOOKINFO.numberOfImages              = typecast(buffer(5:8),'uint32');
    MULTILOOKINFO.APB0                        = typecast(buffer(9:208),'single');
    MULTILOOKINFO.ver_maj                     = 1;                            
    MULTILOOKINFO.ver_min                     = 0;                            
end

function MULTILOOKINFO = unmarshal_MULTILOOKINFO_1_0(MULTILOOKINFO) %#ok<DEFNU>
end

function MULTILOOKINFO = marshal_MULTILOOKINFO_1_0(MULTILOOKINFO) %#ok<DEFNU>
end

function write_MULTILOOKINFO_1_0(fid, MULTILOOKINFO) %#ok<DEFNU>
    fwrite(fid, MULTILOOKINFO.method,                         'uint32');
    fwrite(fid, MULTILOOKINFO.numberOfImages,                 'uint32');
    fwrite(fid, MULTILOOKINFO.APB0,                           'single');
end

function MULTILOOKINFO = init_MULTILOOKINFO_1_0() %#ok<DEFNU>
    MULTILOOKINFO.method                      = zeros(1,1,'uint32');          
    MULTILOOKINFO.numberOfImages              = zeros(1,1,'uint32');          
    MULTILOOKINFO.APB0                        = zeros(1,50,'single');         
    MULTILOOKINFO.ver_maj                     = 1;                            
    MULTILOOKINFO.ver_min                     = 0;                            
end

function RADARINFO = read_RADARINFO_1_0(buffer) %#ok<DEFNU>
    RADARINFO.phDataRecorded                  = typecast(buffer(1:4),'uint32');
    RADARINFO.tapeBlockLogAddr                = typecast(buffer(5:8),'uint32');
    RADARINFO.rxatten                         = typecast(buffer(9:12),'single');
    RADARINFO.rx_gain                         = typecast(buffer(13:16),'single');
    RADARINFO.txatten                         = typecast(buffer(17:20),'single');
    RADARINFO.tx_power                        = typecast(buffer(21:24),'single');
    RADARINFO.tx_power_source                 = typecast(buffer(25:28),'int32');
    RADARINFO.sugg_tx_pwr                     = typecast(buffer(29:32),'single');
    RADARINFO.velDown                         = typecast(buffer(33:36),'single');
    RADARINFO.velEast                         = typecast(buffer(37:40),'single');
    RADARINFO.velNorth                        = typecast(buffer(41:44),'single');
    RADARINFO.passNumber                      = typecast(buffer(45:48),'int32');
    RADARINFO.imageNumber                     = typecast(buffer(49:52),'int32');
    RADARINFO.ver_maj                         = 1;                            
    RADARINFO.ver_min                         = 0;                            
end

function RADARINFO = unmarshal_RADARINFO_1_0(RADARINFO) %#ok<DEFNU>
end

function RADARINFO = marshal_RADARINFO_1_0(RADARINFO) %#ok<DEFNU>
end

function write_RADARINFO_1_0(fid, RADARINFO) %#ok<DEFNU>
    fwrite(fid, RADARINFO.phDataRecorded,                     'uint32');
    fwrite(fid, RADARINFO.tapeBlockLogAddr,                   'uint32');
    fwrite(fid, RADARINFO.rxatten,                            'single');
    fwrite(fid, RADARINFO.rx_gain,                            'single');
    fwrite(fid, RADARINFO.txatten,                            'single');
    fwrite(fid, RADARINFO.tx_power,                           'single');
    fwrite(fid, RADARINFO.tx_power_source,                    'int32');
    fwrite(fid, RADARINFO.sugg_tx_pwr,                        'single');
    fwrite(fid, RADARINFO.velDown,                            'single');
    fwrite(fid, RADARINFO.velEast,                            'single');
    fwrite(fid, RADARINFO.velNorth,                           'single');
    fwrite(fid, RADARINFO.passNumber,                         'int32');
    fwrite(fid, RADARINFO.imageNumber,                        'int32');
end

function RADARINFO = init_RADARINFO_1_0() %#ok<DEFNU>
    RADARINFO.phDataRecorded                  = zeros(1,1,'uint32');          
    RADARINFO.tapeBlockLogAddr                = zeros(1,1,'uint32');          
    RADARINFO.rxatten                         = zeros(1,1,'single');          
    RADARINFO.rx_gain                         = zeros(1,1,'single');          
    RADARINFO.txatten                         = zeros(1,1,'single');          
    RADARINFO.tx_power                        = zeros(1,1,'single');          
    RADARINFO.tx_power_source                 = zeros(1,1,'int32');           
    RADARINFO.sugg_tx_pwr                     = zeros(1,1,'single');          
    RADARINFO.velDown                         = zeros(1,1,'single');          
    RADARINFO.velEast                         = zeros(1,1,'single');          
    RADARINFO.velNorth                        = zeros(1,1,'single');          
    RADARINFO.passNumber                      = zeros(1,1,'int32');           
    RADARINFO.imageNumber                     = zeros(1,1,'int32');           
    RADARINFO.ver_maj                         = 1;                            
    RADARINFO.ver_min                         = 0;                            
end

function RADARINFO = read_RADARINFO_2_0(buffer) %#ok<DEFNU>
    RADARINFO.phDataRecorded                  = typecast(buffer(1:4),'uint32');
    RADARINFO.tapeBlockLogAddr                = typecast(buffer(5:8),'uint32');
    RADARINFO.rxatten                         = typecast(buffer(9:12),'single');
    RADARINFO.rx_gain                         = typecast(buffer(13:16),'single');
    RADARINFO.txatten                         = typecast(buffer(17:20),'single');
    RADARINFO.tx_power                        = typecast(buffer(21:24),'single');
    RADARINFO.tx_power_source                 = typecast(buffer(25:28),'int32');
    RADARINFO.sugg_tx_pwr                     = typecast(buffer(29:32),'single');
    RADARINFO.velDown                         = typecast(buffer(33:36),'single');
    RADARINFO.velEast                         = typecast(buffer(37:40),'single');
    RADARINFO.velNorth                        = typecast(buffer(41:44),'single');
    RADARINFO.passNumber                      = typecast(buffer(45:48),'int32');
    RADARINFO.imageNumber                     = typecast(buffer(49:52),'int32');
    RADARINFO.HPFMeanSource                   = typecast(buffer(53:56),'int32');
    RADARINFO.IChanMean                       = typecast(buffer(57:60),'single');
    RADARINFO.QChanMean                       = typecast(buffer(61:64),'single');
    RADARINFO.ver_maj                         = 2;                            
    RADARINFO.ver_min                         = 0;                            
end

function RADARINFO = unmarshal_RADARINFO_2_0(RADARINFO) %#ok<DEFNU>
end

function RADARINFO = marshal_RADARINFO_2_0(RADARINFO) %#ok<DEFNU>
end

function write_RADARINFO_2_0(fid, RADARINFO) %#ok<DEFNU>
    fwrite(fid, RADARINFO.phDataRecorded,                     'uint32');
    fwrite(fid, RADARINFO.tapeBlockLogAddr,                   'uint32');
    fwrite(fid, RADARINFO.rxatten,                            'single');
    fwrite(fid, RADARINFO.rx_gain,                            'single');
    fwrite(fid, RADARINFO.txatten,                            'single');
    fwrite(fid, RADARINFO.tx_power,                           'single');
    fwrite(fid, RADARINFO.tx_power_source,                    'int32');
    fwrite(fid, RADARINFO.sugg_tx_pwr,                        'single');
    fwrite(fid, RADARINFO.velDown,                            'single');
    fwrite(fid, RADARINFO.velEast,                            'single');
    fwrite(fid, RADARINFO.velNorth,                           'single');
    fwrite(fid, RADARINFO.passNumber,                         'int32');
    fwrite(fid, RADARINFO.imageNumber,                        'int32');
    fwrite(fid, RADARINFO.HPFMeanSource,                      'int32');
    fwrite(fid, RADARINFO.IChanMean,                          'single');
    fwrite(fid, RADARINFO.QChanMean,                          'single');
end

function RADARINFO = init_RADARINFO_2_0() %#ok<DEFNU>
    RADARINFO.phDataRecorded                  = zeros(1,1,'uint32');          
    RADARINFO.tapeBlockLogAddr                = zeros(1,1,'uint32');          
    RADARINFO.rxatten                         = zeros(1,1,'single');          
    RADARINFO.rx_gain                         = zeros(1,1,'single');          
    RADARINFO.txatten                         = zeros(1,1,'single');          
    RADARINFO.tx_power                        = zeros(1,1,'single');          
    RADARINFO.tx_power_source                 = zeros(1,1,'int32');           
    RADARINFO.sugg_tx_pwr                     = zeros(1,1,'single');          
    RADARINFO.velDown                         = zeros(1,1,'single');          
    RADARINFO.velEast                         = zeros(1,1,'single');          
    RADARINFO.velNorth                        = zeros(1,1,'single');          
    RADARINFO.passNumber                      = zeros(1,1,'int32');           
    RADARINFO.imageNumber                     = zeros(1,1,'int32');           
    RADARINFO.HPFMeanSource                   = zeros(1,1,'int32');           
    RADARINFO.IChanMean                       = zeros(1,1,'single');          
    RADARINFO.QChanMean                       = zeros(1,1,'single');          
    RADARINFO.ver_maj                         = 2;                            
    RADARINFO.ver_min                         = 0;                            
end

function SMDINFO = read_SMDINFO_2_0(buffer) %#ok<DEFNU>
    SMDINFO.smdBoxRotAngle                    = typecast(buffer(1:4),'single');
    SMDINFO.eVarLenCount                      = typecast(buffer(5:8),'uint32');
    SMDINFO.eVarLenType.eVarLenType           = typecast(buffer(9:12),'uint32');
    SMDINFO.eVarLenType.numElements           = typecast(buffer(13:16),'uint32');
    SMDINFO.eVarLenType.elementSize           = typecast(buffer(17:20),'uint32');
    SMDINFO.eVarLenType.ver_maj               = 2;                            
    SMDINFO.eVarLenType.ver_min               = 0;                            
    SMDINFO.tlCoordX                          = [];
    SMDINFO.tlCoordY                          = [];
    SMDINFO.brCoordX                          = [];
    SMDINFO.brCoordY                          = [];
    SMDINFO.quality                           = [];
    for ii=1:SMDINFO.eVarLenType.numElements
        SMDINFO.tlCoordX(ii)                  = typecast(buffer(21:24),'uint32');
        SMDINFO.tlCoordY(ii)                  = typecast(buffer(25:28),'uint32');
        SMDINFO.brCoordX(ii)                  = typecast(buffer(29:32),'uint32');
        SMDINFO.brCoordY(ii)                  = typecast(buffer(33:36),'uint32');
        SMDINFO.quality(ii)                   = typecast(buffer(37:37),'uint8');
        buffer=buffer(SMDINFO.eVarLenType.elementSize+1:end);
    end
    SMDINFO.ver_maj                           = 2;                            
    SMDINFO.ver_min                           = 0;                            
end

function SMDINFO = unmarshal_SMDINFO_2_0(SMDINFO) %#ok<DEFNU>
end

function SMDINFO = marshal_SMDINFO_2_0(SMDINFO) %#ok<DEFNU>
end

function write_SMDINFO_2_0(fid, SMDINFO) %#ok<DEFNU>
    fwrite(fid, SMDINFO.smdBoxRotAngle,                       'single');
    fwrite(fid, SMDINFO.eVarLenCount,                         'uint32');
    fwrite(fid, SMDINFO.eVarLenType.eVarLenType,              'uint32');
    fwrite(fid, SMDINFO.eVarLenType.numElements,              'uint32');
    fwrite(fid, SMDINFO.eVarLenType.elementSize,              'uint32');
    for ii=1:SMDINFO.eVarLenType.numElements
        fwrite(fid, SMDINFO.tlCoordX(ii),                         'uint32');
        fwrite(fid, SMDINFO.tlCoordY(ii),                         'uint32');
        fwrite(fid, SMDINFO.brCoordX(ii),                         'uint32');
        fwrite(fid, SMDINFO.brCoordY(ii),                         'uint32');
        fwrite(fid, SMDINFO.quality(ii),                          'uint8');
    end
end

function SMDINFO = init_SMDINFO_2_0() %#ok<DEFNU>
    SMDINFO.smdBoxRotAngle                    = zeros(1,1,'single');          
    SMDINFO.eVarLenCount                      = zeros(1,1,'uint32');          
    SMDINFO.eVarLenType.eVarLenType           = zeros(1,1,'uint32');          
    SMDINFO.eVarLenType.numElements           = zeros(1,1,'uint32');          
    SMDINFO.eVarLenType.elementSize           = zeros(1,1,'uint32');          
    SMDINFO.eVarLenType.ver_maj               = 2;                            
    SMDINFO.eVarLenType.ver_min               = 0;                            
    SMDINFO.tlCoordX                          = zeros(1,1,'uint32');          
    SMDINFO.tlCoordY                          = zeros(1,1,'uint32');          
    SMDINFO.brCoordX                          = zeros(1,1,'uint32');          
    SMDINFO.brCoordY                          = zeros(1,1,'uint32');          
    SMDINFO.quality                           = zeros(1,1,'uint8');           
    SMDINFO.ver_maj                           = 2;                            
    SMDINFO.ver_min                           = 0;                            
end

function SMDINFO = read_SMDINFO_3_0(buffer) %#ok<DEFNU>
    SMDINFO.smdBoxRotAngle                    = typecast(buffer(1:4),'single');
    SMDINFO.totalSmdDetections                = typecast(buffer(5:8),'uint32');
    SMDINFO.eVarLenCount                      = typecast(buffer(9:12),'uint32');
    SMDINFO.eVarLenType.eVarLenType           = typecast(buffer(13:16),'uint32');
    SMDINFO.eVarLenType.numElements           = typecast(buffer(17:20),'uint32');
    SMDINFO.eVarLenType.elementSize           = typecast(buffer(21:24),'uint32');
    SMDINFO.eVarLenType.ver_maj               = 3;                            
    SMDINFO.eVarLenType.ver_min               = 0;                            
    SMDINFO.tlCoordX                          = [];
    SMDINFO.tlCoordY                          = [];
    SMDINFO.brCoordX                          = [];
    SMDINFO.brCoordY                          = [];
    SMDINFO.quality                           = [];
    for ii=1:SMDINFO.eVarLenType.numElements
        SMDINFO.tlCoordX(ii)                  = typecast(buffer(25:28),'single');
        SMDINFO.tlCoordY(ii)                  = typecast(buffer(29:32),'single');
        SMDINFO.brCoordX(ii)                  = typecast(buffer(33:36),'single');
        SMDINFO.brCoordY(ii)                  = typecast(buffer(37:40),'single');
        SMDINFO.quality(ii)                   = typecast(buffer(41:41),'uint8');
        buffer=buffer(SMDINFO.eVarLenType.elementSize+1:end);
    end
    SMDINFO.ver_maj                           = 3;                            
    SMDINFO.ver_min                           = 0;                            
end

function SMDINFO = unmarshal_SMDINFO_3_0(SMDINFO) %#ok<DEFNU>
    SMDINFO.quality = char(SMDINFO.quality.');
    SMDINFO.quality(find(SMDINFO.quality==0,1,'first'):end)=0;
end

function SMDINFO = marshal_SMDINFO_3_0(SMDINFO) %#ok<DEFNU>
    SMDINFO.quality = marshal_string(SMDINFO.quality, 1);
end

function write_SMDINFO_3_0(fid, SMDINFO) %#ok<DEFNU>
    fwrite(fid, SMDINFO.smdBoxRotAngle,                       'single');
    fwrite(fid, SMDINFO.totalSmdDetections,                   'uint32');
    fwrite(fid, SMDINFO.eVarLenCount,                         'uint32');
    fwrite(fid, SMDINFO.eVarLenType.eVarLenType,              'uint32');
    fwrite(fid, SMDINFO.eVarLenType.numElements,              'uint32');
    fwrite(fid, SMDINFO.eVarLenType.elementSize,              'uint32');
    for ii=1:SMDINFO.eVarLenType.numElements
        fwrite(fid, SMDINFO.tlCoordX(ii),                         'single');
        fwrite(fid, SMDINFO.tlCoordY(ii),                         'single');
        fwrite(fid, SMDINFO.brCoordX(ii),                         'single');
        fwrite(fid, SMDINFO.brCoordY(ii),                         'single');
        fwrite(fid, SMDINFO.quality(ii),                          'uint8');
    end
end

function SMDINFO = init_SMDINFO_3_0() %#ok<DEFNU>
    SMDINFO.smdBoxRotAngle                    = zeros(1,1,'single');          
    SMDINFO.totalSmdDetections                = zeros(1,1,'uint32');          
    SMDINFO.eVarLenCount                      = zeros(1,1,'uint32');          
    SMDINFO.eVarLenType.eVarLenType           = zeros(1,1,'uint32');          
    SMDINFO.eVarLenType.numElements           = zeros(1,1,'uint32');          
    SMDINFO.eVarLenType.elementSize           = zeros(1,1,'uint32');          
    SMDINFO.eVarLenType.ver_maj               = 3;                            
    SMDINFO.eVarLenType.ver_min               = 0;                            
    SMDINFO.tlCoordX                          = zeros(1,1,'single');          
    SMDINFO.tlCoordY                          = zeros(1,1,'single');          
    SMDINFO.brCoordX                          = zeros(1,1,'single');          
    SMDINFO.brCoordY                          = zeros(1,1,'single');          
    SMDINFO.quality                           = char(zeros(1,1));             
    SMDINFO.ver_maj                           = 3;                            
    SMDINFO.ver_min                           = 0;                            
end


%</%gfflib%>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%